!=================================================================================================================================
! MODULE       : stomate_growth_fun_all
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
!                This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         Plant growth and C-allocation among the biomass components (leaves, wood, roots, fruit, reserves, labile) 
!!               is calculated making use of functional allocation which combines the pipe model and allometric relationships 
!!               proposed by Sitch et al 2003 and adjusted by Zaehle et al 2010.
!!
!!\n DESCRIPTION: This module calculates three processes: (1) daily maintenance respiration based on the half-hourly
!!               respiration calculated in stomate_resp.f90, (2) the absolute allocation to the different biomass
!!               components based on functional allocation approach and (3) the allocatable biomass as the residual
!!               of GPP-Ra. Multiplication of the allocation fractions and allocatable biomass given the changes in
!!               biomass pools.
!!
!! RECENT CHANGE(S): Until 1.9.6 only one allocation scheme was available (now contained in stomate_grwoth_res_lim.f90). 
!!               This module consists of an alternative formalization of plant growth.
!!                             
!! REFERENCE(S)	: - Sitch, S., Smith, B., Prentice, I.C., Arneth, A., Bondeau, A., Cramer, W.,
!!               Kaplan, J.O., Levis, S., Lucht, W., Sykes, M.T., Thonicke, K., Venevsky, S. (2003), Evaluation of 
!!               ecosystem dynamics, plant geography and terrestrial carbon cycling in the LPJ Dynamic Global Vegetation 
!!               Model, Global Change Biology, 9, 161-185.\n 
!!               - Zaehle, S. and Friend, A.D. (2010), Carbon and nitrogen cycle dynamics in the O-CN land surface model: 1. 
!!               Model description, site-scale evaluation, and sensitivity to parameter estimates, Global Biogeochemical 
!!               Cycles, 24, GB1005.\n
!!               - Magnani F., Mencuccini M. & Grace J. 2000. Age-related decline in stand productivity: the role of 
!!               structural acclimation under hydraulic constraints Plant, Cell and Environment 23, 251–263.\n
!!               - Bloom A.J., Chapin F.S. & Mooney H.A. (1985) Resource limitation in plants. An economic analogy.  
!!               Annual Review Ecology Systematics 16, 363–392.\n
!!               - Case K.E. & Fair R.C. (1989) Principles of Economics. Prentice Hall, London.\n
!!               - McDowell, N., Barnard, H., Bond, B.J., Hinckley, T., Hubbard, R.M., Ishii, H., Köstner, B., 
!!               Magnani, F. Marshall, J.D., Meinzer, F.C., Phillips, N., Ryan, M.G., Whitehead D. 2002. The 
!!               relationship between tree height and leaf area: sapwood area ratio. Oecologia, 132:12–20.\n
!!               - Novick, K., Oren, R., Stoy, P., Juang, F.-Y., Siqueira, M., Katul, G. 2009. The relationship between
!!               reference canopy conductance and simplified hydraulic architecture. Advances in water resources 32,
!!               809-819.     
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ===============================================================================================================================
MODULE stomate_growth_fun_all

  ! Modules used:
  USE ioipsl_para
  USE pft_parameters
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE function_library,    ONLY: wood_to_qmdia, wood_to_qmheight, wood_to_ba_eff, biomass_to_lai,&
                                 calculate_c0_alloc, wood_to_volume, wood_to_ba

  IMPLICIT NONE

  ! Private & public routines

  PRIVATE
  PUBLIC growth_fun_all_clear, growth_fun_all

 ! Variables shared by all subroutines in this module

  LOGICAL, SAVE                                             :: firstcall = .TRUE.  !! Is this the first call? (true/false)
 
!!$  !+++TEMP+++
  INTEGER, SAVE                                             :: istep = 0

CONTAINS



!! ================================================================================================================================
!! SUBROUTINE   : growth_fun_all_clear
!!
!>\BRIEF          Set the flag ::firstcall to .TRUE. and as such activate section 
!! 1.1 of the subroutine alloc (see below).\n
!!
!_ ================================================================================================================================

  SUBROUTINE growth_fun_all_clear
    firstcall = .TRUE.
  END SUBROUTINE growth_fun_all_clear



!! ================================================================================================================================
!! SUBROUTINE 	: growth_fun_all
!!
!>\BRIEF          Allocate net primary production (= photosynthesis
!!                minus autothrophic respiration) to: labile carbon pool carbon reserves, aboveground sapwood,
!!                belowground sapwood, root, fruits and leaves following the pipe model and allometric constraints.
!!
!! DESCRIPTION  : Total maintenance respiration for the whole plant is calculated by summing maintenance 
!!                respiration of the different plant compartments. Maintenance respiration is subtracted 
!!                from whole-plant allocatable biomass (up to a maximum fraction of the total allocatable biomass). 
!!                Growth respiration is then calculated as a prescribed (0.75) fraction of the allocatable
!!                biomass. Subsequently NPP is calculated by substracting total autotrophic  respiration from GPP 
!!                i.e. NPP = GPP - maintenance resp - growth resp.
!!
!!                The pipe model assumes that one unit of leaf mass requires a proportional amount of sapwood to 
!!                transport water from the roots to the leaves. Also a proportional fraction of roots is needed to 
!!                take up the water from the soil. The proportional amounts between leaves, sapwood and roots are 
!!                given by allocation factors. These allocation factors are PFT specific and depend on a parameter 
!!                quantifying the leaf to sapwood area (::k_latosa_target), the specific laeaf area (::sla), wood 
!!                density (::pipe_density) and a scaling parameter between leaf and root mass.
!! 
!!                Lai is optimised for mean annual radiation use efficiency and the C cost for producing the 
!!                canopy. The cost-benefit ratio is optimised when the marginal gain / marginal cost = 1 lai target 
!!                is used to calculate whether the reserves are used. This approach allows plants to get out of 
!!                senescence and to start developping a canopy in early spring. 
!!                  
!!                As soon as a canopy has emerged, C (b_inc_tot) becomes available at the stand level through 
!!                photosynthesis and, C is allocated at the tree level (b_inc) following both the pipe model and 
!!                allometric constraints. Mass conservation requires:
!!                (1) Cs_inc + Cr_inc + Cl_inc = b_inc
!!                (2) sum(b_inc) = b_inc_tot
!!
!!                Wood allocation depends on tree basal area following the rule of Deleuze & Dhote
!!                delta_ba = gammas*(circ - m*sigmas + sqrt((m*sigmas + circ).^2 - (4*sigmas*circ)))/2
!!                (3) <=> delta_ba = circ_class_dba*gammas
!!                Where circ_class_dba = (circ - m*sigmas + sqrt((m*sigmas + circ).^2 - (4*sigmas*circ)))/2
!! 
!!                Allometric relationships
!!                height = pipe_tune2*(dia.^pipe_tune3)
!!                Re-write this relationship as a function of ba 
!!                (4) height = pipe_tune2 * (4/pi*ba)^(pipe_tune3/2)
!!                (5a) Cl/Cs = KF/height for trees
!!                (5b) Cs = Cl / KF
!!                (6) Cl = Cr * LF
!!
!!                Use a linear approximation to avoid iterations. Given that allocation is calculated daily, a 
!!                local lineair assumption is fair. Eq (4) can thus be rewritten as:
!!                s = step/(pipe_tune2*(4/pi*(ba+step)).^(pipe_tune3/2)-pipe_tune2*(4/pi*ba).^(pipe_tune3/2))
!!                Where step is a small but realistic (for the time step) change in ba
!!                (7)  <=> delta_height = delta_ba/s
!!
!!                Calculate Cs_inc from allometric relationships
!!                Cs_inc = tree_ff*pipe_density*(ba+delta_ba)*(height+delta_height) - Cs - Ch         
!!                Cs_inc = tree_ff*pipe_density*(ba+delta_ba)*(height+delta_ba/s) - Cs - Ch
!!                (8)  <=> Cs_inc = tree_ff*pipe_density*(ba+a*gammas)*(height+(a/s*gammas)) - Cs - Ch
!!
!!                Rewrite (5) as 
!!                Cl_inc = KF*(Cs_inc+Cs)/(height+delta_height) - Cl
!!                Substitute (7) in (4) and solve for Cl_inc
!!                Cl_inc = KF*(tree_ff*pipe_density*(ba+circ_class_dba*gammas)*(height+(circ_class_dba/s*gammas)) - Ch)/ & 
!!                   (height+(circ_class_dba/s*gammas)) - Cl  
!!                (9)  <=> Cl_inc = KF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - &
!!                            (KF*Ch)/(height+(circ_class_dba/s*gammas)) - Cl
!!
!!                Rewrite (6) as 
!!                Cr_inc = (Cl_inc+Cl)/LF - Cr
!!                Substitute (9) in (6)
!!                (10)  <=> Cr_inc = KF/LF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - &
!!                            (KF*Ch/LF)/(height+(circ_class_dba/s*gammas)) - Cr
!!
!!                Depending on the specific case that needs to be solved equations (1) takes one of the following forms:
!!                (a) b_inc = Cl_inc + Cr_inc + Cs_inc, (b) b_inc = Cl_inc + Cr_inc, (c) b_inc = Cl_inc + Cs_inc or 
!!                (d) b_inc = Cr_inc + Cs_inc. One of these alternative forms of eq. 1 are then combined with 
!!                eqs 8, 9 and 10 and solved for gammas. The details for the solution of these four cases are given in the
!!                code. Once gammas is know, eqs 6 - 10 are used to calculate the allocation to leaves (Cl_inc), 
!!                roots (Cr_inc) and sapwood (Cs_inc).
!!
!!                Because of turnover, biomass pools are not all the time in balance according to rules prescribed
!!                by the pipe model. To test whether biomass pools are balanced, the target biomasses are calculated 
!!                and balance is restored whenever needed up to the level that the biomass pools for leaves, sapwood
!!                and roots are balanced according to the pipe model. Once the balance is restored C is allocated to
!!                fruits, leaves, sapwood and roots by making use of the pipe model (below this called ordinary 
!!                allocation).
!!
!!                Although strictly speaking allocation factors are not necessary in this scheme (Cl_inc could simply 
!!                be added to biomass(:,:,ileaf,icarbon), Cr_inc to biomass(:,:,iroot,icarbon), etc.), they are 
!!                nevertheless calculated because using allocation factors facilitates comparison to the resource 
!!                limited allocation scheme (stomate_growth_res_lim.f90) and it comes in handy for model-data comparison.
!!
!!                Effective basal area, height and circumferences are use in the allocation scheme because their 
!!                calculations make use of the total (above and belowground) biomass. In forestry the same measures
!!                exist (and they are also calculated in ORCHIDEE) but only account for the aboverground biomass.
!! 
!!
!! RECENT CHANGE(S): - The code by Sonke Zaehle made use of ::Cl_target that was derived from ::lai_target which in turn 
!!                was a function of ::rue_longterm. Cl_target was then used as a threshold value to decide whether there 
!!                was only phenological growth (just leaves and roots) or whether there was full allometric growth to the 
!!                leaves, roots and sapwood. This approach was inconsistent with the pipe model because full allometric 
!!                growth can only occur if all three biomass pools are in balance. ::lai_target is no longer used as a 
!!                criterion to switch between phenological and full allometric growth. Its use is now restricted to trigger
!!                the use of reserves in spring. 
!!
!! MAIN OUTPUT VARIABLE(S): ::npp and :: biomass. Seven different biomass compartments (leaves, roots, above and 
!!                belowground wood, carbohydrate reserves, labile and fruits).
!!
!! REFERENCE(S)	:- Sitch, S., Smith, B., Prentice, I.C., Arneth, A., Bondeau, A., Cramer, W.,
!!                Kaplan, J.O., Levis, S., Lucht, W., Sykes, M.T., Thonicke, K., Venevsky, S. (2003), Evaluation of 
!!                ecosystem dynamics, plant geography and terrestrial carbon cycling in the LPJ Dynamic Global Vegetation 
!!                Model, Global Change Biology, 9, 161-185.\n 
!!                - Zaehle, S. and Friend, A.D. (2010), Carbon and nitrogen cycle dynamics in the O-CN land surface model: 1. 
!!                Model description, site-scale evaluation, and sensitivity to parameter estimates, Global Biogeochemical 
!!                Cycles, 24, GB1005.\n
!!                - Magnani F., Mencuccini M. & Grace J. 2000. Age-related decline in stand productivity: the role of 
!!                structural acclimation under hydraulic constraints Plant, Cell and Environment 23, 251–263.
!!                - Bloom A.J., Chapin F.S. & Mooney H.A. (1985) Resource limitation in plants. An economic analogy. Annual 
!!                Review Ecology Systematics 16, 363–392.
!!                - Case K.E. & Fair R.C. (1989) Principles of Economics. Prentice Hall, London.
!!                - McDowell, N., Barnard, H., Bond, B.J., Hinckley, T., Hubbard, R.M., Ishii, H., Köstner, B., 
!!                Magnani, F. Marshall, J.D., Meinzer, F.C., Phillips, N., Ryan, M.G., Whitehead D. 2002. The 
!!                relationship between tree height and leaf area: sapwood area ratio. Oecologia, 132:12–20
!!                - Novick, K., Oren, R., Stoy, P., Juang, F.-Y., Siqueira, M., Katul, G. 2009. The relationship between
!!                reference canopy conductance and simplified hydraulic architecture. Advances in water resources 32,
!!                809-819.   
!!
!! +++++++++++++++++++++
!! MAKE A NEW FLOW CHART
!! +++++++++++++++++++++
!! FLOWCHART    : 
!!
!_ ================================================================================================================================

  SUBROUTINE growth_fun_all (npts, dt, veget_max, veget, PFTpresent, &
       senescence, when_growthinit, t2m, &
       gpp_daily, gpp_week, resp_maint_part, resp_maint, &
       resp_growth, npp, biomass, age, &
       leaf_age, leaf_frac, use_reserve, &
       lab_fac, ind, rue_longterm, circ_class_n, &
       circ_class_biomass, KF, bm_sync, sigma, &
       gammas, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
       k_latosa_adapt, forest_managed, circ_class_dist)       


 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                        :: npts                   !! Domain size - number of grid cells 
                                                                                !! (unitless)
    REAL(r_std), INTENT(in)                           :: dt                     !! Time step of the simulations for stomate
                                                                                !! (days)
    REAL(r_std), DIMENSION(:), INTENT(in)             :: t2m                    !! Temperature at 2 meter (K)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: ind                    !! Number of individuals at the stand level
                                                                                !! @tex $(m^{-2})$ @endtex     
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget_max              !! PFT "Maximal" coverage fraction of a PFT 
                                                                                !! (= ind*cn_ind) 
                                                                                !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget                  !! Fraction of vegetation type including 
                                                                                !! non-biological fraction (unitless)   
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: when_growthinit        !! Days since beginning of growing season 
                                                                                !! (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: gpp_week               !! "weekly" (default 7-day) gross primary 
                                                                                !! productivity 
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: rue_longterm           !! Longterm "radiation use efficicency"
                                                                                !! calculated as the ratio of GPP over 
                                                                                !! the fraction of radiation absorbed
                                                                                !! by the canopy 
                                                                                !! @tex $(gC.m^{-2}day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_root           !! Effective root turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_sap            !! Effective sapwood turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_leaf           !! Effective leaf turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: circ_class_n           !! Number of individuals in each circ class
    REAL(r_std), DIMENSION(:), INTENT(in)             :: circ_class_dist        !! The probability distribution of trees 
                                                                                !! in a circ class in case of a 
                                                                                !! redistribution (unitless).
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)         :: resp_maint_part        !! Maintenance respiration of different  
                                                                                !! plant parts
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    LOGICAL, DIMENSION(:,:), INTENT(in)               :: senescence             !! Is the PFT senescent?  - only for 
                                                                                !! deciduous trees (true/false)
    LOGICAL, DIMENSION(:,:), INTENT(in)               :: PFTpresent             !! PFT exists (true/false)
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)        :: forest_managed         !! Forest management flag: 0 = orchidee 
                                                                                !! standard, 1= self-thinning only, 2= 
                                                                                !! high-stand, 3= high-stand smoothed, 4= 
                                                                                !! coppices
    

   
    !! 0.2 Output variables

    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: resp_maint             !! PFT maintenance respiration 
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex    
    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: resp_growth            !! PFT growth respiration 
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: npp                    !! PFT net primary productivity 
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: lab_fac                !! Activity of labile pool factor (units??)
    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: sigma                  !! Threshold for indivudal tree growth in 
                                                                                !! the equation of Deleuze & Dhote (2004)(m).
                                                                                !! Trees whose circumference is smaller than 
                                                                                !! sigma don't grow much
    REAL(r_std), DIMENSION(:,:), INTENT(out)          :: gammas                 !! Slope for individual tree growth in the 
                                                                                !! equation of Deleuze & Dhote (2004) (m)
    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: gpp_daily              !! PFT gross primary productivity 
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: use_reserve            !! Flag to use the reserves to support
                                                                                !! phenological growth (0 or 1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: age                    !! PFT age (days)      
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)    :: biomass                !! PFT total biomass 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)      :: leaf_age               !! PFT age of different leaf classes 
                                                                                !! (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)      :: leaf_frac              !! PFT fraction of leaves in leaf age class
                                                                                !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)  :: circ_class_biomass     !! Biomass components of the model tree  
                                                                                !! within a circumference class
                                                                                !! class @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: KF                     !! Scaling factor to convert sapwood mass
                                                                                !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: k_latosa_adapt         !! Leaf to sapwood area adapted for long 
                                                                                !! term water stress (m)
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)       :: bm_sync                !! The difference betweeen the
                                                                                !! biomass in the circ_classes and
                                                                                !! the total biomass 
                                                                                !! @tex $(gC m^{-2})$ @endtex

!!$    !+++HACK+++
!!$    ! This is a hack to prescribe a stand structure to the model
!!$    ! this feature was introduced to parameterize and validate the 
!!$    ! energy budget. When using this hack comment out the declaration
!!$    ! of ::ind and ::circ_class_n above.
!!$    ! the "ind" will be prescribed from the run.def  
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: ind                    !! Number of individuals at the stand level
!!$                                                                                !! @tex $(m^{-2})$ @endtex  
!!$    ! (temp for hardwired values)  REAL(r_std), DIMENSION(:), INTENT(in)          :: circ_class_dist
!!$    REAL(r_std), DIMENSION(:), INTENT(inout)          :: circ_class_dist        !! The probability distribution of trees 
!!$                                                                                !! in a circ class in case of a 
!!$                                                                                !! redistribution (unitless).
!!$    ! (temp for hardwired values)  REAL(r_std), DIMENSION(:,:,:), INTENT(in)      :: circ_class_n 
!!$    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)      :: circ_class_n           !! Number of individuals in each circ class
!!$                                                                                !! @tex $(ind m^{-2})$ @endtex
!!$    !+++++++++++ 

    
    !! 0.4 Local variables

    CHARACTER(30)                                     :: var_name               !! To store variable names for I/O
    REAL(r_std), DIMENSION(npts,nvm)                  :: c0_alloc               !! Root to sapwood tradeoff parameter
    LOGICAL                                           :: grow_wood=.TRUE.       !! Flag to grow wood
    INTEGER(i_std)                                    :: ipts,j,k,l,m,icirc,imed!! Indeces(unitless)
    INTEGER(i_std)                                    :: ipar, iele, imbc       !! Indeces(unitless)
    INTEGER(i_std)                                    :: ilev                   !! Indeces(unitless)
    REAL(r_std)                                       :: frac                   !! No idea??
    REAL(r_std)                                       :: a,b,c                  !! Temporary variables to solve a
                                                                                !! quadratic equation (unitless)
    ! Stand level
    REAL(r_std)                                       :: gtemp                  !! Turnover coefficient of labile C pool
                                                                                !! (0-1??, units??)
    REAL(r_std)                                       :: reserve_pool           !! Intentional size of the reserve pool
                                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: labile_pool            !! Intentional size of the labile apool
                                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: reserve_scal           !! Protection of the reserve against 
                                                                                !! overuse (unitless)
    REAL(r_std)                                       :: use_lab                !! Availability of labile biomass 
                                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: use_res                !! Availability of resource biomass 
                                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: use_max                !! Maximum use of labile and resource pool 
                                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: leaf_meanage           !! Mean age of the leaves (days?)
    REAL(r_std)                                       :: reserve_time           !! Maximum number of days during which 
                                                                                !! carbohydrate reserve may be used (days)
    REAL(r_std)                                       :: b_inc_tot              !! Carbon that needs to allocated in the
                                                                                !! fixed number of trees (gC)
    REAL(r_std)                                       :: b_inc_temp             !! Temporary b_inc at the stand-level
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std)                                       :: scal                   !! Scaling factor between average 
                                                                                !! individual and individual plant
                                                                                !! @tex $(plant.m^{-2})$ @endtex
    REAL(r_std)                                       :: total_inc              !! Total biomass increase
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std)                                       :: KF_old                 !! Scaling factor to convert sapwood mass
                                                                                !! into leaf mass (m) at the previous
                                                                                !! time step
    REAL(r_std), DIMENSION(nvm)                       :: lai_happy              !! Lai threshold below which carbohydrate 
                                                                                !! reserve may be used 
                                                                                !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(nvm)                       :: deleuze_p              !! Percentile of trees that will receive 
                                                                                !! photosynthates. The proxy for intra stand
                                                                                !! competition. Depends on the management
                                                                                !! strategy when ncirc < 6
    REAL(r_std), DIMENSION(npts)                      :: tl                     !! Long term annual mean temperature (C)
    REAL(r_std), DIMENSION(npts)                      :: bm_add                 !! Biomass increase 
		                                                                !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts)                      :: bm_new                 !! New biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std)                                       :: alloc_sap_above        !! Fraction allocated to sapwood above 
                                                                                !! ground
    REAL(r_std), DIMENSION(npts,nvm)                  :: residual               !! Copy of b_inc_tot after all C has been
                                                                                !! allocated @tex $(gC.m^{-2})$ @endtex
                                                                                !! if all went well the value should be zero
    REAL(r_std), DIMENSION(npts,nvm)                  :: lai_target             !! Target LAI @tex $(m^{2}m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                  :: ltor                   !! Leaf to root ratio (unitless)   
    REAL(r_std), DIMENSION(npts,nvm)                  :: lstress_fac            !! Light stress factor, based on total
                                                                                !! transmitted light (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                  :: k_latosa               !! Target leaf to sapwood area ratio
    REAL(r_std), DIMENSION(npts,nvm)                  :: LF                     !! Scaling factor to convert sapwood mass
                                                                                !! into root mass (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                  :: lm_old                 !! Variable to store leaf biomass from 
                                                                                !! previous time step 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                  :: bm_alloc_tot           !! Allocatable biomass for the whole plant
                                                                                !! @tex $(gC.m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm)                  :: leaf_mass_young        !! Leaf biomass in youngest leaf age class 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                  :: lai                    !! PFT leaf area index 
                                                                                !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                  :: qm_dia                 !! Quadratic mean diameter of the stand (m)
    REAL(r_std), DIMENSION(npts,nvm)                  :: qm_height              !! Height of a tree with the quadratic mean
                                                                                !! diameter (m)
    REAL(r_std), DIMENSION(npts,nvm)                  :: ba                     !! Basal area. variable for histwrite only (m2)
    REAL(r_std), DIMENSION(npts,nvm)                  :: wood_volume            !! wood_volume (m3 m-2)
    REAL(r_std), DIMENSION(npts,nvm,nparts)           :: f_alloc                !! PFT fraction of NPP that is allocated to
                                                                                !! the different components (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements) :: bm_alloc               !! PFT biomass increase, i.e. NPP per plant  
                                                                                !! part @tex $(gC.m^{-2}dt^{-1})$ @endtex

    ! Tree level	
    REAL(r_std), DIMENSION(ncirc)                     :: step                   !! Temporary variables to solve a
                                                                                !! quadratic equation (unitless)
    REAL(r_std), DIMENSION(ncirc)                     :: s                      !! tree-level linear relationship between
                                                                                !! basal area and height. This variable is
                                                                                !! used to linearize the allocation scheme
    REAL(r_std), DIMENSION(ncirc)                     :: Cs_inc_est             !! Initial value estimate for Cs_inc. The 
                                                                                !! value is used to linearize the ba~height
                                                                                !! relationship 
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cl                     !! Individual plant, leaf compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cr                     !! Individual plant, root compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cs                     !! Individual plant, sapwood compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Ch                     !! Individual plant, heartwood compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cl_inc                 !! Individual plant increase in leaf 
                                                                                !! compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                     :: Cr_inc                 !! Individual plant increase in root
                                                                                !! compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                     :: Cs_inc                 !! Individual plant increase in sapwood
                                                                                !! compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                     :: Cf_inc                 !! Individual plant increase in fruit
                                                                                !! compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cl_incp                !! Phenology related individual plant 
                                                                                !! increase in leaf compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cr_incp                !! Phenology related individual plant 
                                                                                !! increase in leaf compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cs_incp                !! Phenology related individual plant 
                                                                                !! increase in sapwood compartment
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cl_target              !! Individual plant maximum leaf mass given 
                                                                                !! its current sapwood mass
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cr_target              !! Individual plant maximum root mass given 
                                                                                !! its current sapwood mass
                                                                                !! @tex $(gC.plant^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                     :: Cs_target              !! Individual plant maximum sapwood mass 
                                                                                !! given its current leaf mass or root mass
                                                                                !! @tex $(gC.plant^{-1})$ @endtex     
    REAL(r_std), DIMENSION(ncirc)                     :: delta_ba               !! Change in basal area for a unit
                                                                                !! investment into sapwood mass (m)
    REAL(r_std), DIMENSION(ncirc)                     :: delta_height           !! Change in height for a unit
                                                                                !! investment into sapwood mass (m)
    REAL(r_std), DIMENSION(ncirc)                     :: circ_class_ba          !! Basal area (forestry definition) of the model 
                                                                                !! tree in each circ class 
                                                                                !! @tex $(m^{2} m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                     :: circ_class_ba_eff      !! Effective basal area of the model tree in each
                                                                                !! circ class @tex $(m^{2} m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                     :: circ_class_dba         !! Share of an individual tree in delta_ba
                                                                                !! thus, circ_class_dba*gammas = delta_ba
    REAL(r_std), DIMENSION(ncirc)                     :: circ_class_height_eff  !! Effective tree height calculated from allometric 
                                                                                !! relationships (m)
    REAL(r_std), DIMENSION(ncirc)                     :: circ_class_circ_eff    !! Effective circumference of individual trees (m)
    REAL(r_std)                                       :: woody_biomass          !! Woody biomass. Temporary variable to 
                                                                                !! calculate wood volume (gC m-2)
    REAL(r_std)                                       :: temp_share             !! Temporary variable to store the share
                                                                                !! of biomass of each circumference class
                                                                                !! to the total biomass            
    REAL(r_std)                                       :: temp_class_biomass     !! Biomass across parts for a single circ
                                                                                !! class @tex $(gC m^{-2})$ @endtex
    REAL(r_std)                                       :: temp_total_biomass     !! Biomass across parts and circ classes
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)            :: store_delta_ba         !! Store delta_ba in this variable before writing
                                                                                !! to the output file (m). Adding this variable
                                                                                !! was faster than changing the dimensions
                                                                                !! of delta_ba which would have been the same
    REAL(r_std), DIMENSION(npts,nvm,ncirc)            :: store_circ_class_ba    !! Store circ_class_ba in this variable before
                                                                                !! writing to the output file (m). Adding this
                                                                                !! variable was faster than changing the 
                                                                                !! dimensions of circ_class_ba_ba which would 
                                                                                !! have been the same
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements):: check_intern           !! Contains the components of the internal
                                                                                !! mass balance chech for this routine
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)        :: closure_intern         !! Check closure of internal mass balance
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)        :: pool_start, pool_end   !! Start and end pool of this routine 
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std)                                       :: median_circ            !! Median circumference (m)
    REAL(r_std)                                       :: deficit                !! Carbon that needs to be respired in 
                                                                                !! excess of todays gpp  
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std)                                       :: excess                 !! Carbon that needs to be re-allocated
                                                                                !! after the needs of the reserve and 
                                                                                !! labile pool are satisfied  
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std)                                       :: shortage               !! Shortage in the reserves that needs to 
                                                                                !! be re-allocated after to minimise the
                                                                                !! tension between required and available
                                                                                !! reserves
                                                                                !! @tex $(gC.m^{-2}dt^{-1})$ @endtex
    
    INTEGER                                           :: i,tempi
!   (temp variables for impose intraseasonal LAI dynamic)   
    INTEGER                                           :: month_id               !! index of month 
    REAL(r_std)                                       :: ratio_move             !! tmperal variable to move the allocatable carbon
                                                                                !! from leaf to sapwood
    REAL(r_std)                                       :: impose_lai             !! get value of impose maximun LAI
    REAL(r_std)                                       :: lai_fac                !! LAI agjust coefficient 
    REAL(r_std), DIMENSION(13)                        :: lai_scale              !! monthly lai scaling facter    
    REAL(r_std)                                       :: daily_lai              !! Daily LAI value interpolated by impose lai & lai_scale 
    CHARACTER(len=256)                                :: temp_text              !! dummy text variable exchange
!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering functional allocation growth'

!! 1. Initialize

    !! 1.1 First call only
    IF ( firstcall ) THEN
      
       firstcall = .FALSE.

    ENDIF ! first call


    !! 1.2 Initialize variables at every call
    qm_height(:,:) = zero
    delta_ba = zero
    lai_target(:,:) = zero
    resp_maint(:,:) = zero
    resp_growth(:,:) = zero
    lstress_fac(:,:) = zero
    sigma(:,:) = zero
    gammas(:,:) = zero
    bm_alloc_tot(:,:) = zero
    k_latosa(:,:) = zero
    bm_alloc(:,:,:,:) = zero
    store_circ_class_ba(:,:,:) = zero
    store_delta_ba(:,:,:) = zero

    ! If npp is not initialized, bare soil value is never set.
    npp(:,:) = zero

    ! Not having this results in an unitilized error
    ! with valgrid, but I can't figure out why.  It always
    ! seems to be set before being used.
    residual(:,:) = val_exp

    ! bare soil never gets set here
    lab_fac(:,1) = zero

    !! 1.2.2 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 8
    pool_start = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          !  Initial biomass pool
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    !! 1.2.3 Initialize check for surface area conservation
    !  Veget_max is a INTENT(in) variable and can therefore
    !  not be changed during the course of this subroutine
    !  No need to check whether the subroutine preserves the
    !  total surface area of the pixel.

    !! 1.2.4 Calculate LAI threshold below which carbohydrate reserve is used.
    !  Lai_max is a PFT-dependent parameter specified in stomate_constants.f90
    ! +++CHECK+++
    ! Can we make this a function of Cs or rue_longterm? this double prescribed value does not make
    ! to much sense to me. It is not really dynamic.
    lai_happy(:) = lai_max(:) * lai_max_to_happy(:)
    ! +++++++++++
    

 !! 2. Use carbohydrate reserve to support growth

    ! Save old leaf mass, biomass got last updated in stomate_phenology.f90
    lm_old(:,:) = biomass(:,:,ileaf,icarbon)

    ! lai for bare soil is by definition zero
    lai(:,ibare_sechiba) = zero

    DO j = 2, nvm ! Loop over # PFTs 

       !! 2.1 Calculate demand for carbohydrate reserve to support leaf and root growth.
       !  Maximum time (days) since start of the growing season during which carbohydrate 
       !  may be used
       IF ( is_tree(j) ) THEN

          reserve_time = reserve_time_tree    

       ELSE

          reserve_time = reserve_time_grass

       ENDIF

       ! Growth is only supported by the use of carbohydrate reserves if the following 
       ! conditions are  statisfied:\n
       ! - PFT is not senescent;\n
       ! - LAI must be low (i.e. below ::lai_happy) and\n
       ! - Day of year of the simulation is in the beginning of the growing season.
   
       DO ipts = 1,npts

          ! Calculate lai
          lai(ipts,j) = biomass_to_lai(biomass(ipts,j,ileaf,icarbon),j)

          ! We might need the c0_alloc factor, so let's calculate it.
          c0_alloc(ipts,j) = calculate_c0_alloc(ipts, j, tau_eff_root(ipts,j), &
               tau_eff_sap(ipts,j))

       ENDDO

       WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. min_stomate ) .AND. & 
            ( .NOT. senescence(:,j) ) .AND. &
            ( lai(:,j) .LT. lai_happy(j) ) .AND. &
            ( when_growthinit(:,j) .LT. reserve_time ) )
          
          ! Tell the labile and resource pool to use its reserve
          use_reserve(:,j) = 1.0

       ENDWHERE
 
    ENDDO ! loop over # PFTs



 
 !! 3. Initialize allocation

    DO j = 2, nvm ! Loop over # PFTs 
 
       !!  3.1 Calculate scaling factors, temperature sensitivity, target lai to decide on
       !!  reserve use, labile fraction, labile biomass and total allocatable biomass 
       
       ! Convert long term temperature from K to C
       tl(:) = t2m(:) - ZeroCelsius

       DO ipts = 1, npts

          IF (veget_max(ipts,j) .LE. min_stomate) THEN

              ! this vegetation type is not present, so no reason to do the 
              ! calculation. CYCLE will take us out of the innermost DO loop
               CYCLE

          ENDIF

          !! 3.1 Water stress
          !  The waterstress factor varies between 0.1 and 1 and is calculated
          !  from ::moiavail_growingseason. The latter is only used in the allometric 
          !  allocation and its time integral is determined by tau_sap for trees
          !  (see constantes_mtc.f90 for tau_sap and see pft_constantes.f90 for 
          !  the definition of tau_hum_growingseason). The time integral for 
          !  grasses and crops is a prescribed constant (see constantes.f90). For
          !  trees KF (and indirecrtly LF) and for grasses LF are multiplied
          !  by wstress. Because the calculated values are too low for its purpose
          !  Sonke Zhaele multiply it by two in the N-branch (see stomate_season.f90). 
          !  This approach maintains the physiological basis of KF while combining it 
          !  with a simple multiplicative factor for water stress. Clearly after 
          !  multiplication with 2, wstress is closer to 1 and will thus result in a 
          !  KF values closer to the physiologically expected KF. We did not see the 
          !  need to multiply by 2 because the way we now calculate ::moiavail_growingseason
          !  is less volatile than before. Before it ranged between 0 and 1, now the 
          !  range is more like 0.4 to 0.9.

          ! Veget is now calculated from Pgap to be fully consistent within the model. Hence
          ! dividing by veget_max gives a value between 0 and 1 that denotes the amount of 
          ! light reaching the forest floor.
          IF (veget_max(ipts,j) .GT. min_stomate) THEN

             lstress_fac(ipts,j) = un - (veget(ipts,j) / veget_max(ipts,j))

          ELSE

             lstress_fac(ipts,j) = zero

          ENDIF

          !+++TEMP+++
!!$          IF( j == test_pft .AND. ipts == test_grid) &
!!$               WRITE(numout,'(A,I5,2F16.8)') 'lightstress, lstress_fac, j, ', &
!!$               j, lstress_fac(ipts,j), veget(ipts,j)
          !++++++++++

          !! 3.2 Initialize scaling factors
          ! Stand level scaling factors
          LF(ipts,j) = 1._r_std

          ! Tree level scaling factors
          ltor(ipts,j) = 1._r_std
          circ_class_height_eff(:) = 1._r_std


          !! 3.3 Calculate structural characteristics

          ! Target lai is calculated at the stand level for the tree height of a 
          ! virtual tree with the mean basal area or the so called quadratic mean diameter
          qm_dia(ipts,j) = wood_to_qmdia(circ_class_biomass(ipts,j,:,:,icarbon), &
               circ_class_n(ipts,j,:), j)
          qm_height(ipts,j) = wood_to_qmheight(circ_class_biomass(ipts,j,:,:,icarbon), &
               circ_class_n(ipts,j,:), j)


          !! 3.4 Calculate allocation factors for trees and grasses 
          IF ( SUM(biomass(ipts,j,:,icarbon)) .GT. min_stomate ) THEN

             ! Trees
             IF (is_tree(j)) THEN

                ! Note that KF may already be calculated in stomate_prescribe.f90 (if called)
                ! it is recalculated because the biomass pools for grasses and crops
                ! may have been changed in stomate_phenology.f90. Trees were added to this
                ! calculation just to be consistent.
                
                ! To be fully consistent with the hydraulic limitations and pipe theory,
                ! k_latosa_zero should be calculated from equation (18) in Magnani et al.
                ! To do so, total hydraulic resistance and tree height need to known. This
                ! poses a problem as the resistance depends on the leaf area and the leaf 
                ! area on the resistance. There is no independent equation and equations 12
                ! and 18 depend on each other and substitution would be circular. Hence 
                ! prescribed k_latosa_zero values were obtained from observational records 
                ! and are given in mtc_parameters.f90.

                ! The relationship between height and k_latosa as reported in McDowell 
                ! et al 2002 and Novick et al 2009 is implemented to adjust k_latosa for 
                ! the height of the stand.  The slope of the relationship is calculated in 
                ! stomate_data.f90 This did NOT result in a realistic model behavior.
                !!$ k_latosa(ipts,j) = wstress_fac(ipts,j) * & 
                !!$     (k_latosa_max(j) - latosa_height(j) * qm_height(ipts,j))

                ! Alternatively, k_latosa is also reported to be a function of diameter 
                ! (i.e. stand thinning, Simonin et al 2006, Tree Physiology, 26:493-503).
                ! Here the relationship with thinning was interpreted as a realtionship with
                ! light stress.
                ! +++CHECK+++
                ! How dow we want to account for waterstress?
!!$                k_latosa(ipts,j) = k_latosa_min(j) + (wstress_fac(ipts,j) * lstress_fac(ipts,j) * &
!!$                     (k_latosa_max(j)-k_latosa_min(j)))
!!$                k_latosa(ipts,j) = wstress_fac(ipts,j) * (k_latosa_min(j) + (lstress_fac(ipts,j) * &
!!$                     (k_latosa_max(j)-k_latosa_min(j))))
                k_latosa(ipts,j) = (k_latosa_adapt(ipts,j) + (lstress_fac(ipts,j) * &
                     (k_latosa_max(j)-k_latosa_min(j))))
                ! +++++++++++
               
                ! Also k_latosa has been reported to be a function of CO2 concentration 
                ! (Atwell et al. 2003, Tree Physiology, 23:13-21 and Pakati et al. 2000, 
                ! Global Change Biology, 6:889-897). This effect is not accounted for in 
                ! the current code

                ! Scaling factor to convert sapwood mass into leaf mass (KF)
                ! derived from
                ! LA_ind = k1 * SA_ind, k1=latosa (pipe-model)
                ! <=> Cl * vm/ind * sla = k1 * Cs * vm/ind / wooddens / tree_ff / height_new
                ! <=> Cl = Cs * k1 / wooddens / tree_ff/ height_new /sla
                ! <=> Cl = Cs * KF / height_new, where KF = k1 / (wooddens * sla * tree_ff)
                ! (1) Cl = Cs * KF / height_new
                KF_old = KF(ipts,j) 
                KF(ipts,j) = k_latosa(ipts,j) / (sla(j) * pipe_density(j) * tree_ff(j))
                
                ! KF of the previous time step was stored in ::KF_old to check its absolute 
                ! change. If this absolute change is too big the whole allocation will crash
                ! because it will calculate negative increments which are compensated by 
                ! positive increments that exceed the available carbon for allocation. This 
                ! would suggest that for examples the plant destructs leaves and uses the 
                ! available carbon to produce more roots. This is would repesent an unwanted 
                ! outcome. Large changes from time step to another makes its difficult for 
                ! the scheme to ever reach allometric balance. This balance is needed for the
                ! allocation scheme to allow 'ordinary allocation', which in turn is needed
                ! to make use of the allocation rule of Dhote and Deleuze. It needs to be 
                ! avoided that the code spends too much time in phenological growth and the 
                ! if-then statements that help to restore allometric balance. For this reason
                ! the absolute change in KF from one time step to another are truncated.
                IF (KF_old - KF(ipts,j) .GT. max_delta_KF ) THEN
        
                   IF(ld_warn)THEN
                      WRITE(numout,*) 'WARNING 2: KF was truncated'
                      WRITE(numout,*) 'WARNING 2: PFT, ipts: ',j,ipts
                      WRITE(numout,'(A,3F20.10)') 'WARNING 2: KF_old, KF(ipts,j), max_delta_KF: ',&
                           KF_old, KF(ipts,j), max_delta_KF
                   ENDIF

                   ! Add maximum absolute change
                   KF(ipts,j) = KF_old - max_delta_KF
                   
                   IF(ld_warn)THEN
                      WRITE(numout,'(A,3F20.10)') 'WARNING 2: Reset, KF_old, KF(ipts,j): ',&
                           KF_old, KF(ipts,j)
                   ENDIF
                ELSEIF (KF_old - KF(ipts,j) .LT. -max_delta_KF) THEN
                   
                   IF(ld_warn)THEN
                      WRITE(numout,*) 'WARNING 3: KF was truncated'
                      WRITE(numout,*) 'WARNING 3: PFT, ipts: ',j,ipts
                      WRITE(numout,'(A,3F20.10)') 'WARNING 3: KF_old, KF(ipts,j), max_delta_KF: ',&
                           KF_old, KF(ipts,j), -max_delta_KF
                   ENDIF

                   ! Remove maximum absolute change
                   KF(ipts,j) = KF_old + max_delta_KF
                   
                   IF(ld_warn)THEN
                      WRITE(numout,'(A,3F20.10)') 'WARNING 3: Reset, KF_old, KF(ipts,j): ',&
                        KF_old, KF(ipts,j)
                   ENDIF
                ELSE
                   ! The change in KF is acceptable no action required
                ENDIF
    
                ! Scaling factor to convert sapwood mass into root mass  (LF) 
                ! derived from 
                ! Cs = c0 * height * Cr (Magnani 2000)
                ! Cr = Cs / c0 / height_new
                ! scaling parameter between leaf and root mass, derived from
                ! Cr = Cs / c0 / height_new
                ! let Cs = Cl / KF * height_new
                ! <=> Cr = ( Cl * height_new / KF ) / ( c0 * height_new )
                ! <=> Cl = Cr * KF * c0
                ! <=> Cl = Cr * LF, where LF = KF * c0
                ! (2) Cl = Cr * LF
                ! +++CHECK+++
                ! How do we want to account for waterstress? wstress is accounted for in c0_alloc
                LF(ipts,j) = c0_alloc(ipts,j) * KF(ipts,j) 
!!$                LF(ipts,j) = c0_alloc(ipts,j) * KF(ipts,j) * wstress_fac(ipts,j)
                ! +++++++++++

                ! Calculate non-nitrogen stressed leaf to root ratio to calculate the
                ! allocation to the reserves. Should be multiplied by a nitrogen stress
                ! have a look in OCN. This code should be considered as a placeholder
                ltor(ipts,j) = c0_alloc(ipts,j) * KF(ipts,j)

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                   WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
                   WRITE(numout,*) 'c0_alloc, ', c0_alloc(ipts,j)
                   WRITE(numout,*) 'tau_root, tau_sap, ', tau_eff_root(ipts,j), tau_eff_sap(ipts,j)
                   WRITE(numout,*) 'k_root, k_sap, ', k_root(j), k_sap(j)
                   WRITE(numout,*) 'ltor, ', ltor(ipts,j)
                   
                ENDIF
                !----------

             ! Grasses and crops
             ELSEIF (.NOT. is_tree(j)) THEN
                
                !+++CHECK+++
                ! Similar to ::k_latosa for trees we defined it for grasses. Note that for trees
                ! the definition is supported by some observations. For grasses we didn't look very
                ! hard to check the literature. Someone interested in grasses should invest some
                ! time in this issue and replace this code by a parameter that can be derived
                ! from observations. In the end it was decided to use the same variable name for 
                ! grasses, crops and trees as that allowed us to optimize this parameter.
                k_latosa(ipts,j) = (k_latosa_adapt(ipts,j) + (lstress_fac(ipts,j) * &
                     (k_latosa_max(j)-k_latosa_min(j))))

                ! The mass of the structural carbon relates to the mass of the leaves through
                ! a prescribed parameter ::k_latosa
                KF(ipts,j) = k_latosa(ipts,j) 
                !++++++++++
               
                ! Stressed root allocation, wstress is accounted for in c0_alloc  
	        LF(ipts,j) = c0_alloc(ipts,j) * KF(ipts,j)	

	        ! Calculate non-nitrogen stressed leaf to root ratio to calculate the
                ! allocation to the reserves 
	        ltor(ipts,j) = c0_alloc(ipts,j) * KF(ipts,j)

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                   WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
                   WRITE(numout,*) 'c0_alloc, ', c0_alloc(ipts,j)
                   WRITE(numout,*) 'tau_root, tau_sap, ', tau_eff_root(ipts,j), tau_eff_sap(ipts,j)
                   WRITE(numout,*) 'k_root, k_sap, ', k_root(j), k_sap(j)
                   WRITE(numout,*) 'ltor, ', ltor(ipts,j)
                   
                ENDIF
                !----------
                
             ENDIF

          ENDIF

          !+++CHECK+++
          !! 3.5 Calculate optimal LAI
          !  The calculation of the optimal LAI was copied and adjusted from O-CN. In O-CN it
          !  was also used in the allocation but that seems to be inconsistent with the allometric
          !  rules that are implemented. Say that the actual LAI is below the optimal LAI then
          !  the O-CN approach will keep pumping carbon to grow the optimal LAI. If we would apply
          !  the same method it means that during this phase the rule of Deleuze and Dhote would
          !  not be used. For that reason we dropped the use of LAI_optimal and replaced it by
          !  an allometric-based Cl_target value. Initially, lai_target was still calculated as 
          !  described below and used in the calculation of the reserves.
          !  Further testing showed that for some parameter sets lai_target was over 8 whereas the
          !  realized lai was close to 4. This leaves us with a frustrated plant that will invest a
          !  lot in its reserves but can never use them because it is constrained by the allometric
          !  rules. To grow an LAI of 8 it would need to have a crazy sapwoodmass.
          !  At a more fundamental level it is clear why the plant's LAI should not exceed lai_target
          !  because then it costs more to produce and maintain the leaf than that the new leaf can
          !  produce but there is no reason why the plant should try to reach lai_target. For these
          !  reasons it was decided to abandon this approach to lai_target and simply replace 
          !  lai_target by Cl_target * sla

          !! 3.5.1 Scaling factor
          !  Scaling factor to convert variables to the individual plant
          !  Different approach between the DGVM and statitic approach
          IF (control%ok_dgvm) THEN

             ! The DGVM does currently NOT work with the new allocation, consider this as
             ! placeholder. The original code had two different transformations to 
             ! calculate the scalars. Both could be used but the units will differ.
             ! When fixing the DGVM check which quantities need to be multiplied by scal 
             ! scal = ind(ipts,j) * cn_ind(ipts,j) / veget_max(ipts,j)
             scal = veget_max(ipts,j) / ind(ipts,j)  

          ELSE

             ! circ_class_biomass contain the data at the tree level
             ! no conversion required
             scal = 1.

          ENDIF

          !! 3.5.2 Calculate lai_target based on the allometric rules
          IF ( is_tree(j)) THEN

             ! Basal area at the tree level (m2 tree-1)
             circ_class_ba_eff(:) = wood_to_ba_eff(circ_class_biomass(ipts,j,:,:,icarbon),j)

             ! Current biomass pools per tree (gC tree^-1) 
             ! We will have different trees so this has to be calculated from the diameter relationships            
             Cs(:) = ( circ_class_biomass(ipts,j,:,isapabove,icarbon) + &
                  circ_class_biomass(ipts,j,:,isapbelow,icarbon) ) * scal
             Cr(:) = circ_class_biomass(ipts,j,:,iroot,icarbon) * scal
             Cl(:) = circ_class_biomass(ipts,j,:,ileaf,icarbon) * scal
             Ch(:) = ( circ_class_biomass(ipts,j,:,iheartabove,icarbon) + &
                  circ_class_biomass(ipts,j,:,iheartbelow,icarbon) ) * scal

             DO l = 1,ncirc 

                !  Calculate tree height
                circ_class_height_eff(l) = pipe_tune2(j)*(4/pi*circ_class_ba_eff(l))**(pipe_tune3(j)/2)

                !  Do the biomass pools respect the pipe model?
                !  Do the current leaf, sapwood and root components respect the allometric 
                !  constraints? Due to plant phenology it is possible that we have too much 
                !  sapwood compared to the leaf and root mass (i.e. in early spring). 
                !  Calculate the optimal root and leaf mass, given the current wood mass 
                !  by using the basic allometric relationships. Calculate the optimal sapwood
                !  mass as a function of the current leaf and root mass.
                Cl_target(l) = MAX( KF(ipts,j) * Cs(l) / circ_class_height_eff(l), &
                     Cr(l) * LF(ipts,j) , Cl(l))
                Cs_target(l) = MAX( Cl(l) / KF(ipts,j) * circ_class_height_eff(l), &
                     Cr(l) * LF(ipts,j) / KF(ipts,j) * circ_class_height_eff(l) , Cs(l))

                ! Check dimensions of the trees
                ! If Cs = Cs_target then ba and height are correct, else calculate the correct dimensions
                IF ( Cs_target(l) - Cs(l) .GT. min_stomate ) THEN

                   ! If Cs = Cs_target then dia and height are correct. However, if Cl = Cl_target
                   ! or Cr = Cr_target then dia and height need to be re-estimated. Cs_target should
                   ! satify the relationship Cl/Cs = KF/height where height is a function of Cs_target
                   ! <=> (KF*Cs_target)/(pipe_tune2*(Cs_target+Ch)/pi
                   ! /4)**(pipe_tune3/(2+pipe_tune3)) = Cl_target
                   ! Search Cs needed to sustain the max of Cl or Cr.
                   !  Search max of Cl and Cr first 
                   Cl_target(l) = MAX(Cl(l), Cr(l)*LF(ipts,j))

                   !---TEMP---
                   IF (j.EQ.test_pft .AND. ld_alloc) THEN
                      WRITE(numout,*) 'Does the tree needs reshaping? Class: ',l
                      WRITE(numout,*) 'circ_class_height_eff, ', circ_class_height_eff(l)
                      WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
                      WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(l)-Cl(l), Cl_target(l), Cl(l)
                      WRITE(numout,*) 'Cs, ', Cs(l)
                      WRITE(numout,*) 'Cr, ', Cr(l)
                      WRITE(numout,*) 'Ch, ', Ch(l)
                   ENDIF
                   !----------

                   Cs_target(l) =  newX(KF(ipts,j), Ch(l),&
                        & pipe_tune2(j), pipe_tune3(j), Cl_target(l),&
                        & tree_ff(j)*pipe_density(j)*pi/4*pipe_tune2(j), Cs(l),&
                        & 2*Cs(l), 2, j, ipts)

                   ! Recalculate height and ba from the correct
                   !  Cs_target
                   circ_class_height_eff(l) = Cs_target(l)*KF(ipts,j)&
                        &/Cl_target(l)
                   circ_class_ba_eff(l) = pi/4*(circ_class_height_eff(l)&
                        &/pipe_tune2(j))**(2/pipe_tune3(j))
                   Cl_target(l) = KF(ipts,j) * Cs_target(l) /&
                        & circ_class_height_eff(l)
                   Cr_target(l) = Cl_target(l) / LF(ipts,j)

                   !---TEMP---
                   IF (j.EQ.test_pft .AND. ld_alloc) THEN
                      WRITE(numout,*) 'New Cl_target, ', Cl_target(l)
                      WRITE(numout,*) 'New Cs_target, ', Cs_target(l)
                      WRITE(numout,*) 'New Cr_target, ', Cr_target(l)       
                   ENDIF
                   !---------- 

                ENDIF

             ENDDO

             ! Calculate lai_target
             lai_target(ipts,j) = SUM(Cl_target(:)*circ_class_n(ipts,j,:)) * sla(j)

          ! Grasses and croplands
          ELSEIF ( .NOT. is_tree(j)) THEN
             
             ! Current biomass pools per grass/crop (gC ind^-1)
             ! Cs has too many dimensions for grass/crops. To have a consistent notation the same variables
             ! are used as for trees but the dimension of Cs, Cl and Cr i.e. ::ncirc should be ignored            
             Cs(1) = biomass(ipts,j,isapabove,icarbon) * scal
             Cr(1) = biomass(ipts,j,iroot,icarbon) * scal
             Cl(1) = biomass(ipts,j,ileaf,icarbon) * scal
             Ch(1) = zero
   
             ! Do the biomass pools respect the pipe model?
             ! Do the current leaf, sapwood and root components respect the allometric 
             ! constraints? Calculate the optimal root and leaf mass, given the current wood mass 
             ! by using the basic allometric relationships. Calculate the optimal sapwood
             ! mass as a function of the current leaf and root mass.
             Cl_target(1) = MAX( Cs(1) * KF(ipts,j) , Cr(1) * LF(ipts,j), Cl(1) )
             Cs_target(1) = MAX( Cl_target(1) / KF(ipts,j), Cr(1) * LF(ipts,j) / KF(ipts,j), Cs(1) ) 
             Cr_target(1) = MAX( Cl_target(1) / LF(ipts,j), Cs_target(1) * KF(ipts,j) / LF(ipts,j), Cr(1) )

             ! Calculate lai_target
             lai_target(ipts,j) = Cl_target(1) * circ_class_n(ipts,j,1) * sla(j)

             !---TEMP---
             IF (j.EQ.test_pft .AND. ld_alloc) THEN
                WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
             ENDIF
             !----------

          ENDIF

!!$          !! 3.5 Calculate optimum LAI 
!!$          !  Lai is optimised for mean annual radiation use efficiency and the C costs
!!$          !  for producing the canopy. The cost-benefit ratio is optimised when the 
!!$          !  marginal gain / marginal cost = 1
!!$          !  Investing 1 gC in the canopy comes at a total cost that is composed by the 
!!$          !  C required for the canopy in addition to the roots and the sapwood to support
!!$          !  the canopy. The total cost (C) is thus calculated as C: 
!!$          !  LAI/sla * ( (one_year/tau_leaf) + (one_year/tau_root)/LF + (one_year/tau_sap)*height/KF))
!!$          !  The marginal cost for one unit of LAI is then dC/dLAI : 
!!$          !  (one_year/tau_leaf)/sla + (one_year/tau_root)/LF/sla + (one_year/tau_sap)*height/KF/sla)
!!$          !  Where, tau_leaf is given by ::tau_leaf in days, tau_root by ::tau_root in
!!$          !  days and tau_sap by ::tau_sap in days. LF is unitless, KF is expressed in meters
!!$          !  and sla in m^2.gC^{-1}. The unit of dC/dLAI is thus gC.m^{-2} but all turnover
!!$          !  times need to be expressed on an annual scale.  
!!$          !  Investing 1gC in the canopy enables the plant to assimilate more carbon
!!$          !  The gain (G) can be approximated by using the 'radiation use efficiency' as 
!!$          !  follows: RUE * one_year ( 1. - exp (-0.5 * LAI ))
!!$          !  Where, 0.5 is the extinction factor that accounts for the fact the lower parts 
!!$          !  of the canopy receive less light. Note that RUE has a peculiar definition and is 
!!$          !  calculated as the ratio of GPP over the fraction of radiation absorbed by the canopy.
!!$          !  Hence the unit of RUE is gC.m^{-2}.day^{-1}. The marginal gain of one unit of LAI is dG/dLAI: 
!!$          !  0.5 * one_year * RUE * exp (-0.5 * LAI). 
!!$          !  Subsequently, the optimal LAI is approximated by
!!$          !  LAI_opt = -2. * log(2*(dC/dt)/(RUE*one_year))           
!!$          !  Added the qm_height requirement since for a grass, it had no biomass
!!$          !  but it did have individuals.  This caused qm_height to be zero and a crash
!!$          !  in the calculation of lai_target.
!!$          IF ( (rue_longterm(ipts,j) .GT. min_stomate) .AND. (ind(ipts,j) .NE. zero &
!!$               .AND. qm_height(ipts,j) .NE. 0) ) THEN
!!$
!!$             ! Scheme in line with the documentation
!!$             lai_target(ipts,j) = -deux* log( (deux * (one_year/tau_leaf(j))/sla(j) + &
!!$                  ((one_year/tau_root(j))/LF(ipts,j))/sla(j) + &
!!$                  ((one_year/tau_sap(j))*qm_height(ipts,j)/KF(ipts,j))/sla(j)) / &
!!$                  (rue_longterm(ipts,j)*one_year))
!!$             lai_target(ipts,j) = MAX(MIN(lai_target(ipts,j),12.),.5)
!!$
!!$          ELSE
!!$
!!$             lai_target(ipts,j) = 0.5
!!$
!!$          ENDIF
          !++++++++++

          !! 3.6 Calculate mean leaf age
          leaf_meanage = zero
          DO m = 1,nleafages
          
             leaf_meanage = leaf_meanage + leaf_age(ipts,j,m) * leaf_frac(ipts,j,m)
          
          ENDDO
             

          !! 3.7 Calculate labile fraction 
          !  Use constant labile fraction to initiate on-set of leaves in spring
          IF ( (biomass(ipts,j,ileaf,icarbon) .LE. min_stomate) .AND. &
               (use_reserve(ipts,j) .GT. min_stomate) ) THEN

             lab_fac(ipts,j) = 0.7

          ! Calculate labile fraction when a canopy is present but its lai is below ::lai_target
          ! the labile fraction is a function of ::leaf_meanage, the current lai calculated as 
          ! ::biomass(ipts,j,ileaf,icarbon)*sla(j), the ::lai_target and ::ecureuil. This functions 
          ! scales lab_fac to a value between 0.1 and 0.7. Its scientific basis remains unclear.    
          ELSEIF ( (biomass(ipts,j,ileaf,icarbon) .GT. min_stomate) .AND. & 
               (lai_target(ipts,j) .GT. min_stomate) .AND. (.NOT. senescence(ipts,j)) ) THEN

                lab_fac(ipts,j) = 0.1 + 0.6 * MAX(0.0,1.-MAX(ecureuil(j)*leaf_meanage/45., & 
                   biomass(ipts,j,ileaf,icarbon)*sla(j)/lai_target(ipts,j)))

          ! If the canopy has reached lai_target or is senescent lab_fac = 0.1
          ELSE

                lab_fac(ipts,j) = 0.1

          ENDIF

        
          !! 3.8 Calculate allocatable carbon
          !  Total allocatable biomass during this time step determined from GPP.
          !  GPP was calculated as CO2 assimilation in enerbil.f90
          !  Under some exceptional conditions :gpp could be negative when
          !  the dark respiration exceeds the photosynthesis. When this happens
          !  the dark respiration is paid for by the labile and carbres pools
          IF ( (biomass(ipts,j,ilabile,icarbon) + gpp_daily(ipts,j) * dt) .LT. zero ) THEN

             deficit = (biomass(ipts,j,ilabile,icarbon) + gpp_daily(ipts,j) * dt)

           ! The deficit is less than the carbon reserve
             IF (-deficit .LE. biomass(ipts,j,icarbres,icarbon)) THEN

              ! Pay the deficit from the reserve pool
                biomass(ipts,j,icarbres,icarbon) = &
                     biomass(ipts,j,icarbres,icarbon) + deficit
                biomass(ipts,j,ilabile,icarbon) = &
                     biomass(ipts,j,ilabile,icarbon) - deficit

             ELSE

                ! Not enough carbon to pay the deficit, the individual 
                ! is going to die at the end of this day
                biomass(ipts,j,ilabile,icarbon) = &
                     biomass(ipts,j,ilabile,icarbon) + biomass(ipts,j,icarbres,icarbon) 
                biomass(ipts,j,icarbres,icarbon) = zero

                ! Truncate the dark respiration to the available carbon.  Now we
                ! should use up all the reserves.  If the plant has no leaves, it
                ! will die quickly after this.
                gpp_daily(ipts,j) = - biomass(ipts,j,ilabile,icarbon)/dt 

             ENDIF

          ENDIF
       
          ! Labile carbon pool after possible correction for dark respiration
          biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) + &
               gpp_daily(ipts,j) * dt

          IF(ld_alloc .AND. ipts == test_grid .AND. j == test_pft)THEN
             WRITE(numout,*) 'Adding gpp to labile pool'
             WRITE(numout,*) 'biomass(ipts,j,ilabile,icarbon)',biomass(ipts,j,ilabile,icarbon)
             WRITE(numout,*) 'gpp_daily(ipts,j)',gpp_daily(ipts,j)
          ENDIF

          !! 3.9 Calculate activity of labile carbon pool  
          !  Similar realtionship as that used for the temperature response of 
          !  maintenance respiration.  The parameters in the equation were calibrated
          !  to give a fraction of 0.1 of GPP at reference temperature tl (i.e. 10°C)
          !  Note that the temperature response has a lower slope than for respiration
          !  to avoid too large turnover rates at high temperature.
          IF (tl(ipts) .GT. -2.) THEN

             gtemp = EXP(308.56/4.*(1.0/56.02-1.0/(tl(ipts)+46.02)))

          ELSE

             gtemp = zero            

          ENDIF

          !  If there is a plant, and we are either at the very start or in the growing season
          !  not during senescences, calculate labile pool use for growth
          IF (ind(ipts,j) .GT. min_stomate .AND. &
!!$               when_growthinit(ipts,j) .LT. (large_value - un) .AND. &
               .NOT.senescence(ipts,j)) THEN  

             ! The labile pool is filled. Re-calculate turnover of the labile pool. 
             ! Only if the labile pool is very small turnover will exceed 0.75 
             ! and the pool will thus be almost entirely emptied
             IF (biomass(ipts,j,ilabile,icarbon) .GT. min_stomate) THEN

                 gtemp = MAX(MIN( gtemp * lab_fac(ipts,j), 0.75 ), zero)

             ! The labile pool is empty but the carbohydrate pool is filled. Move carbohydrates to the labile
             ! pool and recalculate the turnover of the labile pool.
             ELSEIF (biomass(ipts,j,icarbres,icarbon) .GT. min_stomate) THEN

                biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) + 0.05 * &
                     biomass(ipts,j,icarbres,icarbon)
                biomass(ipts,j,icarbres,icarbon) = biomass(ipts,j,icarbres,icarbon) * 0.95
                gtemp = MAX(MIN(gtemp*lab_fac(ipts,j), 0.75), zero)

                IF(ld_alloc .AND. ipts == test_grid .AND. j == test_pft)THEN
                   WRITE(numout,*) 'Moving some carbon from carbres to labile pool 1'
                   WRITE(numout,*) 'biomass(ipts,j,ilabile,icarbon)',biomass(ipts,j,ilabile,icarbon)
                   WRITE(numout,*) 'biomass(ipts,j,icarbres,icarbon)',biomass(ipts,j,icarbres,icarbon)
                ENDIF

             ! There is no labile or carbohydrate pool to use carbon from. Labile pool is not used
             ELSE

                gtemp = zero

             ENDIF
   
          ENDIF


          !! 3.10 Calculate allocatable part of the labile pool
          !  If there is a plant, and we are 
          !  either at the very start or in the growing season not during 
          !  senescences and after senescence only if use_reserve > 0., calculate
          !  labile pool use for growth.
          IF (ind(ipts,j) .GT. min_stomate .AND. &
!!$               when_growthinit(ipts,j) .LT. (large_value - un) .AND. &
               .NOT.senescence(ipts,j)) THEN

             ! Use carbon from the labile pool to allocate. The allometric (or 
             ! functional) allocation scheme transfers gpp to the labile pool 
             ! (see above) and then uses the labile pool (gpp + labile(t-1)) to sustain 
             ! growth. The fraction of the labile pool that can be used is a 
             ! function of the temperature and phenology. bm_alloc_tot in 
             ! gC m-2 dt-1
             bm_alloc_tot(ipts,j) = gtemp * biomass(ipts,j,ilabile,icarbon)

          ! The conditions do not support growth
          ELSE

             bm_alloc_tot(ipts,j) = zero
             
          ENDIF

          ! Update the labile carbon pool 
          biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) - &
               bm_alloc_tot(ipts,j)

          IF(ld_alloc .AND. ipts == test_grid .AND. j == test_pft)THEN
             WRITE(numout,*) "First bm_alloc_tot ", bm_alloc_tot(ipts,j)
             WRITE(numout,*) "senescence(ipts,j) ", senescence(ipts,j)
             WRITE(numout,*) "gtemp ", gtemp
             WRITE(numout,*) "biomass(ipts,j,ilabile,icarbon) ", biomass(ipts,j,ilabile,icarbon)
             WRITE(numout,*) "biomass(ipts,j,icarbres,icarbon) ", biomass(ipts,j,icarbres,icarbon)
             WRITE(numout,*) "lab_fac(ipts,j) ", lab_fac(ipts,j)
             WRITE(numout,*) "tl(ipts) ", tl(ipts)
          ENDIF


          !! 3.11 Maintenance respiration
          !  First, total maintenance respiration for the whole plant is calculated by 
          !  summing maintenance respiration of the different plant compartments.
          !  This simply recalculates the maintenance respiration from stomate_resp.f90   
          !  Maintenance respiration of the different plant parts is calculated in 
          !  stomate_resp.f90 as a function of the plant's temperature, the long term 
          !  temperature and plant coefficients:
          !  The unit of ::resp_maint is gC m-2 dt-1
          resp_maint(ipts,j) = resp_maint(ipts,j) + SUM(resp_maint_part(ipts,j,:))
           
          ! Following the calculation of hourly maintenance respiration, verify that 
          ! the PFT has not been killed after calcul of resp_maint_part in stomate.
          ! Can this generaly calculated ::resp_maint be use under the given
          ! conditions? Surpress the respiration for deciduous
          !  PFTs as long as they haven't carried leaves at least once. When 
          !  starting from scratch there is no budburst in the first year because 
          !  the longterm phenological parameters are not initialized yet. If
          !  not surpressed respiration consumes all the reserves before the PFT
          !  can start growing. The code would establish a new PFT but it was
          !  decided to surpress this respiration because it has no physiological
          !  bases.
          IF (ind(ipts,j) .GT. min_stomate .AND. &
               rue_longterm(ipts,j) .NE. un) THEN

             !+++CHECK+++
             ! Can the calculated maintenance respiration be used ? Or 
             ! does it has to be adjusted for special cases. Maintenance 
             ! respiration should be positive. In case it is very low, use 20%
             ! (::maint_from_labile) of the active labile carbon pool (gC m-2 dt-1)
             ! resp_maint(ipts,j) = MAX(zero, MAX(maint_from_labile * gtemp * 
             ! biomass(ipts,j,ilabile,icarbon), resp_maint(ipts,j)))
         
             ! Calculate resp_maint for the labile pool as well, no need to have the
             ! above threshold. Make sure resp_maint is not zero
             resp_maint(ipts,j) = MAX(zero, resp_maint(ipts,j))
             !+++++++++++

             ! Phenological growth makes use of the reserves. Some carbon needs to remain
             ! to support the growth, hence, respiration will be limited. In this case 
             ! resp_maint ((gC m-2 dt-1) should not be more than 80% (::maint_from_gpp) 
             ! of the GPP (gC m-2 s-1) 
             IF (lab_fac(ipts,j) .GT. 0.3) THEN

                resp_maint(ipts,j) = MIN( MAX(zero, maint_from_gpp * gpp_daily(ipts,j) * dt), &
                     resp_maint(ipts,j))

             ENDIF

          ELSE

             ! No plants, no respiration
             resp_maint(ipts,j) = zero

          ENDIF

          ! The calculation of ::resp_maint is solely based on the demand i.e.
          ! given the biomass and the condition of the plant, how much should be
          ! respired. It is not sure that this demand can be satisfied i.e. the 
          ! calculated maintenance respiration may exceed the available carbon
          IF ( bm_alloc_tot(ipts,j) - resp_maint(ipts,j) .LT. zero ) THEN

             deficit = bm_alloc_tot(ipts,j) - resp_maint(ipts,j)

             ! The deficit is less than the carbon reserve
             IF (-deficit .LE. biomass(ipts,j,icarbres,icarbon)) THEN

                ! Pay the deficit from the reserve pool
                biomass(ipts,j,icarbres,icarbon) = &
                     biomass(ipts,j,icarbres,icarbon) + deficit
                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) - deficit

             ELSE

                ! Not enough carbon to pay the deficit, the individual 
                ! is going to die at the end of this day
                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) + &
                     biomass(ipts,j,icarbres,icarbon) 
                biomass(ipts,j,icarbres,icarbon) = zero

                ! Truncate the maintenance respiration to the available carbon
                resp_maint(ipts,j) = bm_alloc_tot(ipts,j)

             ENDIF

          ENDIF
         
          ! Final ::resp_maint is known
          bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) - resp_maint(ipts,j)
          IF(ld_alloc .AND. ipts == test_grid .AND. j == test_pft)THEN
             WRITE(numout,*) "resp_maint ", resp_maint(ipts,j)
          ENDIF

          !! 3.12 Growth respiration
          !  Calculate total growth respiration and update allocatable carbon
          !  Growth respiration is a tax on productivity, not actual allocation
          !  Total growth respiration has be calculated before the allocation 
          !  takes place because the allocation itself is not linear. After 
          !  the allocation has been calculated, growth respiration can be 
          !  calculated for each biomass component separatly. The unit of
          !  resp_growth is gC m-2 dt-1. Surpress the respiration for deciduous
          !  PFTs as long as they haven't carried leaves at least once. When 
          !  starting from scratch there is no budburst in the first year because 
          !  the longterm phenological parameters are not initialized yet. If
          !  not surpressed respiration consumes all the reserves before the PFT
          !  can start growing. The code would establish a new PFT but it was
          !  decided to surpress this respiration because it has no physiological
          !  bases.
          IF (ind(ipts,j) .GT. min_stomate .AND. &
               rue_longterm(ipts,j) .NE. un) THEN

             resp_growth(ipts,j) = frac_growthresp(j) * MAX(zero, bm_alloc_tot(ipts,j))
             bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) - resp_growth(ipts,j)

          ENDIF
          
          IF(ld_alloc .AND. j == test_pft .AND. ipts == test_grid)THEN
             WRITE(numout,*) 'initial bm_alloc_tot, ', bm_alloc_tot(ipts,j) + resp_growth(ipts,j)
             WRITE(numout,*) 'growth_resp, ', resp_growth(ipts,j) 
             WRITE(numout,*) 'bm_alloc_tot after growth resp ',bm_alloc_tot(ipts,j)
             WRITE(numout,*) 'gpp_daily: ',gpp_daily(ipts,j)
          ENDIF

          ! Occasionally, there is a very special situation which arises, where
          ! bm_alloc_tot is greater than min_stomate before accounting for growth respiration,
          ! but not afterwards.  This causes a mass balance error because growth respiration is
          ! non-zero but bm_alloc_tot is too small to trigger loops below, so nothing is
          ! done with that carbon.  In this situation, the amout of carbon to allocate
          ! is so low that nothing really changes.  We set the growth respiration
          ! to zero in this special case to avoid mass imbalance, even though this
          ! will not effect the trajectory of the plant.  It seems to happen on
          ! the same day as leaves start growing, before any GPP is calculated.
          IF(((bm_alloc_tot(ipts,j) + resp_growth(ipts,j)) .GT. min_stomate) .AND. &
               (bm_alloc_tot(ipts,j) .LT. min_stomate))THEN

             bm_alloc_tot(ipts,j)=bm_alloc_tot(ipts,j) + resp_growth(ipts,j)
             resp_growth(ipts,j)=zero
             
          ENDIF

          !! 3.12 Distribute stand level ilabile and icarbres at the tree level
          !  The labile and carbres pools are calculated at the stand level but
          !  are then redistributed at the tree level. This has the advantage 
          !  that biomass and circ_class_biomass have the same dimensions for 
          !  nparts which comes in handy when phenology and mortality are
          !  calculated.
          IF ( is_tree(j) ) THEN

             ! Reset to zero to enable a loop over nparts
             circ_class_biomass(ipts,j,:,ilabile,:) = zero
             circ_class_biomass(ipts,j,:,icarbres,:) = zero

             ! Distribute labile and reserve pools over the circumference classes 
             DO m = 1,nelements

                ! Total biomass across parts and circumference classes
                temp_total_biomass = zero

                DO l = 1,ncirc 

                   DO k = 1,nparts

                      temp_total_biomass = temp_total_biomass + &
                           circ_class_biomass(ipts,j,l,k,m) * circ_class_n(ipts,j,l)

                   ENDDO

                ENDDO

                ! Total biomass across parts but for a specific circumference class
                DO l = 1,ncirc

                   temp_class_biomass = zero

                   DO k = 1,nparts

                      temp_class_biomass = temp_class_biomass + &
                           circ_class_biomass(ipts,j,l,k,m) * circ_class_n(ipts,j,l)

                   ENDDO
    
                   IF (temp_total_biomass .NE. zero) THEN

                      ! Share of this circumference class to the total biomass                      
                      temp_share = temp_class_biomass / temp_total_biomass

                      ! Allocation of ilabile at the tree level (gC tree-1)
                      circ_class_biomass(ipts,j,l,ilabile,m) = temp_share * &
                           biomass(ipts,j,ilabile,m) / circ_class_n(ipts,j,l)

                      ! Allocation of icarbres at the tree level (gC tree-1)
                      circ_class_biomass(ipts,j,l,icarbres,m) = temp_share * &
                           biomass(ipts,j,icarbres,m) / circ_class_n(ipts,j,l)

                   ELSE

                      circ_class_biomass(ipts,j,l,ilabile,m) = zero
                      circ_class_biomass(ipts,j,l,icarbres,m) = zero
                      
                   ENDIF

                ENDDO ! ncirc

             ENDDO  ! nelements

          ! Grasses and crops
          ELSE

             DO m = 1,nelements

                ! synchronize biomass and circ_class_biomass
                IF (ind(ipts,j) .GT. zero) THEN

                   circ_class_biomass(ipts,j,1,:,m) = biomass(ipts,j,:,m) / ind(ipts,j)

                ELSE

                   circ_class_biomass(ipts,j,1,:,m) = zero

                ENDIF

             ENDDO

          ENDIF ! is_tree

       ENDDO ! pnts


 !! 5. Allometric allocation

       DO ipts = 1, npts

          !!  5.1 Initialize allocated biomass pools
          f_alloc(ipts,j,:) = zero
          Cl_inc(:) = zero
          Cs_inc(:) = zero
          Cr_inc(:) = zero
          Cf_inc(:) = zero
          Cl_incp(:) = zero
          Cs_incp(:) = zero
          Cr_incp(:) = zero 
          Cs_inc_est(:) = zero
          Cl_target(:) = zero
          Cr_target(:) = zero
          Cs_target(:) = zero
 
          !! 5.2 Calculate allocated biomass pools for trees

          !! 5.2.1 Stand to tree allocation rule of Deleuze & Dhote
          IF ( is_tree(j) .AND. bm_alloc_tot(ipts,j) .GT. min_stomate ) THEN

             !  Basal area at the tree level (m2 tree-1)
             circ_class_ba_eff(:) = wood_to_ba_eff(circ_class_biomass(ipts,j,:,:,icarbon),j)
             circ_class_circ_eff(:) = 2 * pi * SQRT(circ_class_ba_eff(:)/pi)

             ! According to equation (-) in Bellasen et al 2010. 
             ! ln(sigmas) = a_sig * ln(circ_med) + b_sig
             ! sigmas = exp(a_sig*log(median(circ_med))+b_sig);
             ! However, in the code (sapiens_forestry.f90) a different expression was used
             ! sigmas = 0.023+0.58*prctile(circ_med,0.05);
             ! Any of these implementations could work but seem to be more suited for 
             ! continues or nearly continuous diameter distributions, say n_circ > 10
             ! For a small number of diameter classes sigma depends on a prescribed
             ! circumference percentile.
             IF (ncirc .GE. 6) THEN          
               
                ! Calculate the median circumference
                DO l = 1,ncirc
                   
                      IF (SUM(circ_class_n(ipts,j,1:l)) .GE. 0.5 * ind(ipts,j)) THEN
                         
                         median_circ  = circ_class_circ_eff(l) - 5 * min_stomate
                      
                         EXIT
                      
                      ENDIF
                   
                ENDDO

                sigma(ipts,j) = deleuze_a(j) + deleuze_b(j) * median_circ
                
             ELSE
                
                ! The X percentile of the trees that will receive the photosynthates
                ! depends on the FM type. In a coppice stand there is a lot of 
                ! competition between the shoots and only the top half of the shoots
                ! will receive GPP, the other half receives only little GPP. This was 
                ! implemnted to get a reasonable diameter growth of coppice stands. 
                ! If deleuze_p is independent from FM, FM strategies with high densities
                ! have very slow diameter growth because the GPP has to be distributed 
                ! over a large number of individuals. 
                IF (forest_managed(ipts,j) == 3) THEN

                   deleuze_p(j) = deleuze_p_coppice(j)

                ELSEIF (forest_managed(ipts,j) == 1 .OR. &
                     forest_managed(ipts,j) == 2 .OR. forest_managed(ipts,j) == 4) THEN
                   
                   deleuze_p(j) = deleuze_p_all(j)

                ELSE
                   
                   WRITE(numout, *) 'forest management, ', forest_managed(ipts,j)
                   CALL ipslerr_p (3,'growth_fun_all', &
                        'Forest management strategy does not exist','','')

                ENDIF
                ! Search for the X percentile, where X is given by ::deleuze_p
                ! Substract a very small number (5*min_stomate) just to be sure that 
                ! the circ_class will be corectly accounted for in GE or LE statements
                DO l = 1,ncirc
                   
                      IF (SUM(circ_class_n(ipts,j,1:l)) .GE. deleuze_p(j) * ind(ipts,j)) THEN
                         
                         sigma(ipts,j) = circ_class_circ_eff(l) - 5 * min_stomate
                      
                         EXIT
                      
                      ENDIF
                   
                ENDDO
                
             ENDIF

          
             !! 5.2 Calculate allocated biomass pools for trees
             !  Only possible if there is biomass to allocate
             !  Use sigma and m_dv to calculate a single coefficient that can be
             !  used in the subsequent allocation scheme.
             circ_class_dba(:) = (circ_class_circ_eff(:) - m_dv(j)*sigma(ipts,j) + &
                  SQRT((m_dv(j)*sigma(ipts,j) + circ_class_circ_eff(:))**2 - &
                  (4*sigma(ipts,j)*circ_class_circ_eff(:)))) / 2

             !! 5.2.2 Scaling factor to convert variables to the individual plant
             !  Allocation is on an individual basis. Stand-level variables need to convert to a 
             !  single individual. Different approach between the DGVM and statitic approach
             IF (control%ok_dgvm) THEN

                ! The DGVM does currently NOT work with the new allocation, consider this as
                ! placeholder. The original code had two different transformations to 
                ! calculate the scalars. Both could be used but the units will differ.
                ! When fixing the DGVM check which quantities need to be multiplied by scal 
                ! scal = ind(ipts,j) * cn_ind(ipts,j) / veget_max(ipts,j)
                scal = veget_max(ipts,j) / ind(ipts,j)  
             
             ELSE
             
                ! circ_class_biomass contain the data at the tree level
                ! no conversion required
                scal = 1.
             
             ENDIF
                      

             !! 5.2.3 Current biomass pools per tree (gC tree^-1) 
             ! We will have different trees so this has to be calculated from the 
             ! diameter relationships            
             Cs(:) = ( circ_class_biomass(ipts,j,:,isapabove,icarbon) + &
                  circ_class_biomass(ipts,j,:,isapbelow,icarbon) ) * scal
             Cr(:) = circ_class_biomass(ipts,j,:,iroot,icarbon) * scal
             Cl(:) = circ_class_biomass(ipts,j,:,ileaf,icarbon) * scal
             Ch(:) = ( circ_class_biomass(ipts,j,:,iheartabove,icarbon) + &
                  circ_class_biomass(ipts,j,:,iheartbelow,icarbon) ) * scal
 
             ! Total amount of carbon that needs to ba allocated (::bm_alloc_tot). 
             ! bm_alloc_tot is in gC m-2 day-1. At 1 m2 there are ::ind number of 
             ! trees. We calculate the allocation for ::ncirc trees. Hence b_inc_tot 
             ! needs to be scaled in the allocation routines. For all cases were 
             ! allocation takes place for a single circumference class, scaling 
             ! could be done before the allocation. In the ordinary allocation 
             ! allocation takes place to all circumference classes at the same time. 
             ! Hence scaling takes place in that step for consistency we scale during 
             ! allocation. Note that b_inc (the carbon allocated to an individual 
             ! circumference class cannot be estimates at this point.
             b_inc_tot = bm_alloc_tot(ipts,j)
             

             !! 5.2.4 C-allocation for trees
             !  The mass conservation equations are detailed in the header of this subroutine.
             !  The scheme assumes a functional relationships between leaves, sapwood and 
             !  roots. When carbon is added to the leaf biomass pool, an increase in the root
             !  biomass is to be expected to sustain water transport from the roots to the 
             !  leaves. Also sapwood is needed to sustain this water transport and to support
             !  the leaves.
             DO l = 1,ncirc 

                !! 5.2.4.1 Calculate tree height
                circ_class_height_eff(l) = pipe_tune2(j)* & 
                     (4/pi*circ_class_ba_eff(l))**(pipe_tune3(j)/2)


                !! 5.2.4.2 Do the biomass pools respect the pipe model?
                !  Do the current leaf, sapwood and root components respect the allometric 
                !  constraints? Due to plant phenology it is possible that we have too much 
                !  sapwood compared to the leaf and root mass (i.e. in early spring). 
                !  Calculate the optimal root and leaf mass, given the current wood mass 
                !  by using the basic allometric relationships. Calculate the optimal sapwood
                !  mass as a function of the current leaf and root mass.
                Cl_target(l) = MAX( KF(ipts,j) * Cs(l) / circ_class_height_eff(l), &
                     Cr(l) * LF(ipts,j) , Cl(l))
                Cr_target(l) = MAX( Cl_target(l) / LF(ipts,j), &
                     Cs(l) * KF(ipts,j) / LF(ipts,j) / circ_class_height_eff(l) , Cr(l))
                Cs_target(l) = MAX( Cl(l) / KF(ipts,j) * circ_class_height_eff(l), &
                     Cr(l) * LF(ipts,j) / KF(ipts,j) * circ_class_height_eff(l) , Cs(l))

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'bm_alloc_tot, ', bm_alloc_tot(ipts,j)
                   WRITE(numout,*) 'Does the tree needs reshaping? Class: ',l
                   WRITE(numout,*) 'circ_class_height_eff, ', circ_class_height_eff(l)
                   WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
                   WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(l)-Cl(l), Cl_target(l), Cl(l)
                   WRITE(numout,*) 'Cs_target-Cs, ', Cs_target(l)-Cs(l), Cs_target(l), Cs(l)
                   WRITE(numout,*) 'Cr_target-Cr, ', Cr_target(l)-Cr(l), Cr_target(l), Cr(l)
                ENDIF
                !----------

                !! 5.2.4.2 Check dimensions of the trees
                ! If Cs = Cs_target then ba and height are correct, else calculate 
                ! the correct dimensions
                IF ( Cs_target(l) - Cs(l) .GT. min_stomate ) THEN

                   ! If Cs = Cs_target then dia and height are correct. However, 
                   ! if Cl = Cl_target or Cr = Cr_target then dia and height need 
                   ! to be re-estimated. Cs_target should satify the relationship 
                   ! Cl/Cs = KF/height where height is a function of Cs_target
                   ! <=> (KF*Cs_target)/(pipe_tune2*(Cs_target+Ch)/pi/4)**&
                   ! (pipe_tune3/(2+pipe_tune3)) = Cl_target. Search Cs needed to 
                   ! sustain the max of Cl or Cr. Search max of Cl and Cr first 
                   Cl_target(l) = MAX(Cl(l), Cr(l)*LF(ipts,j))
                   Cs_target(l) =  newX(KF(ipts,j), Ch(l), pipe_tune2(j), &
                        pipe_tune3(j), Cl_target(l), &
                        tree_ff(j)*pipe_density(j)*pi/4*pipe_tune2(j), &
                        Cs(l), 2*Cs(l), 2, j, ipts)

                   ! Recalculate height and ba from the correct Cs_target
                   circ_class_height_eff(l) = Cs_target(l)*KF(ipts,j)/Cl_target(l)
                   circ_class_ba_eff(l) = pi/4*(circ_class_height_eff(l)/ & 
                        pipe_tune2(j))**(2/pipe_tune3(j))
                   Cl_target(l) = KF(ipts,j) * Cs_target(l) / circ_class_height_eff(l)
                   Cr_target(l) = Cl_target(l) / LF(ipts,j)

                ENDIF

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'height_fin, ba_fin, ', circ_class_height_eff(:), &
                        circ_class_ba_eff(:)
                   WRITE(numout,*) 'Cl_target, Cs_target, Cr_target, ', Cl_target(:), &
                        Cs_target(:), Cr_target(:)
                   WRITE(numout,*) 'New target values'
                   WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(l)-Cl(l), Cl_target(l), Cl(l)
                   WRITE(numout,*) 'Cs_target-Cs, ', Cs_target(l)-Cs(l), Cs_target(l), Cs(l)
                   WRITE(numout,*) 'Cr_target-Cr, ', Cr_target(l)-Cr(l), Cr_target(l), Cr(l)
                ENDIF
                !-----------

             ENDDO

             ! The step estimate is used to linearalize the diameter vs height relationship.
             ! Use a prior to distribute b_inc_tot over the individual trees. The share of
             ! the total sapwood mass is used as a prior. Subsequently, estimate the change in 
             ! diameter by assuming all the available C for allocation will be used in Cs.  
             ! Hence, this represents the maximum possible diameter increase. It was not tested
             ! whether this is the best prior but it seems to work OK although it often results
             ! in very small (1e-8) negative values, with even more rare 1e-6 negative values. 
             ! A C-balance closure check could reveal 
             ! whether this is a real issue and requires to change the prior or not.
             ! Calculate the linear slope (::s) of the relationship between ba and h as 
             ! (1) s = (ba2-ba)/(height2-height). 
             ! The goal is to approximate the ba2 that
             ! is predicted through the non-linear ordinary allocation approach, as this will 
             ! keep the trees in allometric balance. In the next time step, allometric 
             ! balance is recalculated and can be corrected through the so-called phenological 
             ! growth; hence, small deviations resulting from the linearization will not 
             ! accumulate with time.
             ! Note that ba2 = ba + delta_ba and that height and ba are related as
             ! (2) height = k2*(4*ba/pi)**(k3/2)
             ! At this stage the only information we have is that there is b_inc_tot (gC m-2) 
             ! available for allocation. There are two obvious approximations both making use
             ! of the same assumption, i.e. that for the initial estimate of delta_ba height is
             ! constant. The first approximation is crude and assumes that all the available C
             ! is used in Cs_inc (thus Cs_inc = b_inc_tot / ind ). The second approximation,
             ! implemented here, makes use of the allometric rules and thus accounts for the
             ! knowledge that allocating one unit the sapwood comes with a cost in leaves and 
             ! roots thus:
             ! b_inc_temp = Cs_inc+Cl_inc+Cr_inc
             ! (3) <=> b_inc_temp ~= (Cs_inc_est+Cs) + KF*(Cs_inc_est+Cs)/H + ...
             !    KF/LF*(Cs_inc_est+Cs)/H - Cs - Cl - Cr
             ! b_inc_temp is the amount of carbon that can be allocated to each diameter class.
             ! However, only the total amount i.e. b_inc_tot is known. Total allocatable carbon
             ! is distributed over the different diameter classes proportional to their share
             ! of the total wood biomass. Divide by circ_class_n to get the correct units 
             ! (gC tree-1)
             ! (4)  b_inc_temp ~= b_inc_tot / circ_class_n * (circ_class_n * ba**(1+k3)) / ...
             !    sum(circ_class_n * ba**(1+k3))
             ! By substituting (4) in (3) an expression is obtained to approximate the carbon
             ! that will be allocated to sapwood growth per diameter class ::Cs_inc_est. This
             ! estimate is then used to calculate delta_ba (called ::step) as
             ! step = (Cs+Ch+Cs_inc_set)/(tree_ff*pipe_density*height) - ba where height is 
             ! calculated from (2) after replacing ba by ba+delta_ba
             
             !+++CHECK+++
             !Alternative decribed in the documentation - most complete
             Cs_inc_est(:) = ( b_inc_tot / circ_class_n(ipts,j,:) * &
                  (circ_class_n(ipts,j,:) * circ_class_ba_eff(:)**(un+pipe_tune3(j))) / &
                  (SUM(circ_class_n(ipts,j,:) * circ_class_ba_eff(:)**(un+pipe_tune3(j)))) + &
                  Cs(:) + Cl(:) + Cr(:)) * circ_class_height_eff(:) / &
                  (circ_class_height_eff(:) + KF(ipts,j) + KF(ipts,j)/LF(ipts,j)) - Cs(:)
             !Keep it simple
             !Cs_inc_est(:) = ( b_inc_tot / circ_class_n(ipts,j,:) * &
             !     (circ_class_n(ipts,j,:) * circ_class_ba_eff(:)**(un+pipe_tune3(j))) / &
             !     (SUM(circ_class_n(ipts,j,:) * circ_class_ba_eff(:)**(un+pipe_tune3(j)))))
             !+++++++++++

             step(:) = ((Ch(:)+Cs(:)+Cs_inc_est(:)) / (tree_ff(j)*pipe_density(j)* &
                  circ_class_height_eff(:))) - circ_class_ba_eff(:)

             !++++ CHECK ++++++
             ! It can happen that step is equal to zero sometimes.  I'm not sure why, but 
             ! there was a case where it was nonzero for circ classes 1 and 3, and zero 
             ! for 2.  This causes s to be zero and provokes a divide by zero error later 
             ! on.  What if we make it not zero?  This might cause a small mass balance
             ! error for this timestep, but I would rather have that than getting an 
             ! infinite biomass, which is what happened in the other case.  These limits 
             ! are arbitrary and adjusted by hand.  If the output file doesn't show this 
             ! warning very often, I think we're okay, since the amount of carbon is really
             ! small.
             DO l=1,ncirc
                IF(step(l) .LT. min_stomate*0.01 .AND. step(l) .GT. zero)THEN
                   step(l)=min_stomate*0.02
                   IF (ld_alloc) THEN
                      WRITE(numout,*) 'WARNING: Might cause mass balance problems in fun_all, position 1'
                      WRITE(numout,*) 'WARNING: ips,j ',ipts,j
                   END IF
                ELSEIF(step(l) .GT. -min_stomate*0.01 .AND. step(l) .LT. zero)THEN
                   step(l)=-min_stomate*0.02
                   IF (ld_alloc) THEN
                      WRITE(numout,*) 'WARNING: Might cause mass balance problems in fun_all, position 2'
                      WRITE(numout,*) 'WARNING: ips,j ',ipts,j
                   END IF
                ENDIF
             ENDDO
             !+++++++++++++++++
             s(:) = step(:)/(pipe_tune2(j)*(4.0_r_std/pi*(circ_class_ba_eff(:)+step(:)))**&
                  (pipe_tune3(j)/deux) - &
                  pipe_tune2(j)*(4.0_r_std/pi*circ_class_ba_eff(:))**(pipe_tune3(j)/deux))

             !! 5.2.4.3 Phenological growth
             !  Phenological growth and reshaping of the tree in line with the pipe model. 
             !  Turnover removes C from the different plant components but at a 
             !  component-specific rate, as such the allometric constraints are distorted 
             !  at every time step and should be restored before ordinary growth can 
             !  take place
             l = ncirc
             DO WHILE ( (l .GT. zero) .AND. (b_inc_tot .GT. min_stomate) )

                !! 5.2.4.3.1 The available wood can sustain the available leaves and roots
                !  Calculate whether the wood is in allometric balance. The target values 
                !  should always be larger than the current pools so the use of ABS is 
                !  redundant but was used to be on the safe side (here and in the rest 
                !  of the module) as it could help to find logical flaws.
                IF ( ABS(Cs_target(l) - Cs(l)) .LT. min_stomate ) THEN

                   ! Use the difference between the target and the actual to
                   ! ensure mass balance closure because l times a values 
                   ! smaller than min_stomate can still add up to a value 
                   ! exceeding min_stomate.
                   Cs_incp(l) = MAX(zero, Cs_target(l) - Cs(l))

                   ! Enough leaves and wood, only grow roots
                   IF ( ABS(Cl_target(l) - Cl(l))  .LT. min_stomate ) THEN

                      ! Allocate at the tree level to restore allometric balance
                      ! Some carbon may have been used for Cs_incp and Cl_incp
                      ! adjust the total allocatable carbon
                      Cl_incp(l) = MAX(zero, Cl_target(l) - Cl(l))
                      Cr_incp(l) = MAX( MIN(b_inc_tot / circ_class_n(ipts,j,l) - &
                           Cs_incp(l) - Cl_incp(l), Cr_target(l) - Cr(l)), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 1)
                      ENDIF

                   ! Sufficient wood and roots, allocate C to leaves
                   ELSEIF ( ABS(Cr_target(l) - Cr(l)) .LT. min_stomate ) THEN

                      ! Allocate at the tree level to restore allometric balance
                      ! Some carbon may have been used for Cs_incp and Cr_incp
                      ! adjust the total allocatable carbon
                      Cr_incp(l) = MAX(zero, Cr_target(l) - Cr(l))
                      Cl_incp(l) = MAX( MIN(b_inc_tot / circ_class_n(ipts,j,l) - &
                           Cs_incp(l) - Cr_incp(l), Cl_target(l) - Cl(l)), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                              grow_wood, circ_class_n, ind, 2)
                      ENDIF
                      
                   ! Both leaves and roots are needed to restore the allometric relationships
                   ELSEIF ( ABS(Cl_target(l) - Cl(l)) .GT. min_stomate .AND. &
                        ABS(Cr_target(l) - Cr(l)) .GT. min_stomate ) THEN                 

                      ! Allocate at the tree level to restore allometric balance
                      !  The equations can be rearanged and written as
                      !  (i) b_inc = Cl_inc + Cr_inc
                      !  (ii) Cr_inc = (Cl_inc+Cl)/LF - Cr
                      !  Substitue (ii) in (i) and solve for Cl_inc
                      !  <=> Cl_inc = (LF*(b_inc+Cr)-Cl)/(1+LF)
                      Cl_incp(l) = MIN( ((LF(ipts,j) * ((b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cs_incp(l)) + Cr(l))) - Cl(l)) / & 
                           (1 + LF(ipts,j)), Cl_target(l) - Cl(l) )
                      Cr_incp(l) = MIN ( ((Cl_incp(l) + Cl(l)) / LF(ipts,j)) - Cr(l), &
                           Cr_target(l) - Cr(l))

                      ! The imbalance between Cr and Cl can be so big that (Cl+Cl_inc)/LF 
                      ! is still less then the available root carbon (observed!). This would 
                      ! result in a negative Cr_incp
                      IF ( Cr_incp(l) .LT. zero ) THEN

                         Cl_incp(l) = MIN( b_inc_tot/circ_class_n(ipts,j,l) - Cs_incp(l), &
                              Cl_target(l) - Cl(l) )
                         Cr_incp(l) = (b_inc_tot/circ_class_n(ipts,j,l)) - Cs_incp(l) - &
                              Cl_incp(l)

                      ELSEIF (Cl_incp(l) .LT. zero) THEN

                         Cr_incp(l) = MIN( b_inc_tot/circ_class_n(ipts,j,l) - Cs_incp(l), &
                              Cr_target(l) - Cr(l) )
                         Cl_incp(l) = (b_inc_tot/circ_class_n(ipts,j,l)) - &
                              Cs_incp(l) - Cr_incp(l)

                      ENDIF                          

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                              grow_wood, circ_class_n, ind, 3)
                      ENDIF      

                   ELSE

                      WRITE(numout,*) 'Exc 1-3: unexpected exception' 
                      IF(ld_stop)THEN
                         CALL ipslerr_p (3,'growth_fun_all',&
                              'Exc 1-3: unexpected exception','','')
                      ENDIF

                   ENDIF

                !! 5.2.4.3.2 Enough leaves to sustain the wood and roots
                ELSEIF ( ABS(Cl_target(l) - Cl(l)) .LT. min_stomate ) THEN

                   ! Use the difference between the target and the actual to
                   ! ensure mass balance closure because l times a values 
                   ! smaller than min_stomate can still add up to a value 
                   ! exceeding min_stomate.
                   Cl_incp(l) = MAX(zero, Cl_target(l) - Cl(l))

                   ! Enough leaves and wood, only grow roots
                   ! This duplicates Exc 1 and these lines should never be called 
                   IF ( ABS(Cs_target(l) - Cs(l)) .LT. min_stomate ) THEN

                      ! Allocate at the tree level to restore allometric balance
                      ! Some carbon may have been used for Cs_incp and Cl_incp
                      ! adjust the total allocatable carbon
                      Cs_incp(l) = MAX(zero, ABS(Cs_target(l) - Cs(l)))
                      Cr_incp(l) = MAX( MIN(b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cl_incp(l) - Cs_incp(l), Cr_target(l) - Cr(l)), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 4)
                      ENDIF

                   ! Enough leaves and roots. Need to grow sapwood to support the available 
                   ! canopy and roots
                   ELSEIF ( ABS(Cr_target(l) - Cr(l)) .LT. min_stomate ) THEN

                      ! In truth, there might be a little root carbon to allocate here,
                      ! since min_stomate is not equal to zero.  If there is
                      ! enough of this small carbon in every circ class, and there
                      ! are enough circ classes, ordinary allocation will be skipped
                      ! below and we might try to force allocation, which is silly
                      ! if the different in the root masses is around 1e-8. This
                      ! means we will allocate a tiny amount to the roots to make
                      ! sure they are exactly in balance.                  
                      Cr_incp(l) = MAX(zero, ABS(Cr_target(l) - Cr(l)))
                      Cs_incp(l) = MAX( MIN(b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cl_incp(l) - Cr_incp(l), Cs_target(l) - Cs(l)), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                              grow_wood, circ_class_n, ind, 5)
                      ENDIF                     

                   ! Need both wood and roots to restore the allometric relationships
                   ELSEIF ( ABS(Cs_target(l) - Cs(l) ) .GT. min_stomate .AND. &
                        ABS(Cr_target(l) - Cr(l)) .GT. min_stomate ) THEN

                      ! circ_class_ba_eff and circ_class_height_eff are already calculated
                      ! for a tree in balance. It would be rather complicated to follow
                      ! the allometric rules for wood allocation (implying changes in height 
                      ! and basal area) because the tree is not in balance yet. First try 
                      ! if we can simply satisfy the allocation needs
                      IF (Cs_target(l) - Cs(l) + Cr_target(l) - Cr(l) .LE. &
                           b_inc_tot/circ_class_n(ipts,j,l) - Cl_incp(l)) THEN
                         
                         Cr_incp(l) = Cr_target(l) - Cr(l)
                         Cs_incp(l) = Cs_target(l) - Cs(l)

                      ! Try to satisfy the need for roots
                      ELSEIF (Cr_target(l) - Cr(l) .LE. b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cl_incp(l)) THEN

                         Cr_incp(l) = Cr_target(l) - Cr(l)
                         Cs_incp(l) = b_inc_tot/circ_class_n(ipts,j,l) - &
                              Cl_incp(l) - Cr_incp(l)
                         
                      ! There is not enough use whatever is available
                      ELSE
                         
                         Cr_incp(l) = b_inc_tot/circ_class_n(ipts,j,l) - Cl_incp(l)
                         Cs_incp(l) = zero
                         
                      ENDIF

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 6)
                      ENDIF   

                   ELSE

                      WRITE(numout,*) 'Exc 4-6: unexpected exception'
                      IF(ld_stop)THEN
                         CALL ipslerr_p (3,'growth_fun_all',&
                              'Exc 4-6: unexpected exception','','')
                      ENDIF
                      
                   ENDIF


                !! 5.2.4.3.3 Enough roots to sustain the wood and leaves
                ELSEIF ( ABS(Cr_target(l) - Cr(l)) .LT. min_stomate ) THEN

                   ! Use the difference between the target and the actual to
                   ! ensure mass balance closure because l times a values 
                   ! smaller than min_stomate can still add up to a value 
                   ! exceeding min_stomate.
                   Cr_incp(l) = MAX(zero, Cr_target(l) - Cr(l))

                   ! Enough roots and wood, only grow leaves
                   ! This duplicates Exc 2 and these lines should thus never be called 
                   IF ( ABS(Cs_target(l) - Cs(l)) .LT. min_stomate ) THEN

                      ! Allocate at the tree level to restore allometric balance
                      Cs_incp(l) = MAX(zero, Cs_target(l) - Cs(l))
                      Cl_incp(l) = MAX( MIN(b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cs_incp(l) - Cr_incp(l), &
                           Cl_target(l) - Cl(l)), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 7)
                      ENDIF

                   ! Enough leaves and roots. Need to grow sapwood to support the 
                   ! available canopy and roots. Duplicates Exc. 4 and these lines 
                   ! should thus never be called 
                   ELSEIF ( ABS(Cl_target(l) - Cl(l)) .LT. min_stomate ) THEN

                      ! Allocate at the tree level to restore allometric balance
                      Cl_incp(l) = MAX(zero, Cl_target(l) - Cl(l))
                      Cs_incp(l) = MAX( MIN(b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cr_incp(l) - Cl_incp(l), Cs_target(l) - Cs(l) ), zero )

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 8)
                      ENDIF

                   ! Need both wood and leaves to restore the allometric relationships
                   ELSEIF ( ABS(Cs_target(l) - Cs(l)) .GT. min_stomate .AND. &
                        ABS(Cl_target(l) - Cl(l)) .GT. min_stomate ) THEN

                      ! circ_class_ba_eff and circ_class_height_eff are already calculated
                      ! for a tree in balance. It would be rather complicated to follow
                      ! the allometric rules for wood allocation (implying changes in height 
                      ! and basal area) because the tree is not in balance.First try if we 
                      ! can simply satisfy the allocation needs
                      IF (Cs_target(l) - Cs(l) + Cl_target(l) - Cl(l) .LE. &
                           b_inc_tot/circ_class_n(ipts,j,l) - Cr_incp(l)) THEN

                         Cl_incp(l) = Cl_target(l) - Cl(l)
                         Cs_incp(l) = Cs_target(l) - Cs(l)

                      ! Try to satisfy the need for leaves
                      ELSEIF (Cl_target(l) - Cl(l) .LE. b_inc_tot/circ_class_n(ipts,j,l) - &
                           Cr_incp(l)) THEN

                         Cl_incp(l) = Cl_target(l) - Cl(l)
                         Cs_incp(l) = b_inc_tot/circ_class_n(ipts,j,l) - &
                              Cr_incp(l) - Cl_incp(l)

                      ! There is not enough use whatever is available
                      ELSE

                         Cl_incp(l) = b_inc_tot/circ_class_n(ipts,j,l) - Cr_incp(l)
                         Cs_incp(l) = zero

                      ENDIF

                      ! Write debug comments to output file
                      IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                         CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                              delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                              KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                              circ_class_n, ind, 9)
                      ENDIF

                   ELSE

                      WRITE(numout,*) 'Exc 7-9: unexpected exception'
                      IF(ld_stop)THEN
                         CALL ipslerr_p (3,'growth_fun_all',&
                              'Exc 7-9: unexpected exception','','')
                      ENDIF

                   ENDIF

                ! Either Cl_target, Cs_target or Cr_target should be zero
                ELSE

                   ! Something possibly important was overlooked
                   WRITE(numout,*) 'WARNING 4: logical flaw in the phenological allocation, PFT, class: ', j, l
                   WRITE(numout,*) 'WARNING 4: PFT, ipts: ',j,ipts
                   WRITE(numout,*) 'Cs - Cs_target', Cs(l), Cs_target(l)
                   WRITE(numout,*) 'Cl - Cl_target', Cl(l), Cl_target(l)
                   WRITE(numout,*) 'Cr - Cr_target', Cr(l), Cr_target(l)
                   IF(ld_stop)THEN
                       CALL ipslerr_p (3,'growth_fun_all',&
                            'WARNING 4: logical flaw in the phenological allocation','','')
                   ENDIF

                ENDIF

                !! 5.2.4.4 Wrap-up phenological allocation
                IF ( Cl_incp(l) .GE. zero .OR. Cr_incp(l) .GE. zero .OR. &
                     Cs_incp(l) .GE. zero) THEN

                   ! Fake allocation for less messy equations in next case, 
                   ! incp needs to be added to inc at the end
                   Cl(l) = Cl(l) + Cl_incp(l)
                   Cr(l) = Cr(l) + Cr_incp(l)
                   Cs(l) = Cs(l) + Cs_incp(l)
                   b_inc_tot = b_inc_tot - (circ_class_n(ipts,j,l) * &
                        (Cl_incp(l) + Cr_incp(l) + Cs_incp(l)))             

                   ! Something is wrong with the calculations
                   IF (b_inc_tot .LT. -min_stomate) THEN

                      WRITE(numout,*) 'WARNING 5: numerical problem, overspending in phenological allocation'
                      WRITE(numout,*) 'WARNING 5: PFT, ipts: ',j,ipts
                      CALL ipslerr_p (3,'growth_fun_all',&
                           'WARNING 5: numerical problem, overspending in phenological allocation','','')
                   ENDIF

                ELSE

                   ! The code was written such that the increment pools should be 
                   ! greater than or equal to zero. If this is not the case, something 
                   ! fundamental is wrong with the if-then constructs under §5.2.4.3
                   WRITE(numout,*) 'WARNING 6: PFT, ipts: ',j,ipts
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 6: numerical problem, one of the increment pools is less than zero','','')
                ENDIF

                ! Set counter for next circumference class
                l = l-1

             ENDDO ! DO WHILE l.GE.1 .AND. b_inc_tot .GT. min_stomate


             !! 5.2.5 Calculate the expected size of the reserve pool 
             !  use the minimum of either (1) 2% of the total sapwood biomass or 
             !  (2) the amount of carbon needed to develop the optimal LAI and the roots
             !  This reserve pool estimate is only used to decide whether wood should be
             !  grown or not. When really dealing with the reserves the reserve pool is 
             !  recalculated. See further below §7.1. 
             reserve_pool = MIN( 0.02 * ( biomass(ipts,j,isapabove,icarbon) + &
                  biomass(ipts,j,isapbelow,icarbon)), &
                  lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j))) 
             grow_wood = .TRUE.

             ! If the carbohydrate pool is too small, don't grow wood
             IF ( (pheno_type(j) .NE. 1) .AND. &
                  (biomass(ipts,j,icarbres,icarbon) .LE. reserve_pool) ) THEN

                grow_wood = .FALSE.

             ENDIF

             ! Write debug comments to output file
             IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                     delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                     KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                     circ_class_n, ind, 10)
             ENDIF


             !! 5.2.6 Ordinary growth
             !  Allometric relationship between components is respected, sustain 
             !  ordinary growth and allocate
             !  biomass to leaves, wood, roots and fruits.
             IF ( (SUM( ABS(Cl_target(:) - Cl(:)) ) .LE. min_stomate) .AND. &
                  (SUM( ABS(Cs_target(:) - Cs(:)) ) .LE. min_stomate) .AND. &
                  (SUM( ABS(Cr_target(:) - Cr(:)) ) .LE. min_stomate) .AND. &
                  (grow_wood) .AND. (b_inc_tot .GT. min_stomate) ) THEN

                ! Allocate fraction of carbon to fruit production (at the tree level)
                Cf_inc(:) = b_inc_tot / SUM(circ_class_n(ipts,j,:)) * fruit_alloc(j)

                ! Residual carbon is allocated to the other components (b_inc_tot is
                ! at the stand level)
                b_inc_tot = b_inc_tot * (un-fruit_alloc(j))

                ! Substitute (7), (8) and (9) in (1) 
                ! b_inc = tree_ff*pipe_density*(ba+circ_class_dba*gammas)*...
                ! (height+(circ_class_dba/s*gammas)) - Cs - Ch + ...
                !    KF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - ... 
                !    (KF*Ch)/(height+(circ_class_dba/s*gammas)) - Cl + ...
                !    KF/LF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - ...
                !    (KF*Ch/LF)/(height+(circ_class_dba/s*gammas)) - Cr
                !
                ! b_inc+Cs+Ch+Cl+Cr = tree_ff*pipe_density*(ba+circ_class_dba*gammas)*...
                !    (height+(circ_class_dba/s*gammas))  + ...
                !    KF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - ...
                !    (KF*Ch)/(height+(circ_class_dba/s*gammas)) + ...
                !    KF/LF*tree_ff*pipe_density*(ba+circ_class_dba*gammas) - ...
                !    (KF*Ch/LF)/(height+(circ_class_dba/s*gammas))
                ! <=> b_inc+Cs+Ch+Cl+Cr = circ_class_dba^2/s*tree_ff*...
                !    pipe_density*gammas^2 + circ_class_dba/s*ba*tree_ff*...
                !    pipe_density*gammas + ...
                !    circ_class_dba*height*tree_ff*pipe_density*gammas + ...
                !    bcirc_class_dba*height*tree_ff*pipe_density - ...
                !    (Ch*KF*s)/(circ_class_dba*gammas+height*s) + ...
                !    circ_class_dba*KF*tree_ff*pipe_density*gammas + ...
                !    ba*KF*tree_ff*pipe_density - ...
                !    (Ch*KF*s)/(LF*(circ_class_dba*gammas+height*s)) + ...
                !    circ_class_dba*KF/LF*tree_ff*pipe_density*gammas + ...
                !    ba*KF/LF*tree_ff*pipe_density
                ! (10) b_inc+Cs+Ch+Cl+Cr = (circ_class_dba^2/s*tree_ff*...
                !    pipe_density)*gammas^2 + ...
                !    (circ_class_dba/s*ba*tree_ff*pipe_density + ...
                !    circ_class_dba*height*tree_ff*pipe_density + ...
                !    circ_class_dba*KF*tree_ff*pipe_density + ...
                !    circ_class_dba*KF/LF*tree_ff*pipe_density)*gammas - ...
                !    (Ch*KF*s)(1+1/LF)/(circ_class_dba*gammas+height*s) + ...
                !    bcirc_class_dba*height*tree_ff*pipe_density + ...
                !    ba*KF*tree_ff*pipe_density + ba*KF/LF*tree_ff*pipe_density
                !
                ! Note that b_inc is not known, only b_inc_tot (= sum(b_inc) is known. 
                ! The above equations are for individual trees, at the stand level we 
                ! have to take the sum over the individuals which is 
                ! equivalant to substituting (10) in (2) 
                ! (11) sum(b_inc) + sum(Cs+Ch+Cl+Cr) = ...
                !    sum(circ_class_dba^2/s*tree_ff*pipe_density) * gammas^2 + ...
                !    sum(circ_class_dba/s*ba*tree_ff*pipe_density + ...
                !    circ_class_dba*height*tree_ff*pipe_density + ...
                !    circ_class_dba*KF*tree_ff*pipe_density + ...
                !    circ_class_dba*KF/LF*tree_ff*pipe_density) * gammas - ...
                !    sum[(Ch*KF*s)(1+1/LF)/(circ_class_dba*gammas+height*s)] + ...
                !    sum(bcirc_class_dba*height*tree_ff*pipe_density + ...
                !    ba*KF*tree_ff*pipe_density + ba*KF/LF*tree_ff*pipe_density)
                !
                ! The term sum[(Ch*KF*s)(1+1/LF)/(circ_class_dba*gammas+height*s)] 
                ! can be approximated by a series expansion
                ! (12) sum((Ch*KF*s)(1+1/LF)/(height*s) + ...
                !    sum((Ch*KF*s)(1+1/LF)*circ_class_dba/(height*s)^2)*gammas + ...
                !    sum((Ch*KF*s)(1+1/LF)*circ_class_dba^2/(height*s)^3)*gammas^2
                !
                ! Substitute (12) in (11)
                ! sum(b_inc) + sum(Cs+Ch+Cl+Cr) = ...
                !    sum(circ_class_dba^2/s*tree_ff*pipe_density - ...
                !    (Ch*KF*s)*(1+1/LF)*circ_class_dba^2/(height*s)^3) * gammas^2 + ...
                !    sum(circ_class_dba/s*ba*tree_ff*pipe_density + ...
                !    circ_class_dba*height*tree_ff*pipe_density + ...
                !    circ_class_dba*KF*tree_ff*pipe_density + ...
                !    circ_class_dba*KF/LF*tree_ff*pipe_density + ...
                !    (Ch*KF*s)*(1+1/LF)*circ_class_dba/(height*s)^2) * gammas + ...
                !    sum(bcirc_class_dba*height*tree_ff*pipe_density + ...
                !    ba*KF*tree_ff*pipe_density + ba*KF/LF*tree_ff*pipe_density - ...
                !    (Ch*KF*s)*(1+1/LF)/(height*s))
                !
                ! Solve this quadratic equation for gammas.
                a = SUM( circ_class_n(ipts,j,:) * &
                     (circ_class_dba(:)**2/s(:)*tree_ff(j)*pipe_density(j) - &
                     (Ch(:)*KF(ipts,j)*s(:))*(1+1/LF(ipts,j))*&
                     (circ_class_dba(:)**2/(circ_class_height_eff(:)*s(:))**3)) )
                b = SUM( circ_class_n(ipts,j,:) * &
                     (circ_class_dba(:)/s(:)*circ_class_ba_eff(:)*tree_ff(j)*pipe_density(j) + &
                     circ_class_dba(:)*circ_class_height_eff(:)*tree_ff(j)*pipe_density(j) + &
                     circ_class_dba(:)*KF(ipts,j)*tree_ff(j)*pipe_density(j) + &
                     circ_class_dba(:)*KF(ipts,j)/LF(ipts,j)*tree_ff(j)*pipe_density(j) + &
                     (Ch(:)*KF(ipts,j)*s(:))*(1+1/LF(ipts,j))*circ_class_dba(:)/&
                     (circ_class_height_eff(:)*s(:))**2) )
                c = SUM( circ_class_n(ipts,j,:) * &
                     (circ_class_ba_eff(:)*circ_class_height_eff(:)*&
                     tree_ff(j)*pipe_density(j) + &
                     circ_class_ba_eff(:)*KF(ipts,j)*tree_ff(j)*pipe_density(j) + &
                     circ_class_ba_eff(:)*KF(ipts,j)/LF(ipts,j)*tree_ff(j)*pipe_density(j) - &
                     (Ch(:)*KF(ipts,j)*s(:))*(1+1/LF(ipts,j))/&
                     (circ_class_height_eff(:)*s(:)) - &
                     (Cs(:) + Ch(:) + Cl(:) + Cr(:))) ) - b_inc_tot

                ! Solve the quadratic equation a*gammas2 + b*gammas + c = 0, for gammas.
                gammas(ipts,j) = (-b + sqrt(b**2-4*a*c)) / (2*a)  

                !++++++ TEMP ++++++
!!$                IF(test_pft == j .AND. test_grid ==ipts)THEN
!!$                   WRITE(numout,*) 'Testing for the slope'
!!$                   DO i=1,100
!!$                      tempi=1
!!$                      delta_ba(tempi) = circ_class_dba(tempi) * gammas * REAL(i)/50.0
!!$                      delta_height(tempi) = delta_ba(tempi)/s(tempi)
!!$                      Cs_inc(tempi) = tree_ff(j)*pipe_density(j)*(circ_class_ba_eff(tempi) + delta_ba(tempi))*(circ_class_height_eff(tempi) + &
!!$                           delta_height(tempi)) - Cs(tempi) - Ch(tempi)
!!$                      Cl_inc(tempi) = KF(ipts,j)*tree_ff(j)*pipe_density(j)*(circ_class_ba_eff(tempi)+delta_ba(tempi)) - &
!!$                           (KF(ipts,j)*Ch(tempi))/(circ_class_height_eff(tempi)+delta_height(tempi)) - Cl(tempi)
!!$                      Cr_inc(tempi) = KF(ipts,j)/LF(ipts,j)*tree_ff(j)*pipe_density(j)*(circ_class_ba_eff(tempi)+delta_ba(tempi)) - &
!!$                           (KF(ipts,j)*Ch(tempi)/LF(ipts,j))/(circ_class_height_eff(tempi)+delta_height(tempi)) - Cr(tempi)    
!!$                      WRITE(numout,'(10F20.10)') delta_height(tempi),delta_ba(tempi),Cs_inc(tempi),Cl_inc(tempi),Cr_inc(tempi)
!!$                   ENDDO
!!$                   WRITE(numout,*) 'End testing for the slope'
!!$                END IF
                !+++++++++++++++++
                
                ! The solution for gammas is then used to calculate delta_ba (eq. 3), 
                ! delta_height (eq. 6), Cs_inc (eq. 7), Cl_inc (eq. 8) and Cr_inc (eq. 9). 
                ! See comment on the calculation of delta_height and its implications on 
                ! numerical consistency at the similar statement in §5.2.4.3.1
                delta_ba(:) = circ_class_dba(:) * gammas(ipts,j)
                store_delta_ba(ipts,j,:) = delta_ba(:)
                delta_height(:) = delta_ba(:)/s(:)              
                Cs_inc(:) = tree_ff(j)*pipe_density(j)*(circ_class_ba_eff(:) + &
                     delta_ba(:))*(circ_class_height_eff(:) + &
                     delta_height(:)) - Cs(:) - Ch(:)
                Cl_inc(:) = KF(ipts,j)*tree_ff(j)*pipe_density(j)*&
                     (circ_class_ba_eff(:)+delta_ba(:)) - &
                     (KF(ipts,j)*Ch(:))/(circ_class_height_eff(:)+delta_height(:)) - Cl(:)
                Cr_inc = KF(ipts,j)/LF(ipts,j)*tree_ff(j)*pipe_density(j)*&
                     (circ_class_ba_eff(:)+delta_ba(:)) - &
                     (KF(ipts,j)*Ch(:)/LF(ipts,j))/(circ_class_height_eff(:)+&
                     delta_height(:)) - Cr(:)

                ! After thousands of simulation years we had a single pixel where 
                ! one of the three circ_class got a negative growth. The cause is not
                ! is not entirely clear but could be related to the fact that KF
                ! changes from one day to another in combination with a low b_inc.
                ! If this happens, we don't allocate and simply leave the carbon
                ! in the labile pool. We will try again with more carbon the next day.
                ! This case is dealt with later in the code - see warning 10
                

                ! Write debug comments to output file
                IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                   WRITE(numout,*) 'a, b, c, gammas, ', a, b, c, gammas(ipts,j)
                   WRITE(numout,*) 'delta_height, ', delta_height(:)
                   CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                        delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                        KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                        circ_class_n, ind, 11)
                ENDIF

                ! Wrap-up ordinary growth  
                ! Calculate C that was not allocated, note that Cf_inc was already substracted
                b_inc_tot = b_inc_tot - &
                     SUM(circ_class_n(ipts,j,:)*(Cl_inc(:) + Cr_inc(:) + Cs_inc(:)))

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'wrap-up ordinary allocation, left b_in_tot, ', b_inc_tot 
                ENDIF
                !----------


             !! 5.2.7 Don't grow wood, use C to fill labile pool
             ELSEIF ( (.NOT. grow_wood) .AND. (b_inc_tot .GT. min_stomate) ) THEN

                ! Calculate the C that needs to be distributed to the 
                ! labile pool. The fraction is proportional to the ratio 
                ! between the total allocatable biomass and the unallocated 
                ! biomass per tree (b_inc now contains the unallocated 
                ! biomass). At the end of the allocation scheme bm_alloc_tot 
                ! is substracted from the labile biomass pool to update the 
                ! biomass pool (biomass(:,:,ilabile) = biomass(:,:,ilabile) - 
                ! bm_alloc_tot(:,:)). At that point, the scheme puts the 
                ! unallocated b_inc into the labile pool. What we 
                ! want is that the unallocated fraction is removed from 
                ! ::bm_alloc_tot such that only the allocated C is removed 
                ! from the labile pool. b_inc_tot will be moved back into
                ! the labile pool in 5.2.11
                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) - b_inc_tot
                biomass(ipts,j,ilabile,icarbon) = &
                     biomass(ipts,j,ilabile,icarbon) + b_inc_tot 

                ! Wrap-up ordinary growth  
                ! Calculate C that was not allocated (b_inc_tot), the 
                ! equation should read b_inc_tot = b_inc_tot - b_inc_tot 
                ! note that Cf_inc was already substracted
                b_inc_tot = zero 

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'No wood growth, move remaining C to labile pool'
                   WRITE(numout,*) 'bm_alloc_tot_new, ',bm_alloc_tot(ipts,j)
                   WRITE(numout,*) 'wrap-up ordinary allocation, left b_inc_tot, ', b_inc_tot 
                ENDIF
                !---------- 


             !! 5.2.8 Error - the allocation scheme is overspending 
             ELSEIF (b_inc_tot .LT. min_stomate) THEN

                IF (b_inc_tot .LT. -min_stomate) THEN

                   ! Something is wrong with the calculations
                   WRITE(numout,*) 'WARNING 7: numerical problem overspending in ordinary allocation'
                   WRITE(numout,*) 'WARNING 7: PFT, ipts: ',j,ipts
                   WRITE(numout,*) 'WARNING 7: b_inc_tot', b_inc_tot
                   IF(ld_stop)THEN
                      CALL ipslerr_p (3,'growth_fun_all',&
                           'WARNING 7: numerical problem overspending in ordinary allocation','','')
                   ENDIF

                ELSE

                   IF (j.EQ.test_pft .AND. ld_alloc) THEN

                      ! Succesful allocation
                      WRITE(numout,*) 'Successful allocation'

                   ENDIF

                ENDIF

                ! Althought the biomass components respect the allometric relationships, there
                ! is no carbon left to allocate                      
                b_inc_tot = zero
                Cl_inc(:) = zero
                Cs_inc(:) = zero
                Cr_inc(:) = zero
                Cf_inc(:) = zero

             ENDIF ! Ordinary allocation

             !! 5.2.9 Forced allocation
             !  Although this should not happen, in case the functional allocation did not 
             !  consume all the allocatable carbon, the remaining C is left for the next day, 
             !  and some of the biomass is used to produce fruits (tuned). The numerical 
             !  precision of the allocation scheme (i.e. the linearisation) is similar to 
             !  min_stomate (i.e. 10-8) resulting in 'false' warnings. In the latter case 
             !  forced allocation is applied but only for very small amounts of carbon 
             !  i.e. between 10-5 and 10-8.  
             IF ( b_inc_tot .GT. min_stomate) THEN

                WRITE(numout,*) 'WARNING 8: b_inc_tot greater than min_stomate force allocation'
                WRITE(numout,*) 'WARNING 8: PFT, ipts: ',j,ipts
                WRITE(numout,*) 'WARNING 8: b_inc_tot, ', b_inc_tot
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 8: b_inc_tot greater than min_stomate force allocation','','')
                ENDIF

                !+++CHECK+++
                ! We should not end-up here. We need some code to break the conditions 
                ! that made us end-up here. The current code will do this job.
!!$                ! Calculate fraction that will be allocated to fruit. The fraction is proportional to the 
!!$                ! ratio between the total allocatable biomass and the unallocated biomass per tree
!!$                frac = fruit_alloc(j) * MIN(1., bm_alloc_tot(ipts,j) / b_inc_tot)  
!!$                Cf_inc(:) = Cf_inc(:) + b_inc_tot * frac
!!$                b_inc_tot = b_inc_tot * (1 - frac)
!!$
!!$                ! Calculate the C that needs to be distributed to the labile pool. The fraction is proportional 
!!$                ! to the ratio between the total allocatable biomass and the unallocated biomass per tree (b_inc 
!!$                ! now contains the unallocated biomass). At the end of the allocation scheme bm_alloc_tot is 
!!$                ! substracted from the labile biomass pool to update the biomass pool (biomass(:,:,ilabile) = 
!!$                ! biomass(:,:,ilabile,icarbon) - bm_alloc_tot(:,:)). At that point, the scheme puts the 
!!$                ! unallocated b_inc into the labile pool.
!!$                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) * &
!!$                     ( 1. - (1.-frac) * b_inc_tot / bm_alloc_tot(ipts,j) )
                !+++++++++++

             ELSEIF ( (b_inc_tot .LT. min_stomate) .AND. (b_inc_tot .GE. -min_stomate) ) THEN

                ! Successful allocation
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'Successful allocation'
                ENDIF

             ELSE

                ! Something possibly important was overlooked
                IF ( (b_inc_tot .LT. 100*min_stomate) .AND. (b_inc_tot .GE. -100*min_stomate) ) THEN
                   IF (j.EQ.test_pft .AND. ld_alloc) THEN
                      WRITE(numout,*) 'Marginally successful allocation - precision better than 10-6'
                      WRITE(numout,*) 'PFT, b_inc_tot', j, b_inc_tot
                   ENDIF
                ELSE
                   WRITE(numout,*) 'WARNING 9: Logical flaw unexpected result in the ordinary allocation'
                   WRITE(numout,*) 'WARNING 9: b_inc_tot, ',b_inc_tot
                   WRITE(numout,*) 'WARNING 9: PFT, ipts: ',j,ipts
                  IF(ld_stop)THEN
                     CALL ipslerr_p (3,'growth_fun_all',&
                          'WARNING 9: Logical flaw unexpected result in the ordinary allocation','','')
                  ENDIF
                ENDIF

             ENDIF

             ! The second problem we need to catch is when one of the increment pools is 
             ! negative. This is an undesired outcome (see comment where ::KF_old is 
             ! calculated in this routine. In that case we write a warning, set all increment
             ! pools to zero and try it again at the next time step. A likely cause of this 
             ! problem is a too large change in KF from one time step to another. Try decreasing
             ! the acceptable value for an absolute increase in KF.
             IF (MINVAL(Cs_inc(:)) .LT. zero .OR. MINVAL(Cr_inc(:)) .LT. zero .OR. &
                  MINVAL(Cs_inc(:)) .LT. zero) THEN

                ! Do not allocate - save the carbon for the next time step
                WRITE(numout,*) 'WARNING 10: numerical problem, one of the increment pools is less than zero'
                WRITE(numout,*) 'WARNING 10: PFT, ipts: ',j,ipts
                WRITE(numout,*) 'WARNING 10: Cl_inc(:): ',Cl_inc(:)
                WRITE(numout,*) 'WARNING 10: Cr_inc(:): ',Cr_inc(:)
                WRITE(numout,*) 'WARNING 10: Cs_inc(:): ',Cs_inc(:)
                WRITE(numout,*) 'WARNING 10: PFT, ipts: ',j,ipts
                WRITE(numout,*) 'WARNING 10: We will undo the allocation'
                WRITE(numout,*) ' and save the carbon for the next day'

                ! Reverse the allocation
                
                b_inc_tot = b_inc_tot + &
                     SUM(circ_class_n(ipts,j,:)*(Cl_inc(:) + Cr_inc(:) + Cs_inc(:)))
                Cl_inc(:) = zero
                Cr_inc(:) = zero
                Cs_inc(:) = zero

             ENDIF

             !! 5.2.10 Wrap-up phenological and ordinary allocation
             Cl_inc(:) = Cl_inc(:) + Cl_incp(:)
             Cr_inc(:) = Cr_inc(:) + Cr_incp(:)
             Cs_inc(:) = Cs_inc(:) + Cs_incp(:)
             residual(ipts,j) = b_inc_tot

             !---TEST---
             IF (j.EQ.test_pft .AND. ld_alloc) THEN
                WRITE(numout,*) 'Final allocation', ipts, j
                WRITE(numout,*) 'Cl, Cs, Cr', Cl(:), Cs(:), Cr(:) 
                WRITE(numout,*) 'Cl_incp, Cs_incp, Cr_incp, ', Cl_incp(:), Cs_incp(:), Cr_incp(:)
                WRITE(numout,*) 'Cl_inc, Cs_ins, Cr_inc, Cf_inc, ', Cl_inc(:), Cs_inc(:), Cr_inc(:), Cf_inc(:)
                WRITE(numout,*) 'unallocated/residual, ', b_inc_tot
                WRITE(numout,*) 'Old ba, delta_ba, new ba, ', circ_class_ba_eff(:), delta_ba(:), circ_class_ba_eff(:)+delta_ba(:)
                DO l=1,ncirc
                   WRITE(numout,*) 'Circ_class_biomass, ',circ_class_biomass(ipts,j,l,:,icarbon)
                ENDDO
             ENDIF
             !----------


             !! 5.2.11 Account for the residual
             !  The residual is usually around ::min_stomate but we deal 
             !  with it anyway to make sure the mass balance is closed
             !  and as a way to detect errors. Move the unallocated carbon
             !  back into the labile pool
             IF (biomass(ipts,j,ilabile,icarbon) + residual(ipts,j) .LE. min_stomate) THEN

                deficit = biomass(ipts,j,ilabile,icarbon) + residual(ipts,j)

                ! The deficit is less than the carbon reserve
                IF (-deficit .LE. biomass(ipts,j,icarbres,icarbon)) THEN

                   ! Pay the deficit from the reserve pool
                   biomass(ipts,j,icarbres,icarbon) = &
                        biomass(ipts,j,icarbres,icarbon) + deficit
                   biomass(ipts,j,ilabile,icarbon)  = &
                        biomass(ipts,j,ilabile,icarbon) - deficit

                ELSE

                   ! Not enough carbon to pay the deficit
                   ! There is likely a bigger problem somewhere in
                   ! this routine
                   WRITE(numout,*) 'WARNING 11: PFT, ipts: ',j,ipts
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 11: numerical problem overspending ',&
                        'when trying to account for unallocatable C ','')

                ENDIF

             ELSE

                ! Move the unallocated carbon back into the labile pool
                biomass(ipts,j,ilabile,icarbon) = &
                     biomass(ipts,j,ilabile,icarbon) + residual(ipts,j)

             ENDIF

             !! 5.2.12 Standardise allocation factors
             !  Strictly speaking the allocation factors do not need to be 
             !  calculated because the functional allocation scheme allocates 
             !  absolute amounts of carbon. Hence, Cl_inc could simply be 
             !  added to biomass(:,:,ileaf,icarbon), Cr_inc to 
             !  biomass(:,:,iroot,icarbon), etc. However, using allocation 
             !  factors bears some elegance in respect to distributing the 
             !  growth respiration if this would be required. Further it 
             !  facilitates comparison to the resource limited allocation 
             !  scheme (stomate_growth_res_lim.f90) and it comes in handy 
             !  for model-data comparison. This allocation takes place at 
             !  the tree level - note that ::biomass is the only prognostic 
             !  variable from the tree-based allocation

             !  Allocation   
             Cl_inc(:) = MAX(zero, circ_class_n(ipts,j,:) * Cl_inc(:))
             Cr_inc(:) = MAX(zero, circ_class_n(ipts,j,:) * Cr_inc(:))
             Cs_inc(:) = MAX(zero, circ_class_n(ipts,j,:) * Cs_inc(:))
             Cf_inc(:) = MAX(zero, circ_class_n(ipts,j,:) * Cf_inc(:))

             ! Total_inc is based on the updated Cl_inc, Cr_inc, Cs_inc and Cf_inc. Therefore, do not multiply
             ! circ_class_n(ipts,j,:) again
             total_inc = SUM(Cf_inc(:) + Cl_inc(:) + Cs_inc(:) + Cr_inc(:))

             ! Relative allocation
             IF ( total_inc .GT. min_stomate ) THEN

                Cl_inc(:) = Cl_inc(:) / total_inc
                Cs_inc(:) = Cs_inc(:) / total_inc
                Cr_inc(:) = Cr_inc(:) / total_inc
                Cf_inc(:) = Cf_inc(:) / total_inc

             ELSE

                bm_alloc_tot(ipts,j) = zero
                Cl_inc(:) = zero
                Cs_inc(:) = zero
                Cr_inc(:) = zero
                Cf_inc(:) = zero

             ENDIF


             !! 5.2.13 Convert allocation to allocation facors
             !  Convert allocation of individuals to ORCHIDEE's allocation 
             !  factors - see comment for 5.2.5. Aboveground sapwood 
             !  allocation is age dependent in trees. ::alloc_min and 
             !  ::alloc_max must range between 0 and 1. 
             alloc_sap_above = alloc_min(j) + ( alloc_max(j) - alloc_min(j) ) * &
                  ( 1. - EXP( -age(ipts,j) / demi_alloc(j) ) )

             ! Leaf, wood, root and fruit allocation
             f_alloc(ipts,j,ileaf) = SUM(Cl_inc(:))
             f_alloc(ipts,j,isapabove) = SUM(Cs_inc(:)*alloc_sap_above)
             f_alloc(ipts,j,isapbelow) = SUM(Cs_inc(:)*(1.-alloc_sap_above))
             f_alloc(ipts,j,iroot) = SUM(Cr_inc(:))
             f_alloc(ipts,j,ifruit) = SUM(Cf_inc(:))

             ! Absolute allocation at the tree level and for an individual tree (gC tree-1)
             ! The labile and reserve pools are not allocated at the tree level. However,
             ! stand level ilabile and icarbres biomass will be redistributed at the tree
             ! level later in this subroutine. This is done after the relative allocation
             ! beacuse now ::alloc_sap_above is known
             circ_class_biomass(ipts,j,:,ileaf,icarbon) = &
                  circ_class_biomass(ipts,j,:,ileaf,icarbon) + &
                  ( Cl_inc(:) * total_inc / circ_class_n(ipts,j,:) )
             circ_class_biomass(ipts,j,:,isapabove,icarbon) = &
                  circ_class_biomass(ipts,j,:,isapabove,icarbon) + &
                  ( Cs_inc(:) * alloc_sap_above * total_inc / &
                  circ_class_n(ipts,j,:) )
             circ_class_biomass(ipts,j,:,isapbelow,icarbon) = &
                  circ_class_biomass(ipts,j,:,isapbelow,icarbon) + &
                  ( Cs_inc(:) * (un - alloc_sap_above) * &
                  total_inc / circ_class_n(ipts,j,:) )
             circ_class_biomass(ipts,j,:,iroot,icarbon) = &
                  circ_class_biomass(ipts,j,:,iroot,icarbon) + &
                  ( Cr_inc(:) * total_inc / circ_class_n(ipts,j,:) )
             circ_class_biomass(ipts,j,:,ifruit,icarbon) = &
                  circ_class_biomass(ipts,j,:,ifruit,icarbon) + &
                  ( Cf_inc(:) * total_inc / circ_class_n(ipts,j,:) )

             !+++TEMP+++
             IF (ld_alloc) THEN
                tempi = zero
                DO icirc = 1,ncirc
                   IF ( Cl_inc(icirc) .LT. zero) THEN
                      WRITE(numout,*) 'Cl_inc, ', j, Cl_inc(icirc)
                      tempi = un
                   ENDIF
                   IF ( Cs_inc(icirc) * alloc_sap_above .LT. zero) THEN
                      WRITE(numout,*) 'Cs_inc aboveground, ', j, &
                           Cs_inc(icirc) * alloc_sap_above
                      tempi = un
                   ENDIF
                   IF ( Cs_inc(icirc) * (un - alloc_sap_above) .LT. zero) THEN
                      WRITE(numout,*) 'Cs_inc aboveground, ', j, &
                           Cs_inc(icirc) * (un-alloc_sap_above)
                      tempi = un
                   ENDIF
                   IF ( Cr_inc(icirc) .LT. zero) THEN
                      WRITE(numout,*) 'Cr_inc, ', j, Cr_inc(icirc)
                      tempi = un
                   ENDIF
                   IF ( Cf_inc(icirc) .LT. zero) THEN
                      WRITE(numout,*) 'Cf_inc, ', j, Cf_inc(icirc)
                      tempi = un
                   ENDIF
                   IF ( total_inc .LT. zero) THEN
                      WRITE(numout,*) 'total_inc, ', j, total_inc
                      tempi = un
                   ENDIF
                   IF ( circ_class_n(ipts,j,icirc) .LT. zero) THEN
                      WRITE(numout,*) 'circ_class_n, ', j, circ_class_n(ipts,j,icirc)
                      tempi = un
                   ENDIF
                ENDDO
                IF (tempi == un) CALL ipslerr_p (3,'growth_fun_all',&
                     'WARNING 11bis: the solution has negative values',&
                     'None of these variables should be negative','')   
             ENDIF
             !++++++++++

          ELSEIF (is_tree(j)) THEN
       
             IF(ld_alloc) WRITE(numout,*) 'there is no tree biomass to allocate, PFT, ', j 

          ENDIF ! Is there biomass to allocate (§5.2 - far far up)


          !! 5.3 Calculate allocated biomass pools for grasses and crops
          !  Only possible if there is biomass to allocate
          IF ( .NOT. is_tree(j) .AND. bm_alloc_tot(ipts,j) .GT. min_stomate ) THEN

             !! 5.3.1 Scaling factor to convert variables to the individual plant
             !  Allocation is on an individual basis (gC ind-1). Stand-level variables 
             !  need to convert to a single individual. The absence of sapwood makes 
             !  this irrelevant because the allocation reduces to a linear function 
             !  (contrary to the non-linearity of tree allocation). For the
             !  beauty of consistency, the transformations will be implemented.
             !  Different approach between the DGVM and statitic approach
             IF (control%ok_dgvm) THEN

                ! The DGVM does NOT work with the functional allocation. Consider
                ! this code as a placeholder. The original code had two different 
                ! transformations to calculate the scalars. Both could be used but 
                ! the units will differ. For consistency only one was retained 
                ! scal = ind(ipts,j) * cn_ind(ipts,j) / veget_max(ipts,j)
                scal = veget_max(ipts,j) / ind(ipts,j)

             ELSE

                ! By dividing the actual biomass by the number of individuals
                ! the biomass of an individual is obtained. Note that a grass/crop 
                ! individual was defined as 1m-2 of vegetation 
                scal = 1./ ind(ipts,j)

             ENDIF


             !! 5.3.2 Current biomass pools per grass/crop (gC ind^-1)
             !  Cs has too many dimensions for grass/crops. To have a consistent notation the same variables
             !  are used as for trees but the dimension of Cs, Cl and Cr i.e. ::ncirc should be ignored            
             Cs(:) = biomass(ipts,j,isapabove,icarbon) * scal
             Cr(:) = biomass(ipts,j,iroot,icarbon) * scal
             Cl(:) = biomass(ipts,j,ileaf,icarbon) * scal
             Ch(:) = zero

             ! Total amount of carbon that needs to be allocated (::bm_alloc_tot). bm_alloc_tot is
             ! in gC m-2 day-1. At 1 m2 there are ::ind number of grasses.
             b_inc_tot = bm_alloc_tot(ipts,j)

             !! 5.3.3 C-allocation for crops and grasses
             !  The mass conservation equations are detailed in the header of this subroutine.
             !  The scheme assumes a functional relationships between leaves and roots for grasses and crops. 
             !  When carbon is added to the leaf biomass pool, an increase in the root biomass is to be 
             !  expected to sustain water transport from the roots to the leaves.

             !! 5.3.3.1 Do the biomass pools respect the pipe model?
             !  Do the current leaf, sapwood and root components respect the allometric 
             !  constraints? Calculate the optimal root and leaf mass, given the current wood mass 
             !  by using the basic allometric relationships. Calculate the optimal sapwood
             !  mass as a function of the current leaf and root mass.
             Cl_target(1) = MAX( Cs(1) * KF(ipts,j) , Cr(1) * LF(ipts,j), Cl(1) )
             Cs_target(1) = MAX( Cl_target(1) / KF(ipts,j), Cr(1) * LF(ipts,j) / KF(ipts,j), Cs(1) ) 
             Cr_target(1) = MAX( Cl_target(1) / LF(ipts,j), Cs_target(1) * KF(ipts,j) / LF(ipts,j), Cr(1) )
             
             ! Write debug comments to output file
             IF (j .EQ. test_pft .AND. ld_alloc) THEN
                WRITE(numout,*) 'bm_alloc_tot, ',bm_alloc_tot(ipts,j)
                WRITE(numout,*) 'Does the grass/crop needs reshaping?'
                WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
                WRITE(numout,*) 'qm_height, ', (Cl_target(1) * sla(j) * lai_to_height(j))
                WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(1)-Cl(1), Cl_target(1), Cl(1)
                WRITE(numout,*) 'Cs_target-Cs, ', Cs_target(1)-Cs(1), Cs_target(1), Cs(1)
                WRITE(numout,*) 'Cr_target-Cr, ', Cr_target(1)-Cr(1), Cr_target(1), Cr(1)
             ENDIF


             !! 5.3.3.2 Phenological growth
             !  Phenological growth and reshaping of the grass/crop in line with the pipe model. Turnover removes 
             !  C from the different plant components but at a component-specific rate, as such the allometric 
             !  constraints are distorted at every time step and should be restored before ordinary growth can 
             !  take place

             !! 5.3.3.2.1 The available structural C can sustain the available leaves and roots
             !  Calculate whether the structural c is in allometric balance. The target values should 
             !  always be larger than the current pools so the use of ABS is redundant but was used to 
             !  be on the safe side (here and in the rest of the module) as it could help to find 
             !  logical flaws.        
             IF ( ABS(Cs_target(1) - Cs(1)) .LT. min_stomate ) THEN

                Cs_incp(1) = MAX(zero, Cs_target(1) - Cs(1))

                ! Enough leaves and structural biomass, only grow roots
                IF ( ABS(Cl_target(1) - Cl(1))  .LT. min_stomate ) THEN

                   ! Allocate at the tree level to restore allometric balance
                   Cl_incp(1) = MAX(zero, Cl_target(1) - Cl(1))
                   Cr_incp(1) = MAX( MIN(b_inc_tot / ind(ipts,j) - Cs_incp(1) - Cl_incp(1), &
                        Cr_target(1) - Cr(1)), zero )

                   ! Write debug comments to output file
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 12)
                   ENDIF

                ! Sufficient structural C and roots, allocate C to leaves
                ELSEIF ( ABS(Cr_target(1) - Cr(1)) .LT. min_stomate ) THEN

                   ! Allocate at the tree level to restore allometric balance 
                   Cr_incp(1) = MAX(zero, Cr_target(1) - Cr(1))
                   Cl_incp(1) = MAX( MIN(b_inc_tot / ind(ipts,j) - Cs_incp(1) - Cr_incp(1), &
                        Cl_target(1) - Cl(1)), zero )

                   ! Update vegetation height
                   qm_height = (Cl(1) + Cl_incp(1)) * sla(j) * lai_to_height(j)
               
                   ! Write debug comments to output file
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 13)
                   ENDIF

                ! Both leaves and roots are needed to restore the allometric relationships
                ELSEIF ( ABS(Cl_target(1) - Cl(1)) .GT. min_stomate .AND. &
                     ABS(Cr_target(1) - Cr(1)) .GT. min_stomate ) THEN                 

                   ! Allocate at the tree level to restore allometric balance
                   !  The equations can be rearanged and written as
                   !  (i) b_inc = Cl_inc + Cr_inc
                   !  (ii) Cr_inc = (Cl_inc+Cl)/LF - Cr
                   !  Substitue (ii) in (i) and solve for Cl_inc
                   !  <=> Cl_inc = (LF*(b_inc+Cr)-Cl)/(1+LF)
                   Cl_incp(1) = MIN( ((LF(ipts,j) * ((b_inc_tot/ind(ipts,j)) - Cs_incp(1) + Cr(1))) - Cl(1)) / & 
                        (1 + LF(ipts,j)), Cl_target(1) - Cl(1) )
                   Cr_incp(1) = MIN ( ((Cl_incp(1) + Cl(1)) / LF(ipts,j)) - Cr(1), &
                        Cr_target(1) - Cr(1))

                   ! The imbalance between Cr and Cl can be so big that (Cl+Cl_inc)/LF is still less
                   ! then the available root carbon (observed!). This would result in a negative Cr_incp
                   IF ( Cr_incp(1) .LT. zero ) THEN

                      Cl_incp(1) = MIN( b_inc_tot/ind(ipts,j) - Cs_incp(1), Cl_target(1) - Cl(1) )
                      Cr_incp(1) = b_inc_tot/ind(ipts,j) - Cs_incp(1) - Cl_incp(1)

                   ELSEIF (Cl_incp(1) .LT. zero) THEN

                      Cr_incp(1) = MIN( b_inc_tot/ind(ipts,j) - Cs_incp(1), Cr_target(1) - Cr(1) )
                      Cl_incp(1) = (b_inc_tot/ind(ipts,j)) - Cs_incp(1) - Cr_incp(1)

                   ENDIF

                   ! Update vegetation height
                   qm_height = (Cl(1) + Cl_incp(1)) * sla(j) * lai_to_height(j)

                   ! Write debug comments to output file
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 14)
                   ENDIF    

                ELSE

                   WRITE(numout,*) 'WARNING 12: Exc 1-3 unexpected exception'
                   WRITE(numout,*) 'WARNING 12: PFT, ipts: ',j,ipts
                   IF(ld_stop)THEN
                      CALL ipslerr_p (3,'growth_fun_all',&
                          'WARNING 12: Exc 1-3 unexpected exception','','') 
                   ENDIF

                ENDIF


             !! 5.3.3.3.2 Enough leaves to sustain the structural C and roots
             ELSEIF ( ABS(Cl_target(1) - Cl(1)) .LT. min_stomate ) THEN
                
                Cl_incp(1) = MAX(zero, Cl_target(1) - Cl(1))

                ! Enough leaves and structural C, only grow roots
                ! This duplicates Exc 1 and these lines should never be called 
                IF ( ABS(Cs_target(1) - Cs(1)) .LT. min_stomate ) THEN

                   ! Allocate at the tree level to restore allometric balance
                   Cs_incp(1) = MAX(zero, Cs_target(1) - Cs(1))
                   Cr_incp(1) = MAX( MIN(b_inc_tot/ind(ipts,j) - Cl_incp(1) - Cs_incp(1), &
                        Cr_target(1) - Cr(1)), zero )

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 15)
                   ENDIF 

                ! Enough leaves and roots. Need to grow structural C to support the available canopy and roots
                ELSEIF ( ABS(Cr_target(1) - Cr(1)) .LT. min_stomate ) THEN

                   Cr_incp(1) = MAX(zero, Cr_target(1) - Cr(1))
                   Cs_incp(1) = MAX( MIN(b_inc_tot/ind(ipts,j) - Cr_incp(1) - Cl_incp(1), &
                        Cs_target(1) - Cs(1)), zero )

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 16)
                   ENDIF                  

                ! Need both structural C and roots to restore the allometric relationships
                ELSEIF ( ABS(Cs_target(1) - Cs(1) ) .GT. min_stomate .AND. &
                     ABS(Cr_target(1) - Cr(1)) .GT. min_stomate ) THEN

                   !  First try if we can simply satisfy the allocation needs
                   IF (Cs_target(1) - Cs(1) + Cr_target(1) - Cr(1) .LE. &
                           b_inc_tot/ind(ipts,j) - Cl_incp(1)) THEN
                         
                         Cr_incp(1) = Cr_target(1) - Cr(1)
                         Cs_incp(1) = Cs_target(1) - Cs(1)

                      ! Try to satisfy the need for the roots
                      ELSEIF (Cr_target(1) - Cr(1) .LE. b_inc_tot/ind(ipts,j) - Cl_incp(1)) THEN

                         Cr_incp(1) = Cr_target(1) - Cr(1)
                         Cs_incp(1) = b_inc_tot/ind(ipts,j) - Cl_incp(1) - Cr_incp(1)
                         

                      ! There is not enough use whatever is available
                      ELSE
                         
                         Cr_incp(1) = b_inc_tot/ind(ipts,j) - Cl_incp(1)
                         Cs_incp(1) = zero
                         
                      ENDIF

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 17)
                   ENDIF 

                ELSE

                   WRITE(numout,*) 'WARNING 13: Exc 4-6 unexpected exception'
                   WRITE(numout,*) 'WARNING 13: PFT, ipts: ',j,ipts
                   IF(ld_stop)THEN
                      CALL ipslerr_p (3,'growth_fun_all',&
                           'WARNING 13: Exc 4-6 unexpected exception','','')
                   ENDIF

                ENDIF


             !! 5.3.3.3.3 Enough roots to sustain the wood and leaves
             ELSEIF ( ABS(Cr_target(1) - Cr(1)) .LT. min_stomate ) THEN

                Cr_incp(1) = MAX(zero, Cr_target(1) - Cr(1)) 

                ! Enough roots and wood, only grow leaves
                ! This duplicates Exc 2 and these lines should thus never be called 
                IF ( ABS(Cs_target(1) - Cs(1)) .LT. min_stomate ) THEN

                   ! Allocate at the tree level to restore allometric balance
                   Cs_incp(1) = MAX(zero, Cs_target(1) - Cs(1)) 
                   Cl_incp(1) = MAX( MIN(b_inc_tot/ind(ipts,j) - Cr_incp(1) - Cs_incp(1), &
                        Cl_target(1) - Cl(1)), zero )

                   ! Update vegetation height
                   qm_height = (Cl(1) + Cl_incp(1)) * sla(j) * lai_to_height(j)

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 18)
                   ENDIF 

                ! Enough leaves and roots. Need to grow sapwood to support the available canopy and roots
                ! Duplicates Exc. 4 and these lines should thus never be called 
                ELSEIF ( ABS(Cl_target(1) - Cl(1)) .LT. min_stomate ) THEN

                   ! Allocate at the tree level to restore allometric balance
                   Cl_incp(1) = MAX(zero, Cl_target(1) - Cl(1)) 
                   Cs_incp(1) = MAX( MIN(b_inc_tot/ind(ipts,j) - Cr_incp(1) - Cl_incp(1), &
                        Cs_target(1) - Cs(1) ), zero )

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 19)
                   ENDIF                       

                ! Need both wood and leaves to restore the allometric relationships
                ELSEIF ( ABS(Cs_target(1) - Cs(1)) .GT. min_stomate .AND. &
                     ABS(Cl_target(1) - Cl(1)) .GT. min_stomate ) THEN

                   ! circ_class_ba_eff and circ_class_height_eff are already calculated
                   ! for a tree in balance. It would be rather complicated to follow
                   ! the allometric rules for wood allocation (implying changes in height 
                   ! and basal area) because the tree is not in balance.First try if we 
                   ! can simply satisfy the allocation needs
                   IF (Cs_target(1) - Cs(1) + Cl_target(1) - Cl(1) .LE. &
                        b_inc_tot/ind(ipts,j) - Cr_incp(1)) THEN
                      
                      Cl_incp(1) = Cl_target(1) - Cl(1)
                      Cs_incp(1) = Cs_target(1) - Cs(1)
                      
                   ! Try to satisfy the need for leaves
                   ELSEIF (Cl_target(1) - Cl(1) .LE. b_inc_tot/ind(ipts,j) - Cr_incp(1)) THEN

                      Cl_incp(1) = Cl_target(1) - Cl(1)
                      Cs_incp(1) = b_inc_tot/ind(ipts,j) - Cr_incp(1) - Cl_incp(1)

                   ! There is not enough use whatever is available
                   ELSE

                      Cl_incp(1) = b_inc_tot/ind(ipts,j) - Cr_incp(1)
                      Cs_incp(1) = zero
                      
                   ENDIF
                      
                   ! Calculate the height of the expanded canopy
                   qm_height(ipts,j) = (Cl(1) + Cl_inc(1)) * sla(j) * lai_to_height(j)

                   ! Write debug comments to output file 
                   IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                      CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                           b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                           grow_wood, circ_class_n, ind, 20)
                   ENDIF

                ELSE

                   WRITE(numout,*) 'WARNING 14: Exc 7-9 unexpected exception'
                   WRITE(numout,*) 'WARNING 14: PFT, ipts: ',j, ipts
                   IF(ld_stop)THEN
                      CALL ipslerr_p (3,'growth_fun_all',&
                           'WARNING 14: Exc 7-9 unexpected exception','','')
                   ENDIF

                ENDIF

             ! Either Cl_target, Cs_target or Cr_target should be zero
             ELSE

                ! Something possibly important was overlooked
                WRITE(numout,*) 'WARNING 15: Logical flaw in phenological allocation '
                WRITE(numout,*) 'WARNING 15: PFT, ipts: ',j, ipts
                WRITE(numout,*) 'Cs - Cs_target', Cs(1), Cs_target(1)
                WRITE(numout,*) 'Cl - Cl_target', Cl(1), Cl_target(1)
                WRITE(numout,*) 'Cr - Cr_target', Cr(1), Cr_target(1)
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 15: Logical flaw in phenological allocation','','')
                ENDIF

             ENDIF


             !! 5.3.4 Wrap-up phenological allocation
             IF ( Cl_incp(1) .GE. zero .OR. Cr_incp(1) .GE. zero .OR. Cs_incp(1) .GE. zero) THEN

                ! Fake allocation for less messy equations in next 
                ! case, incp needs to be added to inc at the end
                Cl(1) = Cl(1) + Cl_incp(1)
                Cr(1) = Cr(1) + Cr_incp(1)
                Cs(1) = Cs(1) + Cs_incp(1)
                b_inc_tot = b_inc_tot - (ind(ipts,j) * (Cl_incp(1) + Cr_incp(1) + Cs_incp(1)))

             ELSE

                ! The code was written such that the increment pools should be greater than or equal
                ! to zero. If this is not the case, something fundamental is wrong with the if-then 
                ! constructs under §5.3.3.2
                WRITE(numout,*) 'WARNING 16: numerical problem, one of the increment pools is less than zero'
                WRITE(numout,*) 'WARNING 16: Cl_incp(1), Cr_incp(1), Cs_incp(1), j, ipts',&
                     Cl_incp(1), Cr_incp(1), Cs_incp(1), j, ipts
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 16: numerical problem, one of the increment pools is less than zero','','')
                ENDIF

             ENDIF

             ! Height depends on Cl, so update height when Cl gets updated
             qm_height(ipts,j) = Cl(1) * sla(j) * lai_to_height(j) 

             ! Something is wrong with the calculations
             IF (b_inc_tot .LT. -min_stomate) THEN

                WRITE(numout,*) 'WARNING 17: numerical problem overspending in the phenological allocation'
                WRITE(numout,*) 'WARNING 17: b_inc_tot, j, ipts',b_inc_tot, j, ipts 
                WRITE(numout,*) 'WARNING 17: Cl_incp, Cr_incp, Cs_incp, ', Cl_incp(1), Cr_incp(1), Cs_incp(1)
                IF(ld_stop)THEN
                    CALL ipslerr_p (3,'growth_fun_all',&
                         'WARNING 17: numerical problem overspending in the phenological allocation','','')
                ENDIF

             ENDIF

    
             !! 5.3.5 Calculate the expected size of the reserve pool 
             !  use the minimum of either (1) 20% of the total sapwood biomass or 
             !  (2) the amount of carbon needed to develop the optimal LAI and the roots
             !  This reserve pool estimate is only used to decide whether wood should be
             !  grown or not. When really dealing with the reserves the reserve pool is 
             !  recalculated. See further below §7.1.

             !+++CHECK+++
             ! Sapwood has no meaning for grasses and crops - reserves should sustain the canopy
             ! For trees 2% was used, given the much lower sapwood pool for grasses and crops we 
             ! set it arbitrairly to 10%
             reserve_pool = MIN( 0.10 * ( biomass(ipts,j,isapabove,icarbon) + &
                  biomass(ipts,j,isapbelow,icarbon)), lai_target(ipts,j)/sla(j)*&
                  (1.+0.3/ltor(ipts,j)) )
             !+++++++++++
             grow_wood = .TRUE.

             ! If the carbohydrate pool is too small, don't grow structural C and 
             ! thus skip ordinary allocation
             IF ( (pheno_type(j) .NE. 1) .AND. &
                  (biomass(ipts,j,icarbres,icarbon) .LE. reserve_pool) ) THEN

                grow_wood = .FALSE.

             ENDIF

             ! Write debug comments to output file 
             IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, &
                     delta_ba, ipts, j, l, b_inc_tot, Cl_incp, Cs_incp, Cr_incp, &
                     KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, grow_wood, &
                     circ_class_n, ind, 21)
             ENDIF


             !! 5.3.6 Ordinary growth
             !  Allometric relationship between components is respected, sustain 
             !  ordinary growth and allocate biomass to leaves, wood, roots and fruits.
             IF ( (ABS(Cl_target(1) - Cl(1) ) .LE. min_stomate) .AND. &
                  (ABS(Cs_target(1) - Cs(1) ) .LE. min_stomate) .AND. &
                  (ABS(Cr_target(1) - Cr(1) ) .LE. min_stomate) .AND. &
                  (grow_wood) .AND. (b_inc_tot/ind(ipts,j) .GT. min_stomate) ) THEN 

                ! Allocate fraction of carbon to fruit production (at the tree level)
                Cf_inc(:) = b_inc_tot * fruit_alloc(j)

                ! Residual carbon is allocated to the other components (b_inc_tot is 
                ! at the stand level)
                b_inc_tot = b_inc_tot * (1-fruit_alloc(j))

                ! Following allometric allocation
                ! (i) b_inc = Cl_inc + Cr_inc + Cs_inc
                ! (ii) Cr_inc = (Cl + Cl_inc)/LF - Cr
                ! (iii) Cs_inc = (Cl + Cl_inc) / KF - Cs
                ! Substitue (ii) and (iii) in (i) and solve for Cl_inc 
                ! <=> b_inc = Cl_inc + ( Cl_inc + Cl ) / KF - Cs + ( Cl_inc + Cl ) / LF - Cr
	        ! <=> b_inc = Cl_inc * ( 1.+ 1/KF + 1./LF ) + Cl/LF - Cs - Cr
	        ! <=> Cl_inc = ( b_inc - Cl/LF + Cs + Cr ) / ( 1.+ 1/KF + 1./LF )
                Cl_inc(1) = MAX( (b_inc_tot/ind(ipts,j) - Cl(1)/LF(ipts,j) - &
                     Cl(1)/KF(ipts,j) + Cs(1) + Cr(1)) / &
                     (1. + 1./KF(ipts,j) + 1./LF(ipts,j)), zero)
               
                IF (Cl_inc(1) .LE. zero) THEN

                   Cr_inc(:) = zero
                   Cs_inc(:) = zero

                ELSE

                   ! Calculate the height of the expanded canopy
                   qm_height(ipts,j) = (Cl(1) + Cl_inc(1)) * sla(j) * lai_to_height(j)

                   ! Use the solution for Cl_inc to calculate Cr_inc and Cs_inc according to (ii) and (iii)
                   Cr_inc(1) = (Cl(1) + Cl_inc(1)) / LF(ipts,j) - Cr(1)                
                   Cs_inc(1) = (Cl(1)+Cl_inc(1)) / KF(ipts,j) - Cs(1)
               
                ENDIF

                ! Write debug comments to output file 
                IF ((j.EQ.test_pft .AND. ld_alloc) .OR. ld_warn) THEN
                   CALL comment(npts, Cl_target, Cl, Cs_target, Cs, Cr_target, Cr, delta_ba, ipts, j, l, &
                        b_inc_tot, Cl_incp, Cs_incp, Cr_incp, KF, LF, Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
                        grow_wood, circ_class_n, ind, 22)
                ENDIF
                
                ! Wrap-up ordinary growth  
                ! Calculate C that was not allocated, note that Cf_inc was already substracted
                b_inc_tot = b_inc_tot - (ind(ipts,j) * (Cl_inc(1) + Cr_inc(1) + Cs_inc(1)))

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'wrap-up ordinary allocation, left b_in_tot, ', b_inc_tot 
                ENDIF
                !----------


             !! 5.3.7 Don't grow wood, use C to fill labile pool
             ELSEIF ( (.NOT. grow_wood) .AND. (b_inc_tot .GT. min_stomate) ) THEN

                ! Calculate the C that needs to be distributed to the 
                ! labile pool. The fraction is proportional to the ratio 
                ! between the total allocatable biomass and the unallocated 
                ! biomass per tree (b_inc now contains the unallocated 
                ! biomass). At the end of the allocation scheme bm_alloc_tot 
                ! is substracted from the labile biomass pool to update the 
                ! biomass pool (biomass(:,:,ilabile) = biomass(:,:,ilabile) - 
                ! bm_alloc_tot(:,:)). At that point, the scheme puts the 
                ! unallocated b_inc into the labile pool. What we 
                ! want is that the unallocated fraction is removed from 
                ! ::bm_alloc_tot such that only the allocated C is removed 
                ! from the labile pool. b_inc_tot will be moved back into
                ! the labile pool in 5.2.11
                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) - b_inc_tot
                biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) + &
                     b_inc_tot
                                
                ! Wrap-up ordinary growth  
                ! Calculate C that was not allocated (b_inc_tot), the 
                ! equation should read b_inc_tot = b_inc_tot - b_inc_tot 
                ! note that Cf_inc was already substracted
                b_inc_tot = zero 

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'No wood growth, move remaining C to labile pool'
                   WRITE(numout,*) 'bm_alloc_tot_new, ',bm_alloc_tot(ipts,j)
                   WRITE(numout,*) 'wrap-up ordinary allocation, left b_inc_tot, ', b_inc_tot 
                ENDIF
                !---------- 

             !! 5.3.8 Error - the allocation scheme is overspending 
             ELSEIF (b_inc_tot .LE. min_stomate) THEN  
      
                IF (b_inc_tot .LT. -min_stomate) THEN

                   ! Something is wrong with the calculations
                   WRITE(numout,*) 'WARNING 18: numerical problem overspending in ordinary allocation'
                   WRITE(numout,*) 'WARNING 18: PFT, ipts, b_inc_tot: ', j, ipts,b_inc_tot
                   IF(ld_stop)THEN
                      CALL ipslerr_p (3,'growth_fun_all',&
                           'WARNING 18: numerical problem overspending in ordinary allocation','','')
                   ENDIF

                ELSE

                   IF (j .EQ. test_pft .AND. ld_alloc) THEN

                      ! Succesful allocation
                      WRITE(numout,*) 'Successful allocation'

                   ENDIF

                ENDIF
              
                ! Althought the biomass components respect the allometric relationships, there
                ! is no carbon left to allocate                      
                b_inc_tot = zero
                Cl_inc(1) = zero
                Cs_inc(1) = zero
                Cr_inc(1) = zero
                Cf_inc(1) = zero

             ELSE

                WRITE(numout,*) 'WARNING 19: Logical flaw unexpected result in ordinary allocation'
                WRITE(numout,*) 'WARNING 19: PFT, ipts: ', j, ipts
                WRITE(numout,*) 'WARNING 19: ',ABS(Cl_target(1) - Cl(1) ) , Cl(1)
                WRITE(numout,*) 'WARNING 19: ',ABS(Cs_target(1) - Cs(1) )  , Cs(1)
                WRITE(numout,*) 'WARNING 19: ',ABS(Cr_target(1) - Cr(1) )  , Cr(1)
                WRITE(numout,*) 'WARNING 19: ',grow_wood
                WRITE(numout,*) 'WARNING 19: ',b_inc_tot,ind(ipts,j),b_inc_tot/ind(ipts,j)
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 19: Logical flaw unexpected result in ordinary allocation','','')
                ENDIF
                
             ENDIF ! Ordinary allocation


             !! 5.3.9 Forced allocation
             !  Although this should not happen, in case the functional allocation did not consume 
             !  all the allocatable carbon, the remaining C is left for the next day, and some of 
             !  the biomass is used to produce fruits (tuned)
             IF ( b_inc_tot .GT. min_stomate ) THEN 

                WRITE(numout,*) 'WARNING 20: unexpected outcome force allocation'
                WRITE(numout,*) 'WARNING 20: grow_wood, b_inc_tot: ', grow_wood, b_inc_tot
                WRITE(numout,*) 'WARNING 20: PFT, ipts: ',j,ipts
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 20: unexpected outcome force allocation','','')
                ENDIF

                !+++CHECK+++
!!$                ! Calculate fraction that will be allocated to fruit. The fraction is proportional to the 
!!$                ! ratio between the total allocatable biomass and the unallocated biomass per tree
!!$                frac = 0.1 * MIN(1., bm_alloc_tot(ipts,j) / b_inc_tot)
!!$                Cf_inc(:) = Cf_inc(:) + b_inc_tot * frac
!!$                b_inc_tot = b_inc_tot * (1 - frac)
!!$
!!$                ! Calculate the C that needs to be distributed to the labile pool. The fraction is proportional 
!!$                ! to the ratio between the total allocatable biomass and the unallocated biomass per tree (b_inc 
!!$                ! now contains the unallocated biomass). At the end of the allocation scheme bm_alloc_tot is 
!!$                ! substracted from the labile biomass pool to update the biomass pool (biomass(:,:,ilabile) = 
!!$                ! biomass(:,:,ilabile,icarbon) - bm_alloc_tot(:,:)). At that point, the scheme puts the 
!!$                ! unallocated b_inc into the labile pool. 
!!$                bm_alloc_tot(ipts,j) = bm_alloc_tot(ipts,j) * ( 1. - (1.-frac) * b_inc_tot / bm_alloc_tot(ipts,j) )
                !++++++++++

             ELSEIF ( (b_inc_tot .LT. min_stomate) .AND. (b_inc_tot .GE. -min_stomate) ) THEN

                ! Successful allocation
                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'Successful allocation'
                ENDIF
                !----------

             ELSE

                ! Something possibly important was overlooked
                IF ( (b_inc_tot .LT. 100*min_stomate) .AND. (b_inc_tot .GE. -100*min_stomate) ) THEN
                   IF (j.EQ.test_pft .AND. ld_alloc) THEN
                      WRITE(numout,*) 'Marginally successful allocation - precision is better than 10-6', j
                   ENDIF
                ELSE
                   WRITE(numout,*) 'WARNING 21: Logical flaw unexpected result in ordinary allocation'
                   WRITE(numout,*) 'WARNING 21: b_inc_tot', b_inc_tot
                   WRITE(numout,*) 'WARNING 21: PFT, ipts: ',j,ipts
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 21: Logical flaw unexpected result in ordinary allocation','','')
                ENDIF

             ENDIF

             ! The second problem we need to catch is when one of the increment pools is 
             ! negative. This is an undesired outcome (see comment where ::KF_old is 
             ! calculated in this routine. In that case we write a warning, set all increment
             ! pools to zero and try it again at the next time step. A likely cause of this 
             ! problem is a too large change in KF from one time step to another. Try decreasing
             ! the acceptable value for an absolute increase in KF.
             IF (Cs_inc(1) .LT. zero .OR. Cr_inc(1) .LT. zero .OR. Cs_inc(1) .LT. zero) THEN
             
                ! Do not allocate - save the carbon for the next time step
                Cl_inc(1) = zero
                Cr_inc(1) = zero
                Cs_inc(1) = zero
                WRITE(numout,*) 'WARNING 22: numerical problem, one of the increment pools is less than zero'
                WRITE(numout,*) 'WARNING 22: PFT, ipts: ',j,ipts
               
             ENDIF


             !! 5.3.10 Wrap-up phenological and ordinary allocation
             Cl_inc(1) = Cl_inc(1) + Cl_incp(1)
             Cr_inc(1) = Cr_inc(1) + Cr_incp(1)
             Cs_inc(1) = Cs_inc(1) + Cs_incp(1)
             residual(ipts,j) = b_inc_tot

             !---TEMP---
             IF (j.EQ.test_pft .AND. ld_alloc) THEN
                WRITE(numout,*) 'Final allocation', j 
                WRITE(numout,*) 'Cl, Cs, Cr', Cl(1), Cs(1), Cr(1) 
                WRITE(numout,*) 'Cl_incp, Cs_incp, Cr_incp, ', Cl_incp(1), Cs_incp(1), Cr_incp(1)
                WRITE(numout,*) 'Cl_inc, Cs_ins, Cr_inc, Cf_inc, ', Cl_inc(1), Cs_inc(1), Cr_inc(1), Cf_inc(1)
                WRITE(numout,*) 'unallocated/residual, ', b_inc_tot
             ENDIF
             !----------
            

             !! 5.3.11 Account for the residual
             !  The residual is usually around ::min_stomate but we deal 
             !  with it anyway to make sure the mass balance is closed
             !  and as a way to detect errors. Move the unallocated carbon
             !  back into the labile pool
             IF (biomass(ipts,j,ilabile,icarbon) + residual(ipts,j) .LE. min_stomate) THEN

                deficit = biomass(ipts,j,ilabile,icarbon) + residual(ipts,j)

                ! The deficit is less than the carbon reserve
                IF (-deficit .LE. biomass(ipts,j,icarbres,icarbon)) THEN

                   ! Pay the deficit from the reserve pool
                   biomass(ipts,j,icarbres,icarbon) = &
                        biomass(ipts,j,icarbres,icarbon) + deficit
                   biomass(ipts,j,ilabile,icarbon)  = &
                        biomass(ipts,j,ilabile,icarbon) - deficit

                ELSE

                   ! Not enough carbon to pay the deficit
                   ! There is likely a bigger problem somewhere in
                   ! this routine
                   WRITE(numout,*) 'WARNING 23: PFT, ipts: ',j,ipts
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'WARNING 23: numerical problem overspending ',&
                        'when trying to account for unallocatable C ','')

                ENDIF

             ELSE
                
                ! Move the unallocated carbon back into the labile pool
                biomass(ipts,j,ilabile,icarbon) = &
                     biomass(ipts,j,ilabile,icarbon) + residual(ipts,j)
                
             ENDIF

             !! 5.3.12 Standardise allocation factors
             !  Strictly speaking the allocation factors do not need to be calculated because the functional
             !  allocation scheme allocates absolute amounts of carbon. Hence, Cl_inc could simply be added to
             !  biomass(:,:,ileaf,icarbon), Cr_inc to biomass(:,:,iroot,icarbon), etc. However, using allocation 
             !  factors bears some elegance in respect to distributing the growth respiration if this would be 
             !  required. Further it facilitates comparison to the resource limited allocation scheme 
             !  (stomate_growth_res_lim.f90) and it comes in handy for model-data comparison. This allocation
             !  takes place at the tree level - note that ::biomass is the only prognostic variable from the tree-based
             !  allocation
                          
             !  Allocation   
             Cl_inc(1) = MAX(zero, ind(ipts,j) * Cl_inc(1))
             Cr_inc(1) = MAX(zero, ind(ipts,j) * Cr_inc(1))
             Cs_inc(1) = MAX(zero, ind(ipts,j) * Cs_inc(1))
             Cf_inc(1) = MAX(zero, ind(ipts,j) * Cf_inc(1))
             
             ! Total_inc is based on the updated Cl_inc, Cr_inc, Cs_inc and Cf_inc. Therefore, do not multiply
             ! ind(ipts,j) again
             total_inc = (Cf_inc(1) + Cl_inc(1) + Cs_inc(1) + Cr_inc(1))
             
             ! Relative allocation
             IF ( total_inc .GT. min_stomate ) THEN

                Cl_inc(1) = Cl_inc(1) / total_inc
                Cs_inc(1) = Cs_inc(1) / total_inc
                Cr_inc(1) = Cr_inc(1) / total_inc
                Cf_inc(1) = Cf_inc(1) / total_inc

             ELSE

                bm_alloc_tot(ipts,j) = zero
                Cl_inc(1) = zero
                Cs_inc(1) = zero
                Cr_inc(1) = zero
                Cf_inc(1) = zero

             ENDIF

 
             !! 5.3.13 Convert allocation to allocation facors
             !  Convert allocation of individuals to ORCHIDEE's allocation factors - see comment for 5.2.5
             !  Aboveground sapwood allocation is age dependent in trees, but there is only aboveground
             !  allocation in grasses 
             alloc_sap_above = un

             ! Leaf, wood, root and fruit allocation
             f_alloc(ipts,j,ileaf) = Cl_inc(1)
             f_alloc(ipts,j,isapabove) = Cs_inc(1)*alloc_sap_above
             f_alloc(ipts,j,isapbelow) = Cs_inc(1)*(1.-alloc_sap_above)
             f_alloc(ipts,j,iroot) = Cr_inc(1)
             f_alloc(ipts,j,ifruit) = Cf_inc(1)

          ELSEIF (.NOT. is_tree(j)) THEN 
       
             IF(ld_alloc) WRITE(numout,*) 'there is no non-tree biomass to allocate, PFT, ', j
             f_alloc(ipts,j,ileaf) = zero
             f_alloc(ipts,j,isapabove) = zero
             f_alloc(ipts,j,isapbelow) = zero
             f_alloc(ipts,j,iroot) = zero
             f_alloc(ipts,j,ifruit) = zero
  
          ENDIF ! .NOT. is_tree(j) and there is biomass to allocate (§5.3 - far far up)

       ENDDO ! npts

       !! 5.4 Allocate allocatable biomass to different plant compartments
       !  The amount of allocatable biomass to each compartment is a fraction ::f_alloc of the total 
       !  allocatable biomass - see comment for 5.2.6
       DO k = 1, nparts

          bm_alloc(:,j,k,icarbon) = f_alloc(:,j,k) * (bm_alloc_tot(:,j) - residual(:,j))

       ENDDO

       !---TEMP---
       IF (j .EQ. test_pft .AND. ld_alloc) THEN
          DO ipts=1,npts
             IF(test_grid == ipts)THEN
                WRITE(numout,*) 'bm_alloc_tot(ipts,j), ', j, bm_alloc_tot(ipts,j)
                WRITE(numout,*) 'f_alloc(ipts,j,:), ', f_alloc(ipts,j,:)
                WRITE(numout,*) 'residual(ipts,j), ', residual(ipts,j)
             ENDIF
          END DO
       ENDIF
       !---------- 

    ENDDO ! # End Loop over # of PFTs   

    ! we need to zero the array for PFT 1, since it has not been calculated but it
    ! is used in implict loops below
    bm_alloc_tot(:,1) = zero
    bm_alloc(:,1,:,icarbon) = zero


 !! 6. Update the biomass with newly allocated biomass after respiration

    ! Update the biomass pools
    !---TEMP---
    IF (ld_alloc) THEN
       DO ipts=1,npts
          DO j=1,nvm
             IF(ipts == test_grid .AND. j == test_pft)THEN
                WRITE(numout,*) 'biomass, ', biomass(test_grid,test_pft,:,icarbon)
                WRITE(numout,*) 'bm_alloc, ', bm_alloc(test_grid,test_pft,:,icarbon)
             ENDIF
          ENDDO
       ENDDO
    ENDIF
    !---------- 
    ! I'm putting in a test here to see if we try to allocate
    ! a negative amount of biomass.  That's a bad thing if we do.
    DO ipts=1,npts
       DO j=1,nvm
          DO ipar=1,nparts
             IF(j == test_pft .AND. ipts == test_grid)THEN
                IF(bm_alloc(test_grid,test_pft,ipar,icarbon) .LT. zero)THEN
                   WRITE(numout,*) 'Trying to allocate negative biomass!'
                   WRITE(numout,*) 'ipts,j,ipar: ',ipts,j,ipar
                   WRITE(numout,*) 'bm_alloc(test_grid,test_pft,ipar,icarbon): ',&
                        bm_alloc(test_grid,test_pft,ipar,icarbon)
                   IF(ld_stop)THEN
                       CALL ipslerr_p (3,'growth_fun_all',&
                            'Trying to allocate negative biomass!','','')
                   ENDIF
                ENDIF
             END IF
          ENDDO
       ENDDO
    ENDDO

    biomass(:,:,:,icarbon) = biomass(:,:,:,icarbon) + bm_alloc(:,:,:,icarbon)
    

 !! 7. Use or fill reserve pools depending on relative size of the labile and reserve C pool

    ! +++ CHECK +++
    ! Externalize all the hard coded values i.e. 0.3

    ! Calculate the labile pool for all plants and also the reserve pool for trees      
    DO j = 2,nvm

       DO ipts = 1,npts


          IF (veget_max(ipts,j) .LE. min_stomate) THEN

             ! this vegetation type is not present, so no reason to do the 
             ! calculation. CYCLE will take us out of the innermost DO loop
             CYCLE

          ENDIF

          ! There is vegetation present and has started growing. The second and third condition
          ! required to make the PFT survive the first year during which the long term climate
          ! variables are initialized for the phenology. If these conditions are not added, the 
          ! reserves are respired well before growth ever starts
          IF ( veget_max(ipts,j) .GT. min_stomate .AND. &
               rue_longterm(ipts,j) .GE. zero .AND. &                  
               rue_longterm(ipts,j) .NE. un) THEN
             
             !! 7.1 Calculate the optimal size of the pools   
             !  The size of the labile pool is proportional to the assumed activity of living tissues 
             !  and its relative nitrogen content). The numerical value of ::lab_fac is already a
             !  tuning variable, the division by 10 (default for ::labile_reserve) stresses the importance 
             !  of this variable to scale other processes.
             !+++CHECK+++
             ! There is an inconsistency in the calculation - most pools are in gN but leaves is in gC
             ! The correction is proposed, that implies that the parameter labile_reserve will need to 
             ! be tuned
!!$          VERSION WITH CONSISTENT UNITS
!!$             labile_pool = lab_fac(ipts,j)/labile_reserve * &
!!$                  ( biomass(ipts,j,ileaf,icarbon) / cn_leaf_prescribed(j) + & 
!!$                  fcn_root(j) * ( biomass(ipts,j,iroot,icarbon) + biomass(ipts,j,ifruit,icarbon) ) + & 
!!$                  fcn_wood(j) * ( biomass(ipts,j,isapabove,icarbon) + biomass(ipts,j,isapbelow,icarbon) + &
!!$                  biomass(ipts,j,icarbres,icarbon) ) )
!!$          ORIGINAL VERSION WITH INCONSISTENT UNITS
!!$             labile_pool = lab_fac(ipts,j)/labile_reserve * ( biomass(ipts,j,ileaf,icarbon)  + & 
!!$                  fcn_root(j) * ( biomass(ipts,j,iroot,icarbon) + biomass(ipts,j,ifruit,icarbon) ) + & 
!!$                  fcn_wood(j) * ( biomass(ipts,j,isapabove,icarbon) + biomass(ipts,j,isapbelow,icarbon) + &
!!$                  biomass(ipts,j,icarbres,icarbon) ) )
!!$             labile_pool = MAX ( labile_pool, gpp_to_labile(j) * gpp_week(ipts,j) )
             
             ! We had an endless series of problems which were often difficult to
             ! understand but which always seemed to be related to a sudden drop
             ! in biomass(ilabile). This drop was often the result of a sudden 
             ! change in labile_pool. Given that there is not much science behind
             ! this approach it seems a good idea to remove this max statement to
             ! avoid sudden changes. Rather than using the actual biomass we propose
             ! to use the target biomass. This assumes that the tree would like to 
             ! fill its labile pool to be optimal when it would be in allometric
             ! balance.
             IF (is_tree(j)) THEN
             
                ! We will make use of the REAL sapwood, heartwood and effective height
                ! and then calculate the target leaves and roots. This approach gives
                ! us a target for a labile_pool of a tree in allometric balance. 
                ! Basal area at the tree level (m2 tree-1)
                circ_class_ba_eff(:) = wood_to_ba_eff(circ_class_biomass(ipts,j,:,:,icarbon),j)

                ! Current biomass pools per tree (gC tree^-1) 
                ! We will have different trees so this has to be calculated from the 
                ! diameter relationships            
                Cs(:) = ( circ_class_biomass(ipts,j,:,isapabove,icarbon) + &
                     circ_class_biomass(ipts,j,:,isapbelow,icarbon) ) * scal

                DO l = 1,ncirc 

                   !  Calculate tree height
                   circ_class_height_eff(l) = pipe_tune2(j)*(4/pi*circ_class_ba_eff(l))**(pipe_tune3(j)/2)
                   
                   ! Use the pipe model to calculate the target leaf and root
                   ! biomasses
                   Cl_target(l) = KF(ipts,j) * Cs(l) / circ_class_height_eff(l)
                   Cr_target(l) = Cl_target(l) / LF(ipts,j)
                   
                ENDDO

             ! grasses and crops
             ELSEIF ( .NOT. is_tree(j)) THEN
             
                Cs(:) = zero
                Cl_target(:) = zero 
                Cr_target(:) = zero
                ! Current biomass pools per grass/crop (gC ind^-1)
                ! Cs has too many dimensions for grass/crops. To have a consistent notation the same variables
                ! are used as for trees but the dimension of Cs, Cl and Cr i.e. ::ncirc should be ignored            
                Cs(1) = biomass(ipts,j,isapabove,icarbon) * scal
   
                ! Use the pipe model to calculate the target leaf and root
                ! biomasses
                Cl_target(1) = Cs(1) * KF(ipts,j)
                Cr_target(1) = Cl_target(1) / LF(ipts,j)

             ENDIF !is_tree

             ! Accounting for the N-concentration of the tissue as a proxy
             ! of tissue activity
             labile_pool = lab_fac(ipts,j)/labile_reserve(j) * &
                  ( SUM(Cl_target(:)) / cn_leaf_prescribed(j) + & 
                  SUM(Cr_target(:)) * fcn_root(j) + & 
                  SUM(Cs(:)) * fcn_wood(j) )
             !+++++++++++

             !+++TEMP+++ 
             IF (j .EQ. test_pft .AND. ld_alloc) THEN
                WRITE(numout,*) 'lab_fac, labile pool, ', lab_fac(ipts,j), labile_pool
             ENDIF
             !++++++++++

             ! The max size of reserve pool is proportional to the size of the storage organ (the sapwood)
             ! and a the leaf functional trait of the PFT (::phene_type_tab). The reserve pool is 
             ! constrained by the mass needed to replace foliage and roots. This constraint prevents the
             ! scheme from putting too much reserves in big trees (which have a lot of sapwood compared to
             ! small trees). Exessive storage would hamper tree growth and would make mortality less likely.
             IF(is_tree(j)) THEN

                IF (pheno_type(j).EQ.1) THEN 

                   ! Evergreen trees are not very conservative with respect to C-storage. Therefore, only 5% 
                   ! of their sapwood mass is stored in their reserve pool.
                   reserve_pool = MIN(evergreen_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                        biomass(ipts,j,isapbelow,icarbon)), lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j)))

                   IF (j .EQ. test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                      WRITE(numout,*) 'What happens to the reserve and labile pools? Evergreen'
                      WRITE(numout,*) 'carbres, reserve_pool: ',&
                           biomass(ipts,j,icarbres,icarbon),reserve_pool
                      WRITE(numout,*) 'ilabile, labile_pool: ',&
                           biomass(ipts,j,ilabile,icarbon),labile_pool
                      WRITE(numout,*) 'evergreen_reserve(j): ',&
                           evergreen_reserve(j)
                      WRITE(numout,*) 'isapabove,isapbelow: ',&
                           biomass(ipts,j,isapabove,icarbon), biomass(ipts,j,isapbelow,icarbon)
                      WRITE(numout,*) 'lai_target,sla,ltor: ',&
                           lai_target(ipts,j),sla(j),ltor(ipts,j)
                      WRITE(numout,*) 'term1, term2: ',&
                           evergreen_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                           biomass(ipts,j,isapbelow,icarbon)),&
                           lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j))
                   ENDIF
               

                ELSE

                   ! Deciduous trees are more conservative and 12% of their sapwood mass is stored in the 
                   ! reserve pool. The scheme avoids that during the growing season too much reserve are 
                   ! accumulated (which would hamper growth), therefore, the reduced rate of 12% is used 
                   ! until scenecence.
                   IF (bm_alloc_tot(ipts,j) .GT. min_stomate) THEN

                      reserve_pool = MIN(deciduous_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                           biomass(ipts,j,isapbelow,icarbon)), lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j)))

                      IF (j .EQ. test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                         WRITE(numout,*) 'What happens to the reserve and labile pools? Deciduous'
                         WRITE(numout,*) 'carbres, reserve_pool: ',&
                              biomass(ipts,j,icarbres,icarbon),reserve_pool
                         WRITE(numout,*) 'deciduous_reserve: ',&
                              deciduous_reserve(j)
                         WRITE(numout,*) 'isapabove,isapbelow: ',&
                              biomass(ipts,j,isapabove,icarbon), biomass(ipts,j,isapbelow,icarbon)
                         WRITE(numout,*) 'lai_target,sla,ltor: ',&
                              lai_target(ipts,j),sla(j),ltor(ipts,j)
                         WRITE(numout,*) 'term1, term2: ',&
                              deciduous_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                              biomass(ipts,j,isapbelow,icarbon)),&
                              lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j))
                      ENDIF

                  ELSE  

                      ! If the plant is scenecent, allow for a higher reserve mass. Plants can then use the 
                      ! excess labile C, that is no longer used for growth and would be respired otherwise,
                      ! to regrow leaves after the dormant period.
                      reserve_pool = MIN(senescense_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                           biomass(ipts,j,isapbelow,icarbon)), lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j)))

                      IF (j .EQ. test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                         WRITE(numout,*) 'What happens to the reserve and labile pools? Senescent'
                         WRITE(numout,*) 'carbres, reserve_pool: ',&
                              biomass(ipts,j,icarbres,icarbon),reserve_pool
                         WRITE(numout,*) 'senescense_reserve(j): ',&
                              senescense_reserve(j)
                         WRITE(numout,*) 'isapabove,isapbelow: ',&
                              biomass(ipts,j,isapabove,icarbon), biomass(ipts,j,isapbelow,icarbon)
                         WRITE(numout,*) 'lai_target,sla,ltor: ',&
                              lai_target(ipts,j),sla(j),ltor(ipts,j)
                         WRITE(numout,*) 'term1, term2: ',&
                              senescense_reserve(j) * ( biomass(ipts,j,isapabove,icarbon) + &
                              biomass(ipts,j,isapbelow,icarbon)),&
                              lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j))
                      ENDIF
                      
                   ENDIF ! Scenecent

                ENDIF ! Phenology type

             ! Grasses
             ELSE

                !+++CHECK+++
                ! The min criterion results in the reserves being zero because isapabove goes to zero 
                ! when the reserves are most needed 
                reserve_pool = MIN(0.3 * ( biomass(ipts,j,iroot,icarbon) + biomass(ipts,j,isapabove,icarbon) + & 
                     biomass(ipts,j,isapbelow,icarbon)), lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j)))
!!$                reserve_pool = lai_target(ipts,j)/sla(j)*(1.+0.3/ltor(ipts,j))
                !+++++++++++

                !+++TEMP+++ 
                IF (j .EQ. test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                   WRITE(numout,*) 'reserve pool, 30%', reserve_pool
                ENDIF
                !++++++++++

             ENDIF

             !! 7.2 Move carbon between the reserve pools
             !  Fill the reserve pools up to their optimal level or until the min/max limits are reached
             !  The original approcah in OCN resulted in instabilities and sometimes oscilations. For
             !  this reason a more simple and straightforward transfer between the pools has been 
             !  implemented.

             !! 7.2.1 Burn excess reserves
             !  The actual reserve and/or labile pool exceed the required pools (as calculated in 7.1)
             !  The excessive reserve pools respires C which needs to be accounted for in the 
             !  growth respiration. Because of this line of code, npp cannot be calculated at the
             !  start of the of this subroutine (i.e. between section 4 and 5). Last, correct the
             !  labile and reserve pool for this respiration flux. Note that instead of respiring
             !  this carbon it could be used for leaching, feeding mycorrhizae, producing DOCs,
             !  producing VOCs or any other component of NPP that does not end up in the biomass.
             IF ( (biomass(ipts,j,icarbres,icarbon) .GE. reserve_pool) .AND. &
                  (biomass(ipts,j,ilabile,icarbon) .GE. labile_pool) ) THEN
                          
                ! reserves are full
                IF ( biomass(ipts,j,icarbres,icarbon) .GT. reserve_pool ) THEN

                   excess = biomass(ipts,j,icarbres,icarbon) - reserve_pool

                   !---TEMP---
                   IF (j.EQ.test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                      WRITE(numout,*) 'excess reserve, ', &
                           excess, biomass(ipts,j,icarbres,icarbon), &
                           reserve_pool 
                   ENDIF
                   !----------

                   resp_growth(ipts,j) = resp_growth(ipts,j) + 0.1 * excess
                   biomass(ipts,j,icarbres,icarbon) = biomass(ipts,j,icarbres,icarbon) - &
                        0.1 * excess

                ENDIF

                ! labile is full
                ! Try not buring the labile pool.  I am commenting out this part because
                ! this can prevent trees from growing after coppicing.  It is unclear
                ! exactly what the point of it is, too.  In another loop, if the
                ! labile pool is overful but the carbres pool is just below the limit,
                ! the code is supposed to move labile carbon to the reserve pool.  It does
                ! it so slowly, though, that it might as well not be doing it at all.  Here
                ! we burn off the reserves while leaving the labile pool, since the labile
                ! pool is used for allocation.  This implicitly supposes that some of the
                ! carbon burned off from the reserve pool turns into labile carbon, enough
                ! to counter whatever is burned off from the labile pool.
!!$                IF ( biomass(ipts,j,ilabile,icarbon) .GT. labile_pool ) THEN
!!$
!!$                   excess = biomass(ipts,j,ilabile,icarbon) - labile_pool
!!$
!!$                   !---TEMP---
!!$                   IF (j.EQ.test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
!!$                      WRITE(numout,*) 'excess labile, ', &
!!$                           excess, biomass(ipts,j,ilabile,icarbon), &
!!$                           labile_pool 
!!$                   ENDIF
!!$                   !----------
!!$
!!$                   resp_growth(ipts,j) = resp_growth(ipts,j) + 0.1 * excess
!!$                   biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) - &
!!$                        0.1 * excess
!!$
!!$                ENDIF

                !---TEMP---
                IF (j.EQ.test_pft .AND. ld_alloc .AND. ipts == test_grid) THEN
                   WRITE(numout,*) 'reserve pools are considered too full'
                   WRITE(numout,*) 'excess, ', excess 
                   WRITE(numout,*) 'resp_growth, ', resp_growth(ipts,j)
                   WRITE(numout,*) 'biomass(ilabile) really final, ', biomass(ipts,j,ilabile,icarbon)
                   WRITE(numout,*) 'biomass(icarbres) really final, ', biomass(ipts,j,icarbres,icarbon)
                   WRITE(numout,*) 'when_growthinit, ', when_growthinit(ipts,j) , j
                   WRITE(numout,*) 'senescence, ',senescence(ipts,j), j
                ENDIF
                !----------        

             !! 7.2.2 Enough reserves, not enough labile
             ELSEIF ( (biomass(ipts,j,icarbres,icarbon) .GE. reserve_pool) .AND. &
                  (biomass(ipts,j,ilabile,icarbon) .LT. labile_pool) ) THEN

                ! We need to move carbon from the reserve pool to the labile pool. We will move
                ! this gradually. Calculate the maximum flow of the carbon reserve pool to labile 
                IF (is_tree(j)) THEN

                   ! During the dormant season for evergreens or the growing season for both functional leaf
                   ! traits. 
                   IF ( (biomass(ipts,j,ileaf,icarbon) .GT. min_stomate .AND. &
                        f_alloc(ipts,j,isapabove) .LE. min_stomate) .OR. & 
                        (lab_fac(ipts,j) .GT. 0.1) ) THEN 
                      
                      ! Don't move more than an arbitrary 5% from the carbohydrate reserve pool
!!$                         use_max = biomass(ipts,j,icarbres,icarbon) * MIN(0.05,reserve_scal)
                      use_max = biomass(ipts,j,icarbres,icarbon) * 0.05

                   ! Dormant season
                   ELSE 

                      ! Don't move any carbon between the reserve and the labile pool
                      use_max = zero

                   ENDIF ! Growing or dormant season

                ! Grasses
                ELSE 

                   ! During the growing season
                   IF (biomass(ipts,j,ileaf,icarbon).GT.min_stomate) THEN 

                      ! Don't move more than an arbitrary 5% from the carbohydrate reserve pool
!!$                         use_max = biomass(ipts,j,icarbres,icarbon) * MIN(0.05,reserve_scal)
                      use_max = biomass(ipts,j,icarbres,icarbon) * 0.05

                   ! Dormant season
                   ELSE 

                      ! Don't move any carbon between the reserve and the labile pool
                      use_max = zero

                   ENDIF ! Growing or dormant season

                ENDIF ! Trees or grasses

                ! Calculate the required flow of the carbon reserve pool to labile  
                ! Propose to use what can be moved from the reserve pool (::use_max) or 
                ! the amount required to fill the pool.
                use_res = MAX(zero, MIN(use_max, labile_pool-biomass(ipts,j,ilabile,icarbon)))
                   
                ! Update labile pool and reserve
                bm_alloc(ipts,j,icarbres,icarbon) =  use_res
                biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) + &
                     bm_alloc(ipts,j,icarbres,icarbon)
                biomass(ipts,j,icarbres,icarbon) = biomass(ipts,j,icarbres,icarbon) - &
                     bm_alloc(ipts,j,icarbres,icarbon)

                !+++TEMP+++ 
                IF (j .EQ. test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'move carbon from reserve to labile'
                   WRITE(numout,*) 'use_res, ', use_res
                   WRITE(numout,*) 'required: reserve and labile pool, ',reserve_pool, labile_pool
                   WRITE(numout,*) 'available: reserve and labile, ', biomass(ipts,j,icarbres,icarbon), &
                        biomass(ipts,j,ilabile,icarbon)
                ENDIF
                !++++++++++
                
             ! 7.2.3 Enough labile, not enough reserves
             ELSEIF ( (biomass(ipts,j,icarbres,icarbon) .LT. reserve_pool) .AND. &
                  (biomass(ipts,j,ilabile,icarbon) .GE. labile_pool) ) THEN
                
                ! The labile carbon is more mobile than the reserve pool but
                ! it is also more important in the allocation scheme because 
                ! it generates growth respiration and it is used to calculate
                ! bm_alloc. Therefore, the mobility of the labile pool was
                ! restricted to an arbitrary 15% 
                use_max = biomass(ipts,j,ilabile,icarbon) * 0.15
                
                ! Propose to use what can be moved from the reserve labile pool(::use_max) or 
                ! the amount required to fill the pool.
                use_lab = MAX(zero, MIN(use_max, reserve_pool-biomass(ipts,j,icarbres,icarbon)))
                
                ! Update labile pool and reserve
                bm_alloc(ipts,j,icarbres,icarbon) =  use_lab
                biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) - &
                     bm_alloc(ipts,j,icarbres,icarbon)
                biomass(ipts,j,icarbres,icarbon) = biomass(ipts,j,icarbres,icarbon) + &
                     bm_alloc(ipts,j,icarbres,icarbon)

                !+++TEMP+++ 
                IF (j .EQ. test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'move carbon from labile to reserve'
                   WRITE(numout,*) 'use_lab, ', use_lab
                   WRITE(numout,*) 'required: reserve and labile pool, ',reserve_pool, labile_pool
                   WRITE(numout,*) 'available: reserve and labile, ', biomass(ipts,j,icarbres,icarbon), &
                        biomass(ipts,j,ilabile,icarbon)
                ENDIF
                !++++++++++
             

             !! 7.2.3 We don't have enough carbon in both reserve pools. We will 
             !  redistribute what we have to minimise the tension between available and 
             !  required
             ELSEIF ( (biomass(ipts,j,icarbres,icarbon) .LT. reserve_pool) .AND. &
                  (biomass(ipts,j,ilabile,icarbon) .LT. labile_pool) ) THEN 
             
                !+++CHECK+++
                ! not really clear what the advantage is of doing this. Best case it avoids 
                ! that one of the pools gets depleted. Worst case it takes a lot of C (relatively)
                ! speaking from the labile pool (which is much smaller than the reserve pool).
                ! The carbon that is lacking to satisfy the needs of the reserve pools
!!$                shortage = (reserve_pool + labile_pool) - &
!!$                     (biomass(ipts,j,icarbres,icarbon) + biomass(ipts,j,ilabile,icarbon)) 
!!$                
!!$                ! Share of the reserve pool in the total requirement
!!$                reserve_scal = reserve_pool/(reserve_pool+labile_pool)
!!$
!!$                ! The shortage that should occur in the reserve pool
!!$                ! under the assumption that the optimal shortage is
!!$                ! proportional to the required reserve and labil pool
!!$                use_res = shortage * reserve_scal
!!$                
!!$                ! Update labile pool and reserve
!!$                bm_alloc(ipts,j,icarbres,icarbon) =  zero
!!$                biomass(ipts,j,ilabile,icarbon) = labile_pool - (shortage - use_res)
!!$                biomass(ipts,j,icarbres,icarbon) = reserve_pool - use_res
                !+++++++++++

                !++++++++++++++++
                !  Try moving carbon from reserves to labile, just because labile seems
                ! to be essential for growing leaves.  If we don't have enough, the
                ! trees stop growing but don't die.
                !shortage = labile_pool - biomass(ipts,j,ilabile,icarbon)
                !IF(shortage .GT. biomass(ipts,j,icarbres,icarbon))THEN
                !   biomass(ipts,j,ilabile,icarbon)=biomass(ipts,j,ilabile,icarbon)+0.9*shortage
                !   biomass(ipts,j,icarbres,icarbon)=biomass(ipts,j,icarbres,icarbon)-0.9*shortage
                !ENDIF
                !++++++++++++++++++++++++++

                !+++Temp+++ 
                IF (j .EQ. test_pft .AND. ld_alloc) THEN
                   WRITE(numout,*) 'not enough carbon to satify the total requirement of labile and reserve'
                   !WRITE(numout,*) 'use_res,shortage,reserve_scale: ', use_res,shortage,reserve_scal
                   WRITE(numout,*) 'bm_alloc: ',bm_alloc(ipts,j,icarbres,icarbon)
                   WRITE(numout,*) 'bm_alloc_tot: ',bm_alloc_tot(ipts,j)
                   WRITE(numout,'(A,2ES20.10)') 'required: reserve and labile pool, ',&
                        reserve_pool, labile_pool
                   WRITE(numout,'(A,2ES20.10)') 'available: reserve and labile, ', &
                        biomass(ipts,j,icarbres,icarbon), &
                           biomass(ipts,j,ilabile,icarbon)
                ENDIF
                !++++++++++

             !!  7.4 Unexpected condition
             ELSE
                
                IF(ld_warn .OR. ld_alloc) THEN
                   WRITE(numout,*) 'An unexpected condition occured for the reserve pools'
                   WRITE(numout,*) 'required: reserve and labile pool, ',reserve_pool, labile_pool
                   WRITE(numout,*) 'available: reserve and labile, ', biomass(ipts,j,icarbres,icarbon), &
                        biomass(ipts,j,ilabile,icarbon)
                   CALL ipslerr_p (3,'growth_fun_all',&
                        'An unexpected condition occured for the reserve pools','','')
                ENDIF

             ENDIF
             

!!$             !! 7.2 Calculate the C that could move from the reserve pool to the labile pool 
!!$             !  Fill the reserve pools up to their optimal level or until the min/max limits are reached
!!$   
!!$             !+++CHECK++++
!!$             ! SL does not understand the benefit of this implementation
!!$             ! Should equally well work without it
!!$             !  Calcualte the tension between the required and demanded reserve pool
!!$             IF (reserve_pool .GT. min_stomate) THEN
!!$
!!$                reserve_scal = MAX( MIN(biomass(ipts,j,icarbres,icarbon)/reserve_pool,un), zero) 
!!$
!!$             ELSE
!!$
!!$                reserve_scal = zero
!!$
!!$             ENDIF
!!$             !+++++++++++++
!!$
!!$     
!!$             !! 7.3 Calculate the C that is required to fill the labile pool
!!$             !  Propose to use what can be moved from the reserve pool (::use_max) or the amount required 
!!$             !  to fill the pool.
!!$             use_res = MAX(zero, MIN(use_max, labile_pool-biomass(ipts,j,ilabile,icarbon))) 
!!$
!!$             !+++TEMP+++ 
!!$             IF (j .EQ. test_pft .AND. ld_alloc) THEN
!!$                WRITE(numout,*) 'use_res, ', use_res
!!$             ENDIF
!!$             !++++++++++
!!$
!!$             ! Check wether there is excessive carbon in the reserve pool 
!!$             IF ( (biomass(ipts,j,icarbres,icarbon).GT.reserve_pool) .AND. (reserve_pool .GT. min_stomate) ) THEN
!!$                
!!$                ! Calculate the amount of carbon that will be moved from the reserve pool to the labile pool            
!!$                use_res = MIN(0.25 * biomass(ipts,j,icarbres,icarbon), & 
!!$                     MAX( biomass(ipts,j,icarbres,icarbon)-reserve_pool, use_res ))
!!$           
!!$             ENDIF
!!$
!!$             !! 7.4 Avoid overflow of the reserve pool
!!$             use_max = labile_pool
!!$             use_lab = MAX(zero, biomass(ipts,j,ilabile,icarbon)-use_max) * (un - reserve_scal)
!!$
!!$
!!$             !! 7.5 Calculate flow between the pools
!!$             !  Net carbon flow between reserve and labile pool adjust the reserve compartment in bm_alloc
!!$             bm_alloc(ipts,j,icarbres,icarbon) = MAX( MIN(use_lab-use_res, biomass(ipts,j,ilabile,icarbon)), &
!!$                  -biomass(ipts,j,icarbres,icarbon) )
!!$
!!$             !+++TEMP+++ 
!!$             IF (j .EQ. test_pft .AND. ld_alloc) THEN
!!$                WRITE(numout,*) 'bm_alloc(icarbes), ', bm_alloc(ipts,j,icarbres,icarbon)
!!$                WRITE(numout,*) 'use_lab, ', use_lab
!!$                WRITE(numout,*) 'use_res, ', use_res
!!$                WRITE(numout,*) 'biomass(ilabile), ', biomass(ipts,j,ilabile,icarbon)
!!$                WRITE(numout,*) 'biomass(icarbres), ', biomass(ipts,j,icarbres,icarbon)
!!$                WRITE(numout,*) 'biomass(ilabile) final, ', biomass(ipts,j,ilabile,icarbon) - &
!!$                     bm_alloc(ipts,j,icarbres,icarbon)
!!$                WRITE(numout,*) 'biomass(icarbres) final, ', biomass(ipts,j,icarbres,icarbon) + &
!!$                     bm_alloc(ipts,j,icarbres,icarbon)
!!$             ENDIF
!!$             !++++++++++
!!$
!!$             ! Update labile pool and reserve
!!$             biomass(ipts,j,ilabile,icarbon) = biomass(ipts,j,ilabile,icarbon) - bm_alloc(ipts,j,icarbres,icarbon)
!!$             biomass(ipts,j,icarbres,icarbon) = biomass(ipts,j,icarbres,icarbon) + bm_alloc(ipts,j,icarbres,icarbon)
               
          ELSEIF ( veget_max(ipts,j) .GT. min_stomate .AND. &
               rue_longterm(ipts,j) .EQ. un) THEN

             ! There hasn't been any photosynthesis yet. This happens when a new vegetation
             ! is prescribed and the longterm phenology variables are not initialized yet. 
             ! These conditions happen when the model is started from scratch (no restart files).
             ! Because the plants are very small, they contain little reserves. We increased the 
             ! amount of reserves by a factor ::tune_r_in_sapling where r stands for reserves.
             ! However, this amount gets simply respired before it is needed because the 
             ! reserve_pool is calculated as a function of the sapwood biomass which is very
             ! low because the plants are really small. Here we skip recalculating the 
             ! reserve_pool until the day we start using it.

          ELSE

             ! No reason to be here
             WRITE(numout,*) 'Error: unexpected condition for the reserve pools, pft, ',j
             WRITE(numout,*) 'veget_max, rue_longterm, ', veget_max(ipts,j), rue_longterm(ipts,j)

          ENDIF ! rue_longterm

          
          !! 7.8 Calculate NPP
          !  Calculate the NPP @tex $(gC.m^{-2}dt^{-1})$ @endtex as the difference between GPP and the two 
          !  components of autotrophic respiration (maintenance and growth respiration). GPP, R_maint and R_growth 
          !  are prognostic variables, NPP is calculated as the residuals and is thus a diagnostic variable. 
          !  Note that NPP is not used in the allocation scheme, instead bm_alloc_tot is allocated. The 
          !  physiological difference between both is that bm_alloc_tot does no longer contain the reserves and
          !  labile pools and is only the carbon that needs to go into the biomass pools. NPP contains the reserves
          !  and labile carbon. Note that GPP is in gC m-2 s-1 whereas the respiration terms were calculated in
          !  gC m-2 dt-1
          npp(ipts,j) = gpp_daily(ipts,j) - resp_growth(ipts,j)/dt - resp_maint(ipts,j)/dt
    
          !---TEMP---
          IF (j.EQ.test_pft .AND. ld_alloc) THEN
             WRITE(numout,'(A,20F20.10)') 'GPP_DAILY, NPP, Rag, Ra, ', gpp_daily(ipts,j), npp(ipts,j), &
                  resp_growth(ipts,j)/dt, resp_maint(ipts,j)/dt
             WRITE(numout,*) 'PFT, bm_alloc_tot, ', bm_alloc_tot(ipts,j)
             WRITE(numout,*) 'PFT, sum of bm_alloc components, ', SUM(bm_alloc(ipts,j,:,icarbon))
          ENDIF
          !----------


          !! 7.9 Distribute stand level ilabile and icarbres at the tree level
          !  The labile and carbres pools are calculated at the stand level but are then redistributed at the
          !  tree level. This has the advantage that biomass and circ_class_biomass have the same dimensions
          !  for nparts which comes in handy when phenology and mortality are calculated.
                
          IF (is_tree(j)) THEN
 
             ! Initialize to enable a loop over nparts
             circ_class_biomass(ipts,j,:,ilabile,:) = zero
             circ_class_biomass(ipts,j,:,icarbres,:) = zero
 
             ! Distribute labile and reserve pools over the circumference classes 
             DO m = 1,nelements

                ! Total biomass across parts and circumference classes
                temp_total_biomass = zero

                DO l = 1,ncirc 
                   
                   DO k = 1,nparts
                      
                      temp_total_biomass = temp_total_biomass + circ_class_biomass(ipts,j,l,k,m) * circ_class_n(ipts,j,l)
                                
                   ENDDO

                ENDDO
              
                ! Total biomass across parts but for a specific circumference class
                DO l = 1,ncirc

                   temp_class_biomass = zero
  
                   DO k = 1,nparts

                      temp_class_biomass = temp_class_biomass + circ_class_biomass(ipts,j,l,k,m) * circ_class_n(ipts,j,l)
               
                   ENDDO

                   IF(  temp_total_biomass .NE. zero) THEN

                      ! Share of this circumference class to the total biomass
                      temp_share = temp_class_biomass / temp_total_biomass
 
                      ! Allocation of ilabile at the tree level (gC tree-1)
                      circ_class_biomass(ipts,j,l,ilabile,m) = temp_share * &
                           biomass(ipts,j,ilabile,m) / circ_class_n(ipts,j,l)
                      
                      ! Allocation of icarbres at the tree level (gC tree-1)
                      circ_class_biomass(ipts,j,l,icarbres,m) = temp_share * &
                           biomass(ipts,j,icarbres,m) / circ_class_n(ipts,j,l)
                   ELSE

                      circ_class_biomass(ipts,j,l,ilabile,m) = zero
                      circ_class_biomass(ipts,j,l,icarbres,m) = zero

                   ENDIF

                ENDDO ! ncirc

             ENDDO  ! nelements

          ! Grasses and crops
          ELSE

             DO m = 1,nelements

                ! synchronize biomass and circ_class_biomass
                IF (ind(ipts,j) .GT. zero) THEN

                   circ_class_biomass(ipts,j,1,:,m) = biomass(ipts,j,:,m) / ind(ipts,j)

                ELSE

                   circ_class_biomass(ipts,j,1,:,m) = zero

                ENDIF

             ENDDO  
          
          ENDIF ! is_tree
  
       ENDDO ! pnts

    ENDDO ! PFTs


 !! 8. Check mass balance closure
    
    ! Calculate pools at the end of the routine
    pool_end = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    ! Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = gpp_daily(:,:) * dt * veget_max(:,:)
    check_intern(:,:,iland2atm,icarbon) = -un * (resp_maint(:,:) + resp_growth(:,:)) * &
         veget_max(:,:)
    check_intern(:,:,ilat2out,icarbon) = -un * zero
    check_intern(:,:,ilat2in,icarbon) = un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - &
         pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + &
            check_intern(:,:,imbc,icarbon)
    ENDDO

    ! Write conclusion
    DO ipts=1,npts
       DO j=1,nvm
          IF(ABS(closure_intern(ipts,j,icarbon)) .LE. min_stomate)THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in stomate_growth_fun_all.f90'
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in stomate_growth_fun_all.f90'
             WRITE(numout,*) '   ipts,j; ', ipts,j
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,j,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,j,icarbon), pool_start(ipts,j,icarbon)
             WRITE(numout,*) '   gpp_daily, veget_max: ', gpp_daily(ipts,j),veget_max(ipts,j)
             WRITE(numout,*) '   resp_maint,resp_growth: ', resp_maint(ipts,j),resp_growth(ipts,j)
             IF(ld_stop)THEN
                CALL ipslerr_p (3,'growth_fun_all', 'Mass balance error.','','')
             ENDIF
          ENDIF
       ENDDO
    ENDDO

    !+++HACK++++
    ! JR 080315 temporary hardwire for testing PFTs 4
    ! comment out this sections for tree growth profiles 
    IF (ld_fake_height) THEN
       !Do nothing
    ELSE 
       !Go to James' Hardwire 
       IF (control%ok_new_enerbil_nextstep ) THEN 
         ! this was a simple means to hardwire a canopy profile without having to use a spin-up file
         ! for testing. It is now commented out to avoid compilation errors for the INPUT variables
         ! ind and circ_class_n

         ! ind(1,4) = 0.6d0     
         ! biomass(1,4,1,1) = 10919.1492214105     
         ! biomass(1,4,2,1) = 119092.584535974     
         ! biomass(1,4,3,1) = 119092.584535974     
         ! biomass(1,4,4,1) = 23818.5169071949   
         ! biomass(1,4,5,1) = 23818.5169071949     
         ! biomass(1,4,6,1) = 3717.22198731141     
         ! biomass(1,4,7,1) = 0.000000000000000E+000
         ! biomass(1,4,8,1) = 0.000000000000000E+000
         ! biomass(1,4,9,1) = 348.380698397579     
         ! circ_class_n(1,4,1) = 0.570200000000000     
         ! circ_class_n(1,4,2) = 2.840000000000000E-002
         ! circ_class_n(1,4,3) = 1.400000000000000E-003

         ! circ_class_biomass(1,4,1,1,1) =  18198.5820356842     
         ! circ_class_biomass(1,4,1,2,1) =  198487.640893291     
         ! circ_class_biomass(1,4,1,3,1) =  198487.640893291     
         ! circ_class_biomass(1,4,1,4,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,1,5,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,1,6,1) =  6195.36997885235     
         ! circ_class_biomass(1,4,1,7,1) = 0.000000000000000E+000
         ! circ_class_biomass(1,4,1,8,1) =  0.000000000000000E+000
         ! circ_class_biomass(1,4,1,9,1) =  580.634497329298     
         ! circ_class_biomass(1,4,2,1,1) =  18198.5820356842     
         ! circ_class_biomass(1,4,2,2,1) =  198487.640893291     
         ! circ_class_biomass(1,4,2,3,1) =  198487.640893291     
         ! circ_class_biomass(1,4,2,4,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,2,5,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,2,6,1) =  6195.36997885235     
         ! circ_class_biomass(1,4,2,7,1) =  0.000000000000000E+000
         ! circ_class_biomass(1,4,2,8,1) =   0.000000000000000E+000
         ! circ_class_biomass(1,4,2,9,1) =  580.634497329298     
         ! circ_class_biomass(1,4,3,1,1) =  18198.5820356842     
         ! circ_class_biomass(1,4,3,2,1) =  198487.640893291     
         ! circ_class_biomass(1,4,3,3,1) =  198487.640893291     
         ! circ_class_biomass(1,4,3,4,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,3,5,1) =  39697.5281786581     
         ! circ_class_biomass(1,4,3,6,1) =  6195.36997885235     
         ! circ_class_biomass(1,4,3,7,1) =  0.000000000000000E+000
         ! circ_class_biomass(1,4,3,8,1) =  0.000000000000000E+000
         ! circ_class_biomass(1,4,3,9,1) =  580.634497329298     
       END IF ! (control%ok_new_enerbil_nextstep ) THEN 
    END IF !(ld_fake_height)
    !----------


 !! 9. Update leaf age

    !  Leaf age is needed to calculate the turnover and vmax in the stomate_turnover.f90 and stomate_vmax.f90 routines. 
    !  Leaf biomass is distributed according to its age into several "age classes" with age class=1 representing the
    !  youngest class, and consisting of the most newly allocated leaf biomass. 
    
    !! 9.1 Update quantity and age of the leaf biomass in the youngest class
    !  The new amount of leaf biomass in the youngest age class (leaf_mass_young) is the sum of :
    !  - the leaf biomass that was already in the youngest age class (leaf_frac(:,j,1) * lm_old(:,j)) with the 
    !  leaf age given in leaf_age(:,j,1) 
    !  - and the new biomass allocated to leaves (bm_alloc(:,j,ileaf,icarbon)) with a leaf age of zero.
    leaf_mass_young(:,:) = leaf_frac(:,:,1) * lm_old(:,:) + bm_alloc(:,:,ileaf,icarbon)

    ! The age of the updated youngest age class is the average of the ages of its 2 components: bm_alloc(leaf) of age
    ! '0', and leaf_frac*lm_old(=leaf_mass_young-bm_alloc) of age 'leaf_age(:,:,1)' 
    DO ipts=1,npts

       DO j=1,nvm

          ! IF(veget_max(ipts,j) == zero)THEN
          !     ! this vegetation type is not present, so no reason to do the 
          !     ! calculation
          !     CYCLE
          ! ENDIF

          IF( (bm_alloc(ipts,j,ileaf,icarbon) .GT. min_stomate ) .AND. &
               ( leaf_mass_young(ipts,j) .GT. min_stomate ) )THEN
             

             leaf_age(ipts,j,1) = MAX ( zero, leaf_age(ipts,j,1) * &
                  ( leaf_mass_young(ipts,j) - bm_alloc(ipts,j,ileaf,icarbon) ) / &
                  & leaf_mass_young(ipts,j) )
             
          ENDIF
          
          !+++TEMP+++
!!$          IF(j == test_pft .AND. ipts == test_grid)THEN
!!$             WRITE(numout,*) 'VCMAX: leaf_age growth: ',leaf_age(ipts,j,1),&
!!$                  bm_alloc(ipts,j,ileaf,icarbon),leaf_mass_young(ipts,j),&
!!$                  biomass(ipts,j,ileaf,icarbon)
!!$          ENDIF
          !++++++++++

       ENDDO

    ENDDO
          
    !! 8.2 Decrease reduction of photosynthesis 
    !  Decrease reduction of photosynthesis from new (undamaged) foliage
!!$    WHERE(biomass(:,:,ileaf,icarbon).GT.min_stomate)
!!$    
!!$      t_photo_stress(:,:) =  (t_photo_stress(:,:) * lm_old(:,:) + & 
!!$            bm_alloc(:,:,ileaf,icarbon))/biomass(:,:,ileaf,icarbon)
!!$   
!!$    ENDWHERE


    !! 9.3 Update leaf age
    !  Update fractions of leaf biomass in each age class (fraction in youngest class increases)

    !! 9.3.1 Update age of youngest leaves
    !  For age class 1 (youngest class), because we have added biomass to the youngest class, we need to update
    !  the fraction of total leaf biomass that belongs to the youngest age class : updated mass in class divided
    !  by new total leaf mass
    WHERE ( biomass(:,:,ileaf,icarbon) .GT. min_stomate )

          leaf_frac(:,:,1) = leaf_mass_young(:,:) / biomass(:,:,ileaf,icarbon)

    ENDWHERE


    !! 9.3.2 Update age of other age classes
    !  Because the total leaf biomass has changed, we need to update the fraction of leaves in each age class:
    !  mass in leaf age class (from previous fraction of leaves in this class and previous total leaf biomass) 
    !  divided by new total mass
    DO m = 2, nleafages ! Loop over # leaf age classes

       WHERE ( biomass(:,:,ileaf,icarbon) .GT. min_stomate )

          leaf_frac(:,:,m) = leaf_frac(:,:,m) * lm_old(:,:) / biomass(:,:,ileaf,icarbon)

       ENDWHERE

    ENDDO       ! Loop over # leaf age classes


 !! 10. Update whole-plant age 
    
    !! 10.1 PFT age
    !  At every time step, increase age of the biomass that was already present at previous time step. 
    !  Age is expressed in years, and the time step 'dt' in days so age increase is: dt divided by number 
    !  of days in a year.
    WHERE ( PFTpresent(:,:) )

       age(:,:) = age(:,:) + dt/one_year

    ELSEWHERE

       age(:,:) = zero

    ENDWHERE


    !! 10.2 Age of grasses and crops
    !  For grasses and crops, biomass with age 0 has been added to the whole plant with age 'age'. New biomass is the sum of 
    !  the current total biomass in all plant parts (bm_new), bm_new(:) = SUM( biomass(:,j,:), DIM=2 ). The biomass that has 
    !  just been added is the sum of the allocatable biomass of all plant parts (bm_add), its age is zero. bm_add(:) = 
    !  SUM( bm_alloc(:,j,:,icarbon), DIM=2 ). Before allocation, the plant biomass is bm_new-bm_add, its age is "age(:,j)". 
    !  The age of the new biomass is the average of the ages of previous and added biomass.
    !  For trees, age is treated in "establish" if vegetation is dynamic, and in turnover routines if it is static (in this 
    !  case, only the age of the heartwood is accounted for).
    DO j = 2,nvm

       IF ( .NOT. is_tree(j) ) THEN

          bm_new(:) = biomass(:,j,ileaf,icarbon) + biomass(:,j,isapabove,icarbon) + &
               biomass(:,j,iroot,icarbon) + biomass(:,j,ifruit,icarbon)
          bm_add(:) = bm_alloc(:,j,ileaf,icarbon) + bm_alloc(:,j,isapabove,icarbon) + &
               bm_alloc(:,j,iroot,icarbon) + bm_alloc(:,j,ifruit,icarbon)

          WHERE ( ( bm_new(:) .GT. min_stomate ) .AND. ( bm_add(:) .GT. min_stomate ) )
             
             age(:,j) = age(:,j) * ( bm_new(:) - bm_add(:) ) / bm_new(:)
          
          ENDWHERE

       ENDIF ! is .NOT. tree

    ENDDO  ! Loop over #PFTs 

!!$    ! +++HACK+++
!!$    !  10.3  This is only for the model validation 
!!$    !        LAI is imposed with "IMPOSE_LAI" (the maximun value within a year)     
!!$    !        
!!$    !        Reset the Biomass in leaf, labile and carbonate pools
!!$    !        In order to impose the LAI value & profile we need to recalculate the biomass
!!$    !        based on impose lai and default SLA(specific of leaf area index)
!!$    !        A monthy LAI scaling factor LAI_SCALE was also introduced for descrption the dynamic of LAI  
!!$    !        >>>  This will arise a mass balance issue in stomate_lpj routine 
!!$    !        >>>  Make sure you change the ERROR level of "check_biomass_sync" in funtion_library
!!$    !        >>>  to "2" to avoid model stop  
!!$    !        So, the model is suggested not to run over than one year when the flage turned on.
    istep=istep+1
    IF (ld_fake_height) THEN
       WRITE(numout,'(A,I6,I6)') '!=== CALL FUNTIONAL ALLOCATION STEP & RESET BIOMASS ====',test_pft, istep


       !GET impose LAI & LAI_SCALE
       CALL getin_p('IMPOSE_LAI',impose_lai)
            lai_fac = 1.0  
       CALL getin_p('LAI_FAC',lai_fac)
            impose_lai = impose_lai*lai_fac
         DO j=1, 13
            WRITE(temp_text,'(A11,I5.5)') 'LAI_SCALE__',j
            CALL getin_p(trim(temp_text),lai_scale(j))
            WRITE(numout,*) trim(temp_text), lai_scale(j)
         ENDDO

       !START a simple linear interpolation based on impose lai and lai scale 
       !Here, we use a simple 30 days for cycling of one month
       month_id = INT(istep/30.) + 1
       IF (month_id .GT. 12) THEN
          ! only for final 5 days set to a constant as impose_lai*lai_scale(13)
          daily_lai = (impose_lai)*lai_scale(month_id) 
       ELSE 
          daily_lai = ( (impose_lai)*lai_scale(month_id) + &
               ((lai_scale(month_id+1)-lai_scale(month_id))*impose_lai/30) * (MOD(istep,30)+1) )  
       ENDIF
       IF (daily_lai .LT. 0.) daily_lai=0.
       WRITE(numout,*) 'MONTH_ID:',month_id ,'MONTH_DAY:', (MOD(istep,30)+1)
       WRITE(numout,*) '!=== Daily LAI ====:', daily_lai
       WRITE(numout,*) 'BIOMASS_IN_LEAF:', daily_lai/sla(test_pft)
       !Covert the daily lai to biomass in leaf, labile ans carbres pools
       biomass(test_grid,test_pft,ileaf,icarbon)=   daily_lai/sla(test_pft)
       !For these two carbon pools we can simply set them as a coinstant value
       !labile and carbre pools are for sustaining the photothesis during bad weather conditions.  
       biomass(test_grid,test_pft,ilabile,icarbon)=   500.
       biomass(test_grid,test_pft,icarbres,icarbon)=  500.
       !Calculate the biomass in each circ_class again.  
       DO icirc=1,ncirc
          circ_class_biomass(test_grid,test_pft,icirc,:,icarbon)= &
               (biomass(test_grid,test_pft,:,icarbon)/float(ncirc))/circ_class_n(test_grid,test_pft,icirc)
          WRITE(numout,*) 'circ_class_n:',circ_class_n(test_grid,test_pft,icirc)
          WRITE(numout,*) 'leaf_biomass:',biomass(test_grid,test_pft,ileaf,icarbon)
          WRITE(numout,*) 'circ_biomass:',circ_class_biomass(test_grid,test_pft,icirc,ileaf,icarbon)
       ENDDO
    ENDIF  !ld_fake_height
!!$    !++++++++ 



 !! 11. Write history files

    !---TEMP---
!!$    DO ipts = 1, npts
!!$       height_out(ipts,1) = zero
!!$       DO j = 2, nvm
!!$          height_out(ipts,j) = SUM(circ_class_height_eff(:))/ncirc
!!$       ENDDO
!!$    ENDDO
    !---------

!!$    !+++++++++ TEMP ++++++++++
!!$    ! Just for testing.  Set the labile and reserve pools to zero to see if it dies.
!!$    istep=istep+1
!!$    IF(istep == 600)THEN
!!$       WRITE(numout,'(A,I6,I6)') '!********** KILLING PFT ',test_pft,istep
!!$       biomass(test_grid,test_pft,ileaf,icarbon)=zero
!!$       biomass(test_grid,test_pft,ilabile,icarbon)=zero
!!$       biomass(test_grid,test_pft,icarbres,icarbon)=zero
!!$       circ_class_biomass(test_grid,test_pft,:,ileaf,icarbon)=zero
!!$       circ_class_biomass(test_grid,test_pft,:,ilabile,icarbon)=zero
!!$       circ_class_biomass(test_grid,test_pft,:,icarbres,icarbon)=zero
!!$    ENDIF
!!$    !+++++++++++++++++++++++++

    ! Save in history file the variables describing the biomass allocated to the plant parts
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_LEAF', itime, &
         bm_alloc(:,:,ileaf,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_SAP_AB', itime, &
         bm_alloc(:,:,isapabove,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_SAP_BE', itime, &
         bm_alloc(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_ROOT', itime, &
         bm_alloc(:,:,iroot,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_FRUIT', itime, &
         bm_alloc(:,:,ifruit,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'BM_ALLOC_RES', itime, &
         bm_alloc(:,:,icarbres,icarbon), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'RUE_LONGTERM', itime, &
         rue_longterm(:,:), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'KF', itime, &
         k_latosa(:,:), npts*nvm, horipft_index)

    DO ipts = 1,npts
       DO j = 1,nvm
          IF(is_tree(j))THEN
             ! Calculate the forestry basal area (thus NOT the effective ba)
             circ_class_ba(:) = wood_to_ba(circ_class_biomass(ipts,j,:,:,icarbon),j)
             ba(ipts,j) = SUM(circ_class_ba(:)*circ_class_n(ipts,j,:)) * m2_to_ha
             wood_volume(ipts,j) = wood_to_volume(biomass(ipts,j,:,icarbon),j,&
                  branch_ratio(j),0)
             store_circ_class_ba(ipts,j,:) = circ_class_ba(:)
          ELSE
             ba(ipts,j) = val_exp
             wood_volume(ipts,j) = val_exp
             store_circ_class_ba(ipts,j,:) = val_exp 
          ENDIF
       ENDDO
    ENDDO

    CALL histwrite (hist_id_stomate, 'BA', itime, &
         ba(:,:), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'WOOD_VOL', itime, &
         wood_volume(:,:), npts*nvm, horipft_index)

    DO icirc = 1,ncirc
       WRITE(var_name,'(A,I3.3)') 'CCBA_',icirc
       CALL histwrite (hist_id_stomate, var_name, itime, &
            store_circ_class_ba(:,:,icirc), npts*nvm, horipft_index)
       WRITE(var_name,'(A,I3.3)') 'CCDELTABA_',icirc
       CALL histwrite (hist_id_stomate, VAR_NAME, itime, &
            store_delta_ba(:,:,icirc), npts*nvm, horipft_index)
    ENDDO

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving functional growth'

     
END SUBROUTINE growth_fun_all



!! ================================================================================================================================
!! FUNCTION	: func_derfunc
!!
!>\BRIEF        Calculate value for a function and its derivative
!!
!!
!! DESCRIPTION  : the routine describes the function and its derivative. Both function and derivative are used
!!              by the optimisation scheme. Hence, this function is part of the optimisation scheme and is only 
!!              called by the optimisation 
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): f, df
!!
!! REFERENCE(S)	: Numerical recipies in Fortran 77
!! 
!! FLOWCHART : 
!! \n
!_ ================================================================================================================================
 
 SUBROUTINE func_derfunc(x, n, o, p, q, r, t, eq_num, f, df)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    REAL(r_std), INTENT(in)                :: x           !! x value for which the function f(x) will be evaluated
    REAL(r_std), INTENT(in)                :: n,o,p,q,r,t !! Coefficients of the equation. Not all equations use all coefficients
    INTEGER(i_std), INTENT(in)             :: eq_num      !! Function i.e. f(x), g(x), ...

    !! 0.2 Output variables
    REAL(r_std), INTENT(out)               :: f           !! Value y for f(x)
    REAL(r_std), INTENT(out)               :: df          !! Value y for derivative[f(x)]   

    !! 0.3 Modified variables

    !! 0.4 Local variables
!_ ================================================================================================================================

!! 1. Calculate f(x) and df(x)

    IF (eq_num .EQ. 1) THEN

       !f = n*x**4 + o*x**3 + p*x**2 + q*x + r  
       !df = 4*n*x**3 + 3*o*x**2 + 2*p*x + q
    
    ELSEIF (eq_num .EQ. 2) THEN
    
       f = ( (n*x)/(p*((x+o)/t)**(q/(2+q))) ) - r
       df = ( n*(o*(q+2)+2*x)*((o+x)/t)**(-q/(q+2)) ) / ( p*(q+2)*(o+x) )
    
    ENDIF

 END SUBROUTINE func_derfunc


!! ================================================================================================================================
!! FUNCTION	: iterative_solver
!!
!>\BRIEF        find best fitting x for f(x)
!!
!!
!! DESCRIPTION  : The function makes use of an iterative approach to optimise the value for X. The solver
!!              splits the search region in two but there is an additional check to ensure that bounds are not
!!              exceeded.   
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): x
!!
!! REFERENCE(S)	: Numerical recipies in Fortran 77
!! 
!! FLOWCHART : 
!! \n
!_ ================================================================================================================================

  FUNCTION newX(n, o, p, q, r, t, x1, x2, eq_num, j, ipts)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    REAL(r_std), INTENT(in)        :: n,o,p,q,r,t      !! Coefficients of the equation. Not all 
                                                       !! equations use all coefficients
    REAL(r_std), INTENT(in)        :: x1               !! Lower boundary off search range
    REAL(r_std), INTENT(in)        :: x2               !! Upper boundary off search range
    INTEGER(i_std), INTENT(in)     :: eq_num           !! Function for which an iterative solution is 
                                                       !! searched
    INTEGER(i_std), INTENT(in)     :: j                !! Number of PFT
    INTEGER(i_std), INTENT(in)     :: ipts             !! Number of grdi square...for debugging
    
    !! 0.2 Output variables
   
    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std), PARAMETER      :: maxit = 20       !! Maximum number of iterations
    INTEGER(i_std), PARAMETER      :: max_attempt = 5  !! Maximum number of iterations
    INTEGER(i_std)                 :: i, attempt       !! Index
    REAL(r_std)                    :: newX             !! New estimate for X
    REAL(r_std)                    :: fl, fh, f        !! Value of the function for the lower bound (x1), 
                                                       !! upper bound (x2) and the new value (newX)
    REAL(r_std)                    :: xh, xl           !! Checked lower and upper bounds
    REAL(r_std)                    :: df               !! Value of the derivative of the function for newX
    REAL(r_std)                    :: dx, dxold        !! Slope of improvement
    REAL(r_std)                    :: temp             !! Dummy variable for value swaps
    REAL(r_std)                    :: low, high        !! temporary variables for x1 and x2 to avoid
                                                       !! intent in/out conflicts with Cs
    LOGICAL                        :: found_range      !! Flag indicating whether the range in which
                                                       !! a solution exists was identified.
    
    
!_ ================================================================================================================================   

!! 1. Find solution for X
 
    ! Not sure whether our initial range is large enough. We will
    ! start with a narrow range so we are more likely to fine the
    ! solution witin ::maxit iterations. If there is no solution
    ! in the initial range we will expande the range and try again

    ! Initilaze flags and counters
    attempt = 2
    found_range = .FALSE.
    low = x1
    high = x2
    

    ! Calculate y for the upper and lower bound
    DO WHILE (.NOT. found_range .AND. attempt .LT. max_attempt)
 
       CALL func_derfunc(low, n, o, p, q, r, t, eq_num, fl, df)
       CALL func_derfunc(high, n, o, p, q, r, t, eq_num, fh, df)
 
       IF ((fl .GT. 0.0 .AND. fh .GT. 0.0) .OR. &
            (fl .LT. 0.0 .AND. fh .LT. 0.0)) THEN
       
          IF (attempt .GT. max_attempt) THEN

             ! If the sign of y does not changes between the upper 
             ! and lower bound there no solution with the specified range
             WRITE(numout,*) 'Iterative procedure - tried really hard but' 
             WRITE(numout,*) 'no solution exists within the specified range'
             WRITE(numout,*) 'PFT, grid square: ',j,ipts
             CALL ipslerr_p (3,'growth_fun_all','newX',&
                  'Iterative procedure - tried really hard but failed','')

          ELSE
 
             ! Update counter
             attempt = attempt + 1
             
             ! Use previous upper boundary as the lower
             ! boundary for the next range search. Increase
             ! the upper boundary
             temp = high
             high = x1 * attempt
             low = temp
             
             ! Enlarge the search range
!!$             WRITE(numout,*) 'Iterative procedure - enlarge the search range'
!!$             WRITE(numout,*) 'New range: ', x1, x2
!!$             WRITE(numout,*) 'PFT, grid square, range: ',j,ipts,attempt

          ENDIF
       
       ELSE

          found_range = .TRUE.
          
       ENDIF
      
    ENDDO

    ! Only when we found a range we will search for the solution 
    IF (found_range) THEN
       
       ! If the sign of y changes between the upper and lower bound there is a solution
       IF ( ABS(fl) .LT. min_stomate ) THEN          

          ! The lower bound is the solution - most likely the lower bound is too high
          newX = x1
          RETURN

       ELSEIF ( ABS(fh) .LT. min_stomate ) THEN

          ! The upper bound is the solution - most likely the upper bound is too low
          newX = x2
          RETURN

       ELSEIF (fl .LT. 0.0) THEN
          
          ! Accept the lower and upper bounds as specified
          xl = x1
          xh = x2
       ELSE

          ! Lower and upper bounds were swapped, correct their ranking 
          xh = x1
          xl = x2
       ENDIF

       ! Estimate the initial newX value 	
       newX = 0.5 * (x1+x2)
       dxold = ABS(x2-x1)
       dx = dxold

    ENDIF
   
    ! Calculate y=f(x) and df(x) for initial guess of newX
    CALL func_derfunc(newX, n, o, p, q, r, t, eq_num, f, df)
    
    ! Evaluate for the maximum number of iterations  
    DO  i = 1,maxit

       IF ( ((newX-xh)*df-f)*((newX-xl)*df-f) .GT. 0.0 .OR. ABS(deux*f) > ABS(dxold*df) ) THEN
             
          ! Bisection
          dxold = dx
          dx = 0.5 * (xh-xl)
          newX = xl+dx
          IF (xl .EQ. newX) RETURN

       ELSE
             
          ! Newton
          dxold = dx
          dx = f/df
          temp = newX
          newX = newX-dx
          IF (temp .EQ. newX) RETURN

       ENDIF

       ! Precision reached
       IF ( ABS(dx) .LT. min_stomate) RETURN
          
       ! Precision was not reached calculate f(x) and df(x) for newX
       CALL func_derfunc(newX, n, o, p, q, r, t, eq_num, f, df)
          
       ! Narrow down the range
       IF (f .LT. 0.0) then
          xl = newX
       ELSE
          xh = newX
       ENDIF

    ENDDO ! maximum number of iterations
       
    !---TEMP---
    IF (j.EQ. test_pft) THEN 
       WRITE(numout,*) 'Iterative procedure: exceeded maximum iterations'
    ENDIF
    !----------

  END FUNCTION newX


!! ================================================================================================================================
!! SUBROUTINE	: comments
!!
!>\BRIEF        Contains all comments to check the code
!!
!!
!! DESCRIPTION  : contains all comments to check the code. By setting pft_test to 0, this routine is not called 
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): none
!!
!! REFERENCE(S)	: none
!! 
!! FLOWCHART : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE comment(npts, Cl_target, Cl, Cs_target, & 
       Cs, Cr_target, Cr, delta_ba, &
       ipts, j, l, b_inc_tot, & 
       Cl_incp, Cs_incp, Cr_incp, KF, LF, &
       Cl_inc, Cs_inc, Cr_inc, Cf_inc, &
       grow_wood, circ_class_n, ind, n_comment)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                         :: npts		                    !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:), INTENT(in)              :: Cl_target, Cs_target, Cr_target   !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:), INTENT(in)              :: Cl_incp, Cs_incp, Cr_incp         !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:), INTENT(in)              :: Cl_inc, Cs_inc, Cr_inc, Cf_inc    !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)          :: circ_class_n                      !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:), INTENT(in)              :: Cl, Cs, Cr                        !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:), INTENT(in)              :: delta_ba                          !! Defined in stomate_growth_fun_all
    REAL(r_std), DIMENSION(:,:), INTENT(in)            :: KF, LF, ind                       !! Defined in stomate_growth_fun_all
    REAL(r_std), INTENT(in)                            :: b_inc_tot                         !! Defined in stomate_growth_fun_all
    INTEGER(i_std), INTENT(in)                         :: ipts, j, l                        !! Defined in stomate_growth_fun_all
    LOGICAL, INTENT(in)                                :: grow_wood                         !! Defined in stomate_growth_fun_all

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                     :: n_comment                         !! Comment number  
    !_ ================================================================================================================================

    SELECT CASE (n_comment)
    CASE (1)
       ! Enough leaves and wood, grow roots
       WRITE(numout,*) 'Exc 1: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 1.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 1.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               (circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l)) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 1.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 24: Exc 1.4 unexpected result'
             WRITE(numout,*) 'WARNING 24: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE (2)
       ! Enough wood and roots, grow leaves 
       WRITE(numout,*) 'Exc 2: Cl_incp(<>0), Cs_incp (=0), Cr_incp (=0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 2.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 2.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 2.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 25: Exc 2.4 unexpected result'
             WRITE(numout,*) 'WARNING 25: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF


    CASE (3)

       ! Enough wood, grow leaves and roots
       WRITE(numout,*) 'Exc 3: Cl_incp(<>0), Cs_incp(=0), Cr_incp(<>0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), b_inc_tot - & 
            (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))), l
       IF (b_inc_tot - circ_class_n(ipts,j,l) * (Cl_incp(l) + Cs_incp(l) + Cr_incp(l))  &
            .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 3.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) )
       ELSE
          IF ( (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) &
               .GE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l)) ) .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l)) ) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 3.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l)) ) .GT. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l)) ) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 3.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 26: Exc 3.4 unexpected result'
             WRITE(numout,*) 'WARNING 26: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(4)
       ! Enough leaves and wood, grow roots
       WRITE(numout,*) 'Exc 4: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 4.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 4.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 4.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 27: Exc 4.4 unexpected result'
             WRITE(numout,*) 'WARNING 27: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(5)
       ! Enough leaves and roots, grow wood
       WRITE(numout,*) 'Exc 5: Cl_incp(=0), Cs_incp (<>0), Cr_incp (=0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 5.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 5.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 5.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 28: Exc 5.4 unexpected result'
             WRITE(numout,*) 'WARNING 28: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(6)
       ! Enough leaves, grow wood and roots
       WRITE(numout,*) 'Exc 6: Cl_incp(=0), Cs_incp(<>0), Cr_incp(<>0), unallocated'
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       IF (b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 6.1: unallocated less then 0: overspending, ', &
               b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 6.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .GT. min_stomate) .OR. &
               ((circ_class_n(ipts,j,l) * ABS(Cr_target(l)-Cr(l)-Cr_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) &
                  'Exc 6.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 29: Exc 6.4 unexpected result'
             WRITE(numout,*) 'WARNING 29: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(7)
       ! Enough leaves and wood, grow roots
       WRITE(numout,*) 'Exc 7: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 7.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 7.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 7.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 30: Exc 7.4 unexpected result'
             WRITE(numout,*) 'WARNING 30: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(8)
       ! Enough leaves and roots, grow wood
       WRITE(numout,*) 'Exc 8: Cl_incp(=0), Cs_incp (<>0), Cr_incp (=0), unallocated, class, '
       WRITE(numout,*) Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 8.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 8.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( circ_class_n(ipts,j,l)*(Cl_incp(l)+Cs_incp(l)+Cr_incp(l)) ) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 8.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 31: Exc 8.4 unexpected result'
             WRITE(numout,*) 'WARNING 31: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(9)
       ! Enough roots, grow leaves and wood
       WRITE(numout,*) 'Exc 9: delta_ba, Cl_incp(<>0), Cs_incp(<>0), Cr_incp(=0), unallocated, class, '
       WRITE(numout,*) delta_ba(:), Cl_incp(l), Cs_incp(l), Cr_incp(l), &
            b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))), l
       IF (b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 9.1: unallocated less then 0: overspending, ', &
               b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l)))
       ELSE
          IF ( (b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) .GE. zero) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 9.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - (circ_class_n(ipts,j,l) * (Cl_incp(l)+Cs_incp(l)+Cr_incp(l))) &
               .LE. min_stomate) .AND. &
               ((circ_class_n(ipts,j,l) * ABS(Cl_target(l)-Cl(l)-Cl_incp(l))) .GT. min_stomate) .OR. &
               ((circ_class_n(ipts,j,l) * ABS(Cs_target(l)-Cs(l)-Cs_incp(l))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 9.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 32: Exc 9.4 unexpected result'
             WRITE(numout,*) 'WARNING 32: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(10)
       ! Ready for ordinary allocation
       WRITE(numout,*) 'Ready for ordinary allocation?'
       WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
       WRITE(numout,*) 'b_inc_tot, ', b_inc_tot
       WRITE(numout,*) 'Cl, Cs, Cr', Cl(:), Cs(:), Cr(:)
       WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(:)-Cl(:)
       WRITE(numout,*) 'Cs_target-Cs, ', Cs_target(:)-Cs(:)
       WRITE(numout,*) 'Cr_target-Cr, ', Cr_target(:)-Cr(:)
       IF (b_inc_tot .GT. min_stomate) THEN
          IF (SUM(ABS(Cl_target(:)-Cl(:))) .LE. min_stomate) THEN
             IF (SUM(ABS(Cs_target(:)-Cs(:))) .LE. min_stomate) THEN
                IF (SUM(ABS(Cr_target(:)-Cr(:))) .LE. min_stomate) THEN
                   IF (grow_wood) THEN
                      WRITE(numout,*) 'should result in exc 10.1 or 10.2'
                   ELSE
                      WRITE(numout,*) 'No wood growth.  Not a problem!  Just an observation.'
                   ENDIF
                ELSE
                   WRITE(numout,*) 'WARNING 34: problem with Cr_target'
                   WRITE(numout,*) 'WARNING 34: PFT, ipts: ',j,ipts
                ENDIF
             ELSE
                WRITE(numout,*) 'WARNING 35: problem with Cs_target'
                WRITE(numout,*) 'WARNING 35: PFT, ipts: ',j,ipts
             ENDIF
          ELSE
             WRITE(numout,*) 'WARNING 36: problem with Cl_target'
             WRITE(numout,*) 'WARNING 36: PFT, ipts: ',j,ipts
          ENDIF
       ELSEIF(b_inc_tot .LT. -min_stomate) THEN 
          WRITE(numout,*) 'WARNING 37: problem with b_inc_tot'
          WRITE(numout,*) 'WARNING 37: PFT, ipts: ',j,ipts
       ELSE
          WRITE(numout,*) 'no unallocated fraction'
       ENDIF

    CASE(11)
       ! Ordinary allocation
       WRITE(numout,*) 'delta_ba, ', delta_ba
       IF ( (SUM(Cl_inc(:)) .GE. zero) .AND. (SUM(Cs_inc(:)) .GE. zero) .AND. &
            (SUM(Cr_inc(:)) .GE. zero) .AND. &
            ( b_inc_tot - SUM(circ_class_n(ipts,j,:) * (Cl_inc(:)+Cs_inc(:)+Cr_inc(:))) .GT. -1*min_stomate) .AND. &
            ( b_inc_tot - SUM(circ_class_n(ipts,j,:) * (Cl_inc(:)+Cs_inc(:)+Cr_inc(:))) .LT. min_stomate ) ) THEN
          WRITE(numout,*) 'Exc 10.1: Ordinary allocation was succesful'
          WRITE(numout,*) 'Cl_inc, Cs_inc, Cr_inc, unallocated', Cl_inc(:), Cs_inc(:), Cr_inc(:), & 
               b_inc_tot - SUM(circ_class_n(ipts,j,:) * (Cl_inc(:)+Cs_inc(:)+Cr_inc(:)))
       ELSE
          WRITE(numout,*) 'WARNING 38: Exc 10.2 problem with ordinary allocation'
          WRITE(numout,*) 'WARNING 38: PFT, ipts: ',j,ipts
          WRITE(numout,*) 'Cl_inc, Cs_inc, Cr_inc, unallocated', Cl_inc(:), Cs_inc(:), Cr_inc(:), & 
               b_inc_tot - SUM(circ_class_n(ipts,j,:) * (Cl_inc(:)+Cs_inc(:)+Cr_inc(:)))
       ENDIF

    CASE(12)
       ! Enough leaves and structure, grow roots
       WRITE(numout,*) 'Exc 1: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 1.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 1.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               (ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1)) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 1.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 39: Exc 1.4 unexpected result'
             WRITE(numout,*) 'WARNING 39: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(13)
       ! Enough structural C and roots, grow leaves
       WRITE(numout,*) 'Exc 2: Cl_incp(<>0), Cs_incp (=0), Cr_incp (=0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 2.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 2.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 2.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 40: Exc l.4 unexpected result'
             WRITE(numout,*) 'WARNING 40: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(14)
       ! Enough structural C and root, grow leaves
       WRITE(numout,*) 'Exc 3: Cl_incp(<>0), Cs_incp(=0), Cr_incp(<>0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), b_inc_tot - & 
            (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       IF (b_inc_tot - ind(ipts,j) * (Cl_incp(1) + Cs_incp(1) + Cr_incp(1))  &
            .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 3.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       ELSE
          IF ( (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1))) &
               .GE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1)) ) .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1)) ) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 3.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1)) ) .GT. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1)) ) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 3.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 41: Exc 3.4 unexpected result'
             WRITE(numout,*) 'WARNING 41: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(15)
       ! Enough leaves and structural C, grow roots
       WRITE(numout,*) 'Exc 4: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 4.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 4.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 4.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 42: Exc 4.4 unexpected result'
             WRITE(numout,*) 'WARNING 42: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(16)
       ! Enough leaves and roots, grow structural C            
       WRITE(numout,*) 'Exc 5: Cl_incp(=0), Cs_incp (<>0), Cr_incp (=0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 5.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 5.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 5.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 43: Exc 5.4 unexpected result'
             WRITE(numout,*) 'WARNING 43: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(17)
       ! Enough leaves, grow structural C and roots
       WRITE(numout,*) 'Exc 6: Cl_incp(=0), Cs_incp(<>0), Cr_incp(<>0), unallocated'
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       IF (b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1))) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 6.1: unallocated less then 0: overspending, ', &
               b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 6.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .GT. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cr_target(1)-Cr(1)-Cr_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) &
                  'Exc 6.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 44: Exc 6.4 unexpected result'
             WRITE(numout,*) 'WARNING 44: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(18)
       ! Enough leaves and structural C, grow roots
       WRITE(numout,*) 'Exc 7: Cl_incp(=0), Cs_incp (=0), Cr_incp (<>0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 7.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 7.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 7.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 45: Exc 7.4 unexpected result'
             WRITE(numout,*) 'WARNING 45: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(19)
       ! Enough leaves and roots, grow structural C
       WRITE(numout,*) 'Exc 8: Cl_incp(=0), Cs_incp (<>0), Cr_incp (=0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) )
       IF (b_inc_tot - (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 8.1: unallocated less then 0: overspending, ', b_inc_tot - &
               (ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 8.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - ( ind(ipts,j)*(Cl_incp(1)+Cs_incp(1)+Cr_incp(1)) ) &
               .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .GT. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 8.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 46: Exc 8.4 unexpected result'
             WRITE(numout,*) 'WARNING 46: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(20)
       ! Enough roots, grow structural C and leaves
       WRITE(numout,*) 'Exc 9: Cl_incp(<>0), Cs_incp(<>0), Cr_incp(=0), unallocated, '
       WRITE(numout,*) Cl_incp(1), Cs_incp(1), Cr_incp(1), &
            b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       WRITE(numout,*) 'term 1', b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       WRITE(numout,*) 'term 2', (ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1)))
       WRITE(numout,*) 'term 3', (ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1)))
       IF (b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1))) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Exc 9.1: unallocated less then 0: overspending, ', &
               b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1)))
       ELSE
          IF ( (b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1))) .GE. zero) .AND. &
               ((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .LE. min_stomate) .AND. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .LE. min_stomate) ) THEN
             WRITE(numout,*) 'Exc 9.2: unallocated <>= 0 but tree is in good shape: successful allocation'
          ELSEIF ( (b_inc_tot - (ind(ipts,j) * (Cl_incp(1)+Cs_incp(1)+Cr_incp(1))) .LE. min_stomate) .AND. &
               (((ind(ipts,j) * ABS(Cl_target(1)-Cl(1)-Cl_incp(1))) .GT. min_stomate) .OR. &
               ((ind(ipts,j) * ABS(Cs_target(1)-Cs(1)-Cs_incp(1))) .GT. min_stomate) ) ) THEN
             WRITE(numout,*) 'Exc 9.3: unallocated = 0, but the tree needs more reshaping: successful allocation'
          ELSE
             WRITE(numout,*) 'WARNING 47: Exc 9.4 unexpected result'
             WRITE(numout,*) 'WARNING 47: PFT, ipts: ',j,ipts
          ENDIF
       ENDIF

    CASE(21)
       ! Ready for ordinary allocation
       WRITE(numout,*) 'Ready for ordinary allocation?'
       WRITE(numout,*) 'KF, LF, ', KF(ipts,j), LF(ipts,j)
       WRITE(numout,*) 'b_inc_tot, ', b_inc_tot
       WRITE(numout,*) 'Cl, Cs, Cr', Cl(1), Cs(1), Cr(1)
       WRITE(numout,*) 'Cl_target-Cl, ', Cl_target(1)-Cl(1)
       WRITE(numout,*) 'Cs_target-Cs, ', Cs_target(1)-Cs(1)
       WRITE(numout,*) 'Cr_target-Cr, ', Cr_target(1)-Cr(1)
       IF (b_inc_tot .GT. min_stomate) THEN
          IF (ABS(Cl_target(1)-Cl(1)) .LE. min_stomate) THEN
             IF (ABS(Cs_target(1)-Cs(1)) .LE. min_stomate) THEN
                IF (ABS(Cr_target(1)-Cr(1)) .LE. min_stomate) THEN
                   IF (b_inc_tot .GT. min_stomate) THEN
                      IF (grow_wood) THEN
                         WRITE(numout,*) 'should result in exc 10.1 or 10.2'
                      ELSE
                         WRITE(numout,*) 'WARNING 48: no wood growth'
                         WRITE(numout,*) 'WARNING 48: PFT, ipts: ',j,ipts
                      ENDIF
                   ENDIF
                ELSE
                   WRITE(numout,*) 'WARNING 49: problem with Cr_target'
                   WRITE(numout,*) 'WARNING 49: PFT, ipts: ',j,ipts
                ENDIF
             ELSE
                WRITE(numout,*) 'WARNING 50: problem with Cs_target'
                WRITE(numout,*) 'WARNING 50: PFT, ipts: ',j,ipts
             ENDIF
          ELSE
             WRITE(numout,*) 'WARNING 51: problem with Cl_target'
             WRITE(numout,*) 'WARNING 51: PFT, ipts: ',j,ipts
          ENDIF
       ELSEIF(b_inc_tot .LT. -min_stomate) THEN 
          WRITE(numout,*) 'WARNING 52: problem with b_inc_tot'
          WRITE(numout,*) 'WARNING 52: PFT, ipts: ',j,ipts
       ELSE
          WRITE(numout,*) 'no unallocated fraction'
       ENDIF

    CASE(22)
       ! Ordinary allocation
       IF ( ((Cl_inc(1)) .GE. zero) .AND. ((Cs_inc(1)) .GE. zero) .AND. &
            ((Cr_inc(1)) .GE. zero) .AND. &
            ( b_inc_tot - (ind(ipts,j) * (Cl_inc(1)+Cs_inc(1)+Cr_inc(1))) .GT. -1*min_stomate) .AND. &
            ( b_inc_tot - (ind(ipts,j) * (Cl_inc(1)+Cs_inc(1)+Cr_inc(1))) .LT. min_stomate ) ) THEN
          WRITE(numout,*) 'Exc 10.1: Ordinary allocation was succesful'
          WRITE(numout,*) 'Cl_inc, Cs_inc, Cr_inc, unallocated', Cl_inc(1), Cs_inc(1), Cr_inc(1), & 
               b_inc_tot - (ind(ipts,j) * (Cl_inc(1)+Cs_inc(1)+Cr_inc(1)))
       ELSE
          WRITE(numout,*) 'WARNING 53: Exc 10.2 problem with ordinary allocation'
          WRITE(numout,*) 'WARNING 53: PFT, ipts: ',j,ipts
          WRITE(numout,*) 'Cl_inc, Cs_inc, Cr_inc, unallocated', Cl_inc(1), Cs_inc(1), Cr_inc(1), & 
               b_inc_tot - (ind(ipts,j) * (Cl_inc(1)+Cs_inc(1)+Cr_inc(1)))
       ENDIF

    END SELECT

  END SUBROUTINE comment

END MODULE stomate_growth_fun_all
